<?php
/*
 * iPod / iPhone / iPad video streaming
 * Included by video.php
 * Parameters:
 * 	array $uri: parsed video parameters
 *	string $completeFilename: video (decoded) source file name, access rights checked
 *

 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

define('M3U8NL',"\n");
$bandwidths=array('low'=>96,'medium'=>256,'high'=>384);


/**
 * Display a page redirecting to video, and that will self-close when returning from video, OR AN ERROR PAGE IF FFMPEG GPL NOT FOUND
 */
function vIPodRedirectPage(){
	global $uri;
	require_once(INCLUDE_DIR.'outputFunctions.php');
	require_once(INCLUDE_DIR.'mobileFunctions.php');


	/**
	 * GPL version not installed
	 */
	if(cfFfmpegVersion()=='LGPL'){
	// Redirection & self-closing page
	mfInsertHEAD(array('context'=>'view'));
?>
<meta name="viewport" content="width=device-width, initial-scale=1.0, user-scalable=no">
</head><body><center style="margin-top:50%;font-size:200%;padding:0 15% 0 15%" id="c">
<?php echo outImage(outIcon('alertBig')).'<br>'.cfCaption('ffmpegGPLIPhoneWarning'); ?><br>
</center>
</body>
<?php
		exit();
	}

	// SSL connection not compatible with iPhone / iPad.
	if (@$_SERVER['SERVER_PORT']==443) {
		require_once(INCLUDE_DIR.'mobileFunctions.php');
		outDisplayErrorPage(cfCaption('SSLIncompatible').'<br><br><br><div style="width:1px">'.mfListTable().mfListItem('onclick="W.close()"',outImage(outIcon('closeM')),false,'<div class="lit">&nbsp;'.cfCaption('genClose').'</div>').'</div>',array('icon'=>'alertBig','title'=>cfCaption('alertWarningTitle')));
	}

	// If iPhoneRedirectPage matches with last saved RVar, then we're comming back on the page => close it
	$closeWindow=$uri['iPhoneRedirectPage']===cfRGetVar('iPhoneRedirectPage');
	cfRSetVar('iPhoneRedirectPage',$uri['iPhoneRedirectPage']);

	// Redirection & self-closing page
	mfInsertHEAD(array('context'=>'view'));
?>
<meta name="viewport" content="width=device-width, initial-scale=1.0, user-scalable=no">
</head>
<body style="background:black;color:white;font-familly:Arial">
<center style="margin-top:50%;font-size:200%;padding:0 15% 0 15%" id="c">
<?php echo cfCaption('loading'); ?>
</center>
<a id="go" href="#shown">&nbsp;</a><a name="shown" href="javascript:void()">&nbsp;</a>
<script type="text/javascript">
if(D.location.href.indexOf('#')==-1){
	D.location.href="#shown";
	D.location="<?php echo str_replace('iPhoneRedirectPage','foo',$_SERVER['REQUEST_URI']);?>";
}
else window.close();
</script>
</body>
<?php
	exit();
}

/**
 * URL rewriting: convert back shortened URL (from m3u8 file) to standard video parameters encoded URL
 */
function vIPodRewriteURL(){
	global $completeFilename;
	global $uri;

	wSession_start_nowrite();

	// Retreive / set session Id
	$_SESSION['activeResourceId']=$_GET['resId'];

	// Restore stream parameters (filename & bitrate)
	if(!($completeFilename=cfRGetVar('videoIPhoneCompleteFilename'))) die();

	$uri['format']='ts';
	$uri['bandwidth']=cfRGetVar('videoIPhoneBandwidth');
	$uri['offset']=(int)@$_GET['offset'];
}

/**
 * @desc Send M3U8 Stream Alternates file with links to M3U8 files for available reencoding bandwidths
 * (not used as iPhone starts with the lowest bandwidth and would skip to upper one only at end of 1st segment... but as segments are not used, it wouldn't work)
 *
 */
function vIPodM3UStreamAlternates(){
	global $completeFilename;
	global $bandwidths;

	// m3u8 content-type header
	WHeaders::contentType('foo.m3u8');

	// Start output buffering to satisfy potential HTTP Range request
	cfRangeAccessibleOutputStart();

	echo "#EXTM3U".M3U8NL;

	// Lowest bitrate
	echo "#EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=".$bandwidths['low']."000".M3U8NL;
	echo str_replace('.m3u8sa','.m3u8/bandwidth.low',$_SERVER['REQUEST_URI']).M3U8NL;

	// Max output size
	$os=cfRGetVar('flashOutputSize'); if($os=='original') $os=9999;

	// Intermediate bitrate
	if($os>160 && cfRGetVar('flashOutputQuality')!='low'){
		echo "#EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=".$bandwidths['medium']."000".M3U8NL;
		echo str_replace('.m3u8sa','.m3u8/bandwidth.medium',$_SERVER['REQUEST_URI']).M3U8NL;

		// High bitrate
		if($os>320 && cfRGetVar('flashOutputQuality')!='medium'){
			echo "#EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=".$bandwidths['high']."000".M3U8NL;
			echo str_replace('.m3u8sa','.m3u8/bandwidth.high',$_SERVER['REQUEST_URI']).M3U8NL;
		}
	}

	// Send output
	cfRangeAccessibleOutputSend();
	exit;
}

/**
 * @desc Send M3U file splitting video into segments
 *
 */
function vIPodM3U(){
	global $completeFilename;
	global $uri;
	global $bandwidths;

	// Get kbps bandwidth from low/medium/high
	$bandwidth=@$bandwidths[$uri['bandwidth']]; if(!$bandwidth) $bandwidth=$bandwidths['low'];

	// Store stream parameters so they can be restored after URL rewritting
	cfRSetVar('videoIPhoneCompleteFilename',$completeFilename);
	cfRSetVar('videoIPhoneBandwidth',$bandwidth);

	// Start buffering output so it can be served for a range request
	cfRangeAccessibleOutputStart();

	// Get video duration
	require_once(INCLUDE_DIR.'viewVideoFunctions.php');
	$t=vvfVideoDuration($completeFilename);
	if(!$t) $t=3600*4;

	// Compute segments length
	$int=max(5,min(10,$t/100));
	$nb=ceil($t/$int);

	// m3u8 content-type header
	WHeaders::contentType('foo.m3u8');
	WHeaders::noCache();

	// Send M3U8 segments file
	echo "#EXTM3U".M3U8NL;
	echo "#EXT-X-MEDIA-SEQUENCE:0".M3U8NL;
	echo "#EXT-X-TARGETDURATION:$int".M3U8NL;

	// Generate segment links
	for($i=1;$i<$nb;$i++){
		echo "#EXTINF:$int,".M3U8NL;
		// Use shortened URL, to reduce overall size. URL will be converted back with apache rewriting and vIPodRewriteURL()
		echo /*cfHostName().*/"/ext/".$uri['resId'].'.'.($i*$int).".ts".M3U8NL;
	}
	echo '#EXT-X-ENDLIST';

	// Send buffer to output, or part of it for a range request
	cfRangeAccessibleOutputSend();
	exit;
}

/**
 * @desc Reencode video and sent to output
 *
 */
function vIPodVideo(){
	global $completeFilename;
	global $bandwidths;
	global $uri;

	require_once(INCLUDE_DIR.'viewVideoFunctions.php');

	// Get video dimensions
	list($sw,$sh)=vvfVideoDimensions($completeFilename);

	// Adapt dimension to iPhone viewport size & Compute audio & video bitrate
	if(cfBGetVar('name')=='iPhone') {
		list($w,$h)=cfThumbnailDimensions($sw,$sh,480,320);

		switch ($uri['bandwidth']){
			case 'low':
			case $bandwidths['low']:
				$vbr=96; $abr=48;
				$w/=2; $h/=2;
				$audioChannels=1;
				break;
			case 'medium':
			case $bandwidths['medium']:
				$vbr=128; $abr=48;
				$audioChannels=1;
				break;
			case 'high':
			case $bandwidths['high']:
				$vbr=512; $abr=64;
				$audioChannels=2;
				break;
		}
	}
	// Adapt dimension to iPad viewport size & Compute audio & video bitrate
	elseif(cfBGetVar('name')=='iPad') {
		list($w,$h)=cfThumbnailDimensions($sw,$sh,1024,768);
		switch ($uri['bandwidth']){
			case 'low':
			case $bandwidths['low']:
				$vbr=256; $abr=48;
				$w/=2; $h/=2;
				$audioChannels=1;
				break;
			case 'medium':
			case $bandwidths['medium']:
				$vbr=768; $abr=64;
				$audioChannels=1;
				break;
			case 'high':
			case $bandwidths['high']:
				$vbr=4096; $abr=96;
				$audioChannels=2;
				break;
		}
	}
	// Other devices (untested)
	else {
		list($w,$h)=cfThumbnailDimensions($sw,$sh,max(cfGGetVar('userScreenWidth'),cfGGetVar('userScreenHeight')),min(cfGGetVar('userScreenWidth'),cfGGetVar('userScreenHeight')));
		// TODO: Values below should be recomputed
		switch ($uri['bandwidth']){
			case 'low':
			case $bandwidths['low']:
				$vbr=256; $abr=48;
				$w/=2; $h/=2;
				$audioChannels=1;
				break;
			case 'medium':
			case $bandwidths['medium']:
				$vbr=768; $abr=64;
				$audioChannels=1;
				break;
			case 'high':
			case $bandwidths['high']:
				$vbr=4096; $abr=96;
				$audioChannels=2;
				break;
		}
	}

	// mkv are likely to contain 6 channels AC3, that can't be reencoded to single channel
	if($audioChannels==1 && cfFileExtension($completeFilename)=='mkv') $audioChannels=2;

	// Offset
	if(!isset($uri['offset'])) $uri['offset']=0;

	// Set Width & Height multiple of 16 (needed for optimal encoding)
	$w=16*round($w/16);
	$h=16*round($h/16);


	// Webcam: tweak filename to read from webcam
	if(cfCmpLeft($completeFilename,'webcam:')) {
		require_once(INCLUDE_DIR.'webcamFunctions.php');
		$completeFilename=webcamFfmpegCommandLineInput($completeFilename,$soundMinQuality,$soundMaxQuality);
		if($outputQuality=='original' || $outputQuality=='high') $sound=$soundMaxQuality; else $sound=$soundMinQuality;
		$preExtra=' -f dshow ';
	}
	else {
		$completeFilename='"'.$completeFilename.'"';
		$preExtra='';
	}


	// Use ffmpeg to reencode
	$cl='"'.cfFfmpegExe().'" -re -redelay 10 -ss '.((int)$uri['offset']).$preExtra.' -i '.$completeFilename;
	$cl.=' -acodec libmp3lame -async 1 -ab '.$abr.'k -ac '.$audioChannels;
	$cl.=' -f mpegts  -s '.$w.'x'.$h.' -vcodec libx264 -b '.$vbr.'k -threads 4';
	$cl.=' -flags +loop -partitions +parti4x4+partp8x8+partb8x8 -subq 5 -trellis 1 -refs 1 -coder 0 -me_range 16 -keyint_min 25 -sc_threshold 40 -i_qfactor 0.71';
	$cl.=' -bt 200k -maxrate '.$vbr.'k -bufsize '.$vbr.'k -rc_eq \'blurCplx^(1-qComp)\' -qcomp 0.6 -qmin 10 -qmax 51 -qdiff 4 -level 30';
	$cl.=' -aspect '.$w.':'.$h.' -g 30 -async 2 -';
	cfFfmpegLog($cl,'iPhone/iPad video encoding');

	header('Content-Type: video/MP2T');
	header('Accept-Ranges: byte');

	// If bytes range requested
	$h=getallheaders();
	if(isset($h['Range'])) {
		header('HTTP/1.1 206 Partial Content');
		//cfDebugSingle($uri['offset'].'=>'.$h['Range']);

		$h['Range']=substr($h['Range'],6);
		list($from,$to)=explode('-',$h['Range']);

		header("Content-Range: bytes $from-$to/9999999999");
		header("Content-Length: ".($to-$from));

		if($to==1) die('G'); // opt: if 1st byte only requested, don't bother with ffmpeg
		cfStreamProc($cl,array('stdout'=>'stdout'),array('range'=>array('from'=>$from, 'to'=>$to)));
	}
	// All content requested (no HTTP Range)
	else{
		header("Content-Length: 999999999");

		cfStreamProc($cl,array('stdout'=>'stdout'));
	}
	exit;
}



// URL rewriting: convert back shortened URL (from m3u8 file) to standard video parameters encoded URL
if(isset($_GET['resId']) && isset($_GET['offset'])) vIPodRewriteURL();

// Else, verify script has been included by video.php
if(!isset($completeFilename) || !isset($uri['format'])) die('Incorrect parameters');

/**
 * Display a page redirecting to video, and that will self-close when returning from video
 */
if(isset($uri['iPhoneRedirectPage'])) vIPodRedirectPage();

// Serve content depending on requested format
switch ($uri['format']){
	// Stream Alternates (Bitrate) m3u8 file
	case 'm3u8sa'; vIPodM3UStreamAlternates();

	// m3u8 file for given bitrate
	case 'm3u8'; vIPodM3U();

	// video
	case 'ts'; vIPodVideo();
}

