<?php
/**
 * Set of functions used by browser scripts (explorer, photo, music)
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */
define('W_EXPLORERFUNCTIONS_LOADED',1); // Include once bug WA
define('COMMENTS_FILE','comments.txt');
define('MAX_COMMENT_LENGHT','2048');
define('EDITABLE_FILES',';;txt;ini;xml;html;htm;cfg;css;res;php;php3;csv;js;bas;ctl;cls;frm;log;py;js;inc;c;h;cpp;bat;asp;mk;pas;rc;sql;dtd');
define('SLIDESHOW_TIMER_CONTROL_NAME','slideShowInterval');

// Main audio/video/image/text extensions, used for javascript detection of files that can be viewed
$_ENV['mainFilesExtensions']=array(
	'video'=>array('avi','mkv','mp4','mpg','mpeg','mov','qt','ogm','dv','flv','m4v','asf','wmv','ogv','webm'),
	'audio'=>array('mp3','mid','aif','aiff','wav','aac','flac','ogg','wma'),
	'image'=>array('jpg','jpeg','bmp','gif','png','svg','tiff','ico'),
	'text'=>array('txt','html','htm','css','sub','srt','nfo','log','c','h','cpp','php','ini','conf','doc','docx','ppt','pptx','xls','xlsx','pdf')
);

/**
 * @desc display error page
 *
 * @param integer $errorCode : HTTP error code
 */
function efDisplayError($errorCode){
	require_once(INCLUDE_DIR.'initFunctions.php');
	require_once(INCLUDE_DIR.'outputFunctions.php');

	switch ($errorCode){
		case 403:
			$message='403 Forbidden';
			break;
		case 404:
		default:
			$message='404 Not Found';
			break;
	}
	header('HTTP/1.1 '.$message);

	cfLog($message .' - '.$_SERVER["REQUEST_URI"],LOG_DBG);
	wSession_write_close();
	outDisplayErrorPage($message .' - '.$_SERVER["REQUEST_URI"]);
	exit();
}


/*
***************************************************************************************************************************
* File information functions
***************************************************************************************************************************
*/


/**
 * @desc return (internal, based on mime_type array) file type (left part of mime type) depending on $file's extension
 *
 * @param string $file : file name (with or without path)
 * @return string file type
 */
function efFileType($file){
	if(!defined('MT_LOADED')) require(INCLUDE_DIR.'mime_type.php');
	if((substr($file,1)==":/") || (strlen($file)==2 && substr($file,1,1)=="/")) return 'drive';
	if(is_dir($file)) return 'folder';
	if (($extension=strtolower(substr(strrchr($file,"."),1)))==false) return "unknown";
	if(!isset($_ENV['weezoMime'][$extension])) return 'unknown';
	return substr($_ENV['weezoMime'][$extension],0,strpos($_ENV['weezoMime'][$extension],"/"));
}

/**
 * @desc return (internal, based on mime_type array) full mime file type
 *
 * @param string $file : file name (with or without path)
 * @return string mime type
 */
function efFileMimeType($file){
	if(!defined('MT_LOADED')) require(INCLUDE_DIR.'mime_type.php');
	if((strlen($file)==3 && substr($file,1,2)==":/") || (strlen($file)==2 && substr($file,1,1)=="/")) return false;
	if(is_dir($file)) return false;
	$extension=strtolower(substr(strrchr($file,"."),1));
	if ($extension==false) return "text/plain";
	return mimeType($extension);
}

/**
 * @desc return icon file name according to file's extension
 *
 * @param string $file : file name (with or without path)
 * @param mixed $useExtracted: false to use weezo icons, true to use host system icons, 'exe' to extract icons only of exe/ico
 * @param int $iconSize: size of extracted icon (16,32)
 * @return string icon link
 */
function efIcon($file, $useExtracted=false,$iconSize=16){
	$ext=cfFileExtension($file);

	if($useExtracted!=false && ($useExtracted===true||$ext=='exe'||$ext=='ico')){
		static $extIcons=0;
		if(!$extIcons) $extIcons=cfMGetVar('extensionIcons');
		if ($ext=='exe'||$ext=='ico') return cfExtImage($file,$iconSize,$iconSize,true,0,array('icon'=>1));
		// If not existing, generate
		if(!isset($extIcons[$ext])){
			efExtractIcon($file,cfAppDocRoot().'/gfx/ext/'.$ext.'.png',$iconSize);
			$extIcons[$ext]=1;
			cfMSetVar('extensionIcons',$extIcons);
		}

		return '/gfx/ext/'.$ext.'.png';
	}
	switch($ext){
		case 'm3u': case 'pls': return outIcon('fi/icoM3u');
		case 'xls': case 'csv': case 'xlw': case 'xla': return outIcon('fi/icoXls');
		case 'html': case 'htm': case 'xml': return outIcon('fi/icoXml');
		case 'doc': case 'rtf': case 'dot': return outIcon('fi/icoDoc');
		case 'ppt': case 'pps': return outIcon('fi/icoPpt');
		case 'pdf': return outIcon('fi/icoPdf');
		case 'zip': case 'rar': case 'ace': case 'gzip': case 'tar': case 'tgz': return outIcon('fi/icoZip');
		case 'divx': case 'mkv': case 'wmv': return outIcon('fi/icoVid'); break;
		case 'nfo': case 'sub': case 'log': case 'ini': case 'srt': case 'sub': return outIcon('fi/icoText');
		case 'flv': case 'swf': return outIcon('fi/icoFlv');
		case 'torrent': return outIcon('fi/icoTorrent');
		case 'rss': case 'opml': return outIcon('fi/icoRss');
		case 'url': return outIcon('fi/icoUrl');
	}
	switch(efFileType($file)){
		case 'drive': return outIcon('fi/driveHDD');
		case 'folder': return outIcon('fi/folder');
		case 'image': return outIcon('fi/icoImage');
		case 'audio': return outIcon('fi/icoMus');
		case 'text':return outIcon('fi/icoText');
		case 'video':return outIcon('fi/icoVid');
	}
	if(!$ext) return outIcon('fi/icoBlank');
	if(efFileType($file)=='application') return outIcon('fi/icoProg');
	return outIcon('fi/icoU');
}

/**
 * Extract icon associated to a file to png format
 *
 * @param string $file
 * @param string $dest: destination png file, false to return
 * @param int $size: icon width & height
 * @return png icon if $dest is false
 */
function efExtractIcon($file,$dest=false,$size=16,$options=false){
	$tmpFile=cfAppTempDir().'/tmp_ext'.$_ENV['requestID'].'.png';
	if($dest==false) $dest=$tmpFile;

	// Dirty workaround for png files embeded into .ico
	if(isset($options['icoPngCheck']) && cfFileExtension($file)=='ico'){
		$fp=@fopen($file,'rb');$l=@fread($fp,4);@fclose($fp);
		if($l=='PNG'){
			if($dest==$tmpFile) return file_get_contents($file);
			file_put_contents($dest,file_get_contents($file));
			return true;
		}
	}
	if(function_exists('customExtractIcon'))
		$res=customExtractIcon(str_replace('/','\\',$file),str_replace('/','\\',$dest),$size);
	else
		return false;
	if($dest!=$tmpFile) return true;
	$result=@file_get_contents($dest);
	@unlink($dest);

	return $result;
}

/**
 * @desc Get Drive information
 *
 * @param string $driveName: drive letter
 * @return array (property=>value)
 */
function efDriveInfo($driveName){
	if(strlen($driveName)==2) $driveName.='/';
	$driveName=strtoupper($driveName);
	$d=array();
	$d['name']=$driveName;
	$d['completeFileName']=$driveName;
	$d['freeSpace']=@disk_free_space($driveName);
	$d['totalSpace']=@disk_total_space($driveName);
	$d['type']='drive';

	$d['subType']=customDriveType(substr($driveName,0,1));
	switch ($d['subType']){
		case 'Fixed':
			$d['volumeType']=cfUTF8Decode(cfCaption('explorerDriveFixed'));
			$d['icon']=outIcon('fi/driveHDD');;
			break;
		case 'CDROM':
			$d['volumeType']=cfUTF8Decode(cfCaption('explorerDriveCDRom'));
			$d['icon']=outIcon('fi/driveCdrom');;
			break;
		case 'RamDisk':
			$d['volumeType']=cfUTF8Decode(cfCaption('explorerDriveRamDisk'));
			$d['icon']=outIcon('fi/driveRam');;
			break;
		case 'Remote':
			$d['volumeType']=cfUTF8Decode(cfCaption('explorerDriveRemote'));
			$d['icon']=outIcon('fi/driveNetwork');;
			break;
		case 'Removable':
			if($driveName=='A:/' || $driveName=='B:/') {
				$d['volumeType']=cfUTF8Decode(cfCaption('explorerDriveFloppy'));
				$d['icon']=outIcon('fi/driveFloppy');;
			}
			else{
				$d['volumeType']=cfUTF8Decode(cfCaption('explorerDriveRemovable'));
				$d['icon']=outIcon('fi/driveRemovable');;
			}
			break;
		case 'Unknown':
			$d['volumeType']=cfUTF8Decode(cfCaption('explorerDriveUnknown'));
			$d['icon']=outIcon('fi/driveHDD');;
			break;
	}
	$d['fileSystem']=customDriveFileSystem(substr($driveName,0,1));
	$d['volumeName']=customDriveName(substr($driveName,0,1));
	$d['driveReady']=customDriveReady(substr($driveName,0,1));
	return $d;
}

/**
 * @desc Return audio file information based on id3 tag
 *
 * @param string $fileName: name (without path) of file to add (note : path is efCurrentDirectory)
 * @param boolean $returnCaptionForEmptyFields: true to remplace empty fields by localized caption
 * @param boolean $extendedAnalysis: true to merge all ID3 fields (120% slower, better results)
 * @param array $options:
 * 			"vbrParsing": does a full analysis of mp3 to get exact track duration (slower)
 * 			"noCache": doesn't use / fill cache database
 *
 * @return array :
 * 				bitrate
 * 				bitrateMode
 * 				channelMode
 * 				playtime_seconds
 * 				title
 * 				artist
 * 				album
 * 				year
 * 				genre
 * 				track
 * 				comment
 * 				label : label generated from artist, album, track, title and filename
 */
function efGetAudioInfo($completeFilename,$returnCaptionForEmptyFields=true,$options=array()){
	static $id3V2Tags=false; // Object used to parse mp3 files. Static is used not to recreate object each time.
	static $genre=array(0=>'Blues',1=>'Classic Rock',2=>'Country',3=>'Dance',4=>'Disco',5=>'Funk',6=>'Grunge',7=>'Hip-Hop',8=>'Jazz',9=>'Metal',10=>'New Age',11=>'Oldies',12=>'Other',13=>'Pop',14=>'R&B',15=>'Rap',16=>'Reggae',17=>'Rock',18=>'Techno',19=>'Industrial',20=>'Alternative',21=>'Ska',22=>'Death Metal',23=>'Pranks',24=>'Soundtrack',25=>'Euro-Techno',26=>'Ambient',27=>'Trip-Hop',28=>'Vocal',29=>'Jazz+Funk',30=>'Fusion',31=>'Trance',32=>'Classical',33=>'Instrumental',34=>'Acid',35=>'House',36=>'Game',37=>'Sound Clip',38=>'Gospel',39=>'Noise',40=>'AlternRock',41=>'Bass',42=>'Soul',43=>'Punk',44=>'Space',45=>'Meditative',46=>'Instrumental Pop',47=>'Instrumental Rock',48=>'Ethnic',49=>'Gothic',50=>'Darkwave',51=>'Techno-Industrial',52=>'Electronic',53=>'Pop-Folk',54=>'Eurodance',55=>'Dream',56=>'Southern Rock',57=>'Comedy',58=>'Cult',59=>'Gangsta',60=>'Top 40',61=>'Christian Rap',62=>'Pop/Funk',63=>'Jungle',64=>'Native American',65=>'Cabaret',66=>'New Wave',67=>'Psychadelic',68=>'Rave',69=>'Showtunes',70=>'Trailer',71=>'Lo-Fi',72=>'Tribal',73=>'Acid Punk',74=>'Acid Jazz',75=>'Polka',76=>'Retro',77=>'Musical',78=>'Rock & Roll',79=>'Hard Rock',80=>'Folk',81=>'Folk-Rock',82=>'National Folk',83=>'Swing',84=>'Fast Fusion',85=>'Bebob',86=>'Latin',87=>'Revival',88=>'Celtic',89=>'Bluegrass',90=>'Avantgarde',91=>'Gothic Rock',92=>'Progressive Rock',93=>'Psychedelic Rock',94=>'Symphonic Rock',95=>'Slow Rock',96=>'Big Band',97=>'Chorus',98=>'Easy Listening',99=>'Acoustic',100=>'Humour',101=>'Speech',102=>'Chanson',103=>'Opera',104=>'Chamber Music',105=>'Sonata',106=>'Symphony',107=>'Booty Bass',108=>'Primus',109=>'Porn Groove',110=>'Satire',111=>'Slow Jam',112=>'Club',113=>'Tango',114=>'Samba',115=>'Folklore',116=>'Ballad',117=>'Power Ballad',118=>'Rhythmic Soul',119=>'Freestyle',120=>'Duet',121=>'Punk Rock',122=>'Drum Solo',123=>'A capella');
	static $iso14496Conv=array('day'=>'year','alb'=>'album','nam'=>'title','art'=>'artist','gen'=>'genre','cmt'=>'comment','wrt'=>'composer','too'=>'encodedBy','cprt'=>'copyrightInfo');
	static $cacheData=-1;

	if($options) $options=cfArrayValuesToKeys($options);

	// Seek in cache database for information
	if($cacheData==-1) $cacheData=cfHGetVar('cacheAudioData');
	if($cacheData && !isset($options['noCache'])){
		if(!isset($_ENV['efGetAudioInfoCacheDB'])){
			if(!defined('W_DATABASE_ENGINE')) require_once(INCLUDE_DIR.'databaseFunctions.php');

			$_ENV['efGetAudioInfoCacheDB']=dbOpenDB(cfAppDataDir().'/audioCacheV5.db');

			// Create an array of pending INSERT requests, that will be commited on request shutdown
			$_ENV['efGetAudioInfoCache']=array();

			// Register function that will commit insert and delete queries on request completition
			register_shutdown_function('efGetAudioInfoCommitCache');

			// Create db if needed
			if(!dbTableExists('tracks',$_ENV['efGetAudioInfoCacheDB']))
				dbExec('CREATE TABLE tracks ('.
				'completeFilename TEXT PRIMARY KEY, filemtime INTEGER, label TEXT, title TEXT, artist TEXT, '.
				'album TEXT, track TEXT, year TEXT, genre TEXT, comment TEXT, length INTEGER, vbrParsing INTEGER, '.
				'APIC INTEGER, lyrics INTEGER)',$_ENV['efGetAudioInfoCacheDB']);
		}


		// Search for file in db
		@list($id3)=dbArrayQuery('SELECT * from tracks WHERE completeFilename="'.$completeFilename.'" LIMIT 1',$_ENV['efGetAudioInfoCacheDB']);
		if($id3){
			// outdated info or not generated with requested vbr parsing
			if($id3['filemtime']!=filemtime($completeFilename) || (isset($options['vbrParsing'])&&!$id3['vbrParsing']))
				$_ENV['efGetAudioInfoCache'][]='DELETE FROM tracks WHERE completeFilename="'.$completeFilename.'";';
			// Found: return cache
			else {
				if($returnCaptionForEmptyFields){
					if(!$id3['artist']) $id3['artist']=cfCaption('explorerAudioUnknownArtist');
					if(!$id3['album']) $id3['album']=cfCaption('explorerAudioUnknownAlbum');
				}
				return $id3;
			}
		}
	}

	$id3=array('artist'=>false,'album'=>false,'track'=>false,'year'=>false,'genre'=>false,'length'=>0,'comment'=>false);

	switch (cfFileExtension($completeFilename)){
		// WMA/WMV/ASF
		case 'asf':
		case 'wma':
		case 'wmv':
			require_once('ASF.php');
			$asf=new ASF($completeFilename,array('base'=>'header'));

			if(isset($asf->header->contentDescription)) $id3['artist']=$asf->header->contentDescription->author;
			if(isset($asf->header->contentDescription)) $id3['title']=$asf->header->contentDescription->title;
			if(isset($asf->header->extendedContentDescription)) {
				$id3['length']=$asf->header->fileProperties->playDuration/10/1000;
				// Browse tags
				foreach ($asf->header->extendedContentDescription->descriptors as $key => $value) {
					// Convert to ID3 format
					if($key!='ID3' && $key!='WM/Picture') $value=cfUTF8Decode($value);
					if($key=='WM/TrackNumber') $id3['track']=$value;
					elseif($key=='WM/Track' && !$id3['track']) $id3['track']=$value;
					elseif ($value){
						if($key=='Title') $id3['title']=$value;
						elseif($key=='ID3'){}
						elseif ($key=='WM/Picture') $id3['APIC']='TRUE';
						elseif ($key=='WM/AlbumTitle') $id3['album']=$value;
						elseif ($key=='WM/AlbumArtist') $id3['band']=$value;
						elseif(substr($key,0,3)=='WM/') $id3[strtolower(substr($key,3,1)).substr($key,4)]=$value;
						else $id3[strtolower(substr($key,0,1)).substr($key,1)]=$value;
					}
				}
			}
			unset($asf);
			break;

		// Misc mp4? formats
		case '3gp': case '3gpp': case 'avc': case 'dcf': case 'm21': case 'm4a':
		case 'm4b': case 'm4p': case 'm4v': case 'maf': case 'mj2': case 'mjp':
		case 'mov': case 'mp4': case 'odf': case 'sdv': case 'qt': case 'aac':case 'aif':
			if(!class_exists('ISO14496')) require(INCLUDE_DIR.'/php-reader/ISO14496.php'); // require_once not working in VC9 => WA

			$isom=new ISO14496($completeFilename,array("base" => "moov.mvhd"));
			if(isset($isom->moov->mvhd)) $id3['length']=floor(1000*($isom->moov->mvhd->duration/$isom->moov->mvhd->timescale));
			$isom=new ISO14496($completeFilename,array("base" => "moov.udta.meta.ilst"));
			if (isset($isom->moov->udta->meta->ilst)){
				foreach ($isom->moov->udta->meta->ilst->boxes as $name => $boxes)
				foreach ($boxes as $box){
					$name=strtolower((($name[0]=='')?substr($name,1):$name));
					if(isset($iso14496Conv[$name])) $id3[$iso14496Conv[$name]]=cfUTF8Decode($box->data->value,true,false,false);
					// Track number
					elseif ($name=='trkn') {
						$id3['track']=substr($box->data->value,3);
						$pos=0;while ($pos+1<strlen($id3['track']) && !(substr($id3['track'],$pos,1)=="0" && substr($id3['track'],$pos+1,1)!="0")) $pos++;
						$id3['track']=(int)substr($id3['track'],0,$pos);
					}
					elseif ($name=='covr') $id3['APIC']='TRUE';
				}
			}
			unset($isom);
			break;

		// OGG vorbis
		case 'ogg':
			if(!defined('MF_LOADED')) require(INCLUDE_DIR.'miscFunctions.php');
			// Get FLAC metadata
			$oggMetaData=new OGGMetaData($completeFilename);

			// If OK, use them
			if(count($oggMetaData->tag))	$id3=$oggMetaData->tag+$id3;

			/*
			if($fp=@fopen('ogg://'.$completeFilename,'r')){
			if(!defined('MF_LOADED')) require(INCLUDE_DIR.'miscFunctions.php');

			$metadata = stream_get_meta_data($fp);
			fclose($fp);
			if(isset($metadata['wrapper_data'][0])){
			$metadata=$metadata['wrapper_data'][0];
			if(isset($metadata['bitrate_nominal']))
			$id3['length']=8000000*filesize($completeFilename)/$metadata['bitrate_nominal'];
			if(isset($metadata['comments'])) foreach ($metadata['comments'] as $item){
			@list($tag,$value)=explode('=',$item,2);
			if($value){
			$tag=strtolower($tag);

			if($tag=='date') $id3['year']=$value;
			elseif($tag=='tracknumber') $id3['track']=$value;
			elseif(in_array($tag,id3V2Ext::$frames)) $id3[$tag]=cfUTF8Decode($value);
			}
			}
			}
			}
			*/
			unset($oggMetaData);
			break;
			// FLAC
		case 'flac':
			if(!defined('MF_LOADED')) require(INCLUDE_DIR.'miscFunctions.php');
			// Get FLAC metadata
			$flacMetaData=new FLACMetaData($completeFilename);

			// If OK, use them
			if(count($flacMetaData->tag))	$id3=$flacMetaData->tag+$id3;

			break;
			// MPC: not supported
		case 'mpc':
			break;
			// MP3
		default:
			if(!defined('MF_LOADED')) require(INCLUDE_DIR.'miscFunctions.php');
			// Get ID3V2 tags
			if(!$id3V2Tags)	$id3V2Tags=new id3V2Ext($completeFilename,false,true,isset($options['vbrParsing']));
			else $id3V2Tags->parseFile($completeFilename,false,true,isset($options['vbrParsing']));

			// If OK, use them
			if(count($id3V2Tags->tag)>3) $id3=$id3V2Tags->tag+$id3;
			// Else, use php_id3 to get ID3V1 tags
			else{
				$id3Best=@id3_get_tag($completeFilename, ID3_BEST);
				// If V2 unexpectedly failed (problem reading frames), force try ID3 V1
				if($id3Best===false){
					$version = @id3_get_version($completeFilename);
					if($version & ID3_V1_1) $id3Best=@id3_get_tag($completeFilename, ID3_V1_1);
					elseif($version & ID3_V1_0) $id3Best=@id3_get_tag($completeFilename, ID3_V1_0);
				}
				if(!is_array($id3Best)) $id3Best=array();
				$id3=array_merge($id3,efGetAudioInfoUnsetEmptyValues($id3Best));

				// UTF8/16 decode
				foreach ($id3 as $key=>$value) {
					if(substr($value,0,2)=='') $id3[$key]=cfCodePageDecode(substr($value,2),'UTF-16');
					elseif(substr($value,0,2)=='') $id3[$key]=cfUTF8Decode(substr($value,2),true,false,false);
				}

				// Clear null caracters
				foreach ($id3 as $key=>$value) $id3[$key]=str_replace(chr(0),'',$value);

				// Mix with $id3V2 tags
				foreach ($id3V2Tags->tag as $k=>$v) if($v!==false) $id3[$k]=$v;
			}
			// Genre
			if(isset($id3['genre']) && is_numeric($id3['genre'])){
				if(isset($genre[(int)$id3['genre']])) $id3['genre']=$genre[$id3['genre']];
			}
			//unset($id3V2Tags);
			break;
	}


	// Use album artist instead of artist
	if(isset($id3['band']) && $id3['band']) {
		// Move artist in original artist
		if(!isset($id3['originalArtist'])) $id3['originalArtist']=$id3['artist'];
		$id3['artist']=$id3['band'];
	}

	// Clean strings (used by files/music explorers)
	if($returnCaptionForEmptyFields){
		if(!$id3['artist']) {$id3['artist']=cfCaption('explorerAudioUnknownArtist'); $filledArtist=true;}
		if(!$id3['album']) {$id3['album']=cfCaption('explorerAudioUnknownAlbum'); $filledAlbum=true;}

		$id3['artist']=str_replace('"',"'",$id3['artist']);
		$id3['album']=str_replace('"',"'",$id3['album']);
		if(isset($id3['title'])) $id3['title']=str_replace('"',"'",$id3['title']);
	}
	// Clean unsupported ascii chars in text fields
	foreach ($id3 as $tag=>$v) if($tag=='artist'||$tag=='album'||$tag=='title'||$tag=='band'||$tag=='composer') {
		$fixed=0;
		for($i=strlen($v)-1;$i>=0;$i--) if(ord(substr($v,$i,1))<32){
			$fixed=1;
			$v=substr($v,0,$i).'?'.substr($v,$i+1,strlen($v)-1);
		}
		if($fixed) $id3[$tag]=$v;
	}

	// Generate song's label
	if(isset($id3['title'])){
		if($id3['artist'])	$mdArtist=$id3['artist']; else $mdArtist=' '.cfCaption('explorerAudioUnknownArtist');
		if($id3['album'])	$mdAlbum=$id3['album']; else $mdAlbum=cfCaption('explorerAudioUnknownAlbum');
		if($id3['track'])	$id3['label']= $mdArtist.' - '.$mdAlbum.' - '.$id3['track'].' - '.$id3['title'];
		else 						$id3['label']= $mdArtist.' - '.$mdAlbum.' - '.$id3['title'];
	}
	else {
		$id3['label']=cfFileWithoutExtension(basename($completeFilename));
		$id3['title']=cfFileWithoutExtension(basename($completeFilename));
	}
	// Insert data into cache
	if($cacheData && !isset($options['noCache'])){
		if(isset($filledArtist)) $id3['artist']='';	if(isset($filledAlbum)) $id3['album']='';
		foreach ($id3 as $k=>$v) if(strpos($v,'"')) $id3[$k]=str_replace('"',"'",$v);

		// Concatenate request to queue
		$_ENV['efGetAudioInfoCache'][]="INSERT INTO tracks VALUES ('".dbEscapeString($completeFilename,$_ENV['efGetAudioInfoCacheDB'])."', ".@filemtime($completeFilename).", '".
			dbEscapeString($id3["label"],$_ENV['efGetAudioInfoCacheDB'])."', '".dbEscapeString($id3["title"],$_ENV['efGetAudioInfoCacheDB'])."', '".dbEscapeString($id3["artist"],$_ENV['efGetAudioInfoCacheDB'])."', '".
			dbEscapeString($id3["album"],$_ENV['efGetAudioInfoCacheDB'])."', '".dbEscapeString($id3["track"],$_ENV['efGetAudioInfoCacheDB'])."', '".dbEscapeString($id3["year"],$_ENV['efGetAudioInfoCacheDB'])."', '".
			dbEscapeString($id3["genre"],$_ENV['efGetAudioInfoCacheDB'])."', '".dbEscapeString($id3["comment"],$_ENV['efGetAudioInfoCacheDB'])."', ".$id3["length"].", ".
			((int)(isset($options["vbrParsing"]))).", ".((isset($id3["APIC"]))?'1':'0').", ".((isset($id3["lyrics"]))?'1':'0').")";
	}
	return $id3;
}
function efGetAudioInfoUnsetEmptyValues($arr){
	if(!is_array($arr)) return array();
	foreach ($arr as $key=>$value) if($value==='') unset($arr[$key]);
	return $arr;
}

/**
 * @desc Commit changes to audio info cache DB
 *
 */
function efGetAudioInfoCommitCache(){
	if(count($_ENV['efGetAudioInfoCache'])){
		dbBeginTransaction($_ENV['efGetAudioInfoCacheDB']);
		foreach ($_ENV['efGetAudioInfoCache'] as $query) dbExec($query,$_ENV['efGetAudioInfoCacheDB']);
		dbCommitTransaction($_ENV['efGetAudioInfoCacheDB']);
		unset($_ENV['efGetAudioInfoCache']);
	}
	dbClose($_ENV['efGetAudioInfoCacheDB']);
}

/**
 * @desc Return utf-8 encoded lyrics of a given song
 *
 * @param string $cfn: filename
 * @param string $rawLyrics: raw ID3 lyrics tag content (unsynchLyricTranscr, ULT/USLT), to be used if this content has already been retreived
 * @return string Lyrics
 */
function efGetAudioLyrics($cfn,$rawLyrics=false){
	if(!$rawLyrics){
		require_once(INCLUDE_DIR.'miscFunctions.php');
		$a=new id3V2Ext($cfn,false,false,false);
		if(!isset($a->tag['USLT'])) return false;
		$l=$a->tag['USLT'];
	}
	else $l=$rawLyrics;

	// Get character encoding from header
	$enc=ord(substr($l,0,1));
	$l=substr($l,strpos($l,"\0\0")+2);

	// Convert from unicode or 8859-1 to utf8
	if($enc==1)	$l=(iconv('UTF-16','UTF-8',$l)); else $l=(iconv('ISO-8859-1','UTF-8',$l));

	// Remove comments
	while (($cs=strpos($l,'[[')) && ($ce=strpos($l,']]',$cs))){
		$l=substr($l,0,$cs).substr($l,$ce+3);
	}
	return $l;
}

/**
 * @desc compute number of files and total size of a directory and subdirectories
 *
 * @param string : $completeFilename : path of upper dir
 * @return array : nb of files, total size
 */
function efDirGetFilesSize($completeFilename){
	$nb=0;$size=0;
	if($handle=@opendir($completeFilename)){
		while (false !== ($file = readdir($handle))) if($file!='.' && $file!='..'){
			if(is_file($completeFilename.'/'.$file)){
				if(!cfFileRights($completeFilename,'download',false)) continue;
				$nb++;
				$size+=cfFileSize($completeFilename.'/'.$file);
			}
			else{
				if(!cfFileRights($completeFilename,'state',false)) continue;
				list($nbD,$sizeD)=efDirGetFilesSize($completeFilename.'/'.$file);
				$nb+=$nbD;
				$size+=$sizeD;
			}
			cfPingUpdate(100); // Ping app
		}
	}
	return array($nb,$size);
}

/**
 * @desc Get custom meta data on a file (rating, id3 tags, lyrics link...)
 *
 * @param mixed $completeFilename: name of file to query or array of files
 * @param string $dataName: name of meta data to be retrieved, or false to get everything about a file
 * @return mixed
 */
function efFileMetaGet($completeFilename,$dataName=false){

	// First call: connect to meta data database (and create if needed)
	if(!isset($_ENV['efMetaInfoDB'])) {

		/**
		 * @desc Close meta tags database. Called on script shutdown when needed
		 *
		 */
		function efFileMetaCommitDB(){dbClose($_ENV['efMetaInfoDB']);}

		require_once(INCLUDE_DIR.'databaseFunctions.php');
		$metaDbFile=cfAppDataDir().'/metaDataV1.db';
		if(!is_file($metaDbFile)) $new=1;
		$_ENV['efMetaInfoDB']=dbOpenDB($metaDbFile);
		if(!$_ENV['efMetaInfoDB']) return false;
		if(isset($new)) dbSingleQuery('CREATE TABLE meta (fmd5 TEXT PRIMARY KEY, metaData TEXT)', $_ENV['efMetaInfoDB']);
		register_shutdown_function('efFileMetaCommitDB');
	}
	// Pass an empty file name to jus open meta database
	if(!$completeFilename) return;

	// Multiple files query
	if(is_array($completeFilename)){
		$w='';
		$md5s=array();

		// Large number of file: put MD5 list into a temp table
		if(0&&count($completeFilename)>100){

		}
		else{
			$nb=0;
			foreach ($completeFilename as $file){
				$m=md5($file);
				$md5s[$m]=$file;
				$w.=" OR fmd5='".$m."'";
				if($nb++>900) break; // TODO: large number of files
			}
			if(!$w) return;
			$w=substr($w,4);
		}
	}
	// Single file query
	else $w="fmd5='".md5($completeFilename)."'";

	// Do query
	$query='SELECT * FROM meta WHERE '.$w;

	$result=dbArrayQuery($query,$_ENV['efMetaInfoDB']);

	// Single file response
	if(!is_array($completeFilename)){
		if(!count($result)) return false;
		$result=unserialize($result[0]['metaData']);
		if($dataName) return @$result[$dataName];
		return $result;
	}
	// Multi file response
	else{
		$out=array();
		/*if(is_array($result)) */foreach ($result as $res) {
			if($dataName===false) $out[$md5s[$res['fmd5']]]=unserialize($res['metaData']);
			else $out[$md5s[$res['fmd5']]]=cfArrayItem(unserialize($res['metaData']),$dataName);
		}

		return $out;
	}
}

/**
 * @desc Set a meta data for a given file
 *
 * @param string $completeFilename
 * @param string $dataName: name of data to pass, set to false to indicate $dataValue contains all meta data
 * @param mixed $dataValue
 * @return bool
 */
function efFileMetaSet($completeFilename,$dataName=false,$dataValue=false){
	if(!file_exists($completeFilename)) return false;
	if($dataName) {
		$currentMeta=efFileMetaGet($completeFilename);
		$currentMeta[$dataName]=$dataValue;
	}
	else {
		if(!isset($_ENV['efMetaInfoDB'])) efFileMetaGet(false);
		$currentMeta=$dataValue;
	}
	$query="INSERT OR REPLACE INTO meta VALUES ('".md5($completeFilename)."', '".dbEscapeString(serialize($currentMeta),$_ENV['efMetaInfoDB'])."')";
	$res=dbSingleQuery($query,$_ENV['efMetaInfoDB']);
	return $res;
}

/**
 * @desc Rate a file (1 to 5)
 *
 * @param string $completeFilename
 * @return bool
 */
function efFileRate($completeFilename,$score){
	if(!is_numeric($score) || $score<1 || $score>5) return false;
	if(!is_file($completeFilename)) return false;

	$rating=efFileMetaGet($completeFilename,'rating');

	// Verify user (IP) has not already rated this file
	if($rating) foreach (explode('/',$rating) as $vote){
		list($ip,$note)=explode(':',$vote);

		if($ip==$_SERVER['REMOTE_ADDR']) return false;
	}
	if($rating) $rating.='/';
	$rating.=$_SERVER['REMOTE_ADDR'].':'.$score;

	efFileMetaSet($completeFilename,'rating',$rating);
	return true;
}

/**
 * @desc Return rating data for a file
 *
 * @param string $completeFilename
 * @param string $ratingData: meta data for rating. If not set, extract metadata in this function
 * @param boolean $compact: set to true to return javascript compacted string instead of array
 * @return array 'score', 'nb', 'hasVoted'
 */
function efFileGetRating($completeFilename, $ratingData=false, $compact=false){
	// If rating data not already retreived, do it now
	if($ratingData===false)	{
		if(!file_exists($completeFilename)) return false;
		$ratingData=efFileMetaGet($completeFilename,'rating');
	}

	// Compute score and check if user has already voted
	$nb=0;
	$score=0;
	$voted=false;

	if($ratingData) foreach (explode('/',$ratingData) as $vote){
		if(!strpos($vote,':')) break;
		list($ip,$note)=@explode(':',$vote);
		$score+=$note;
		$nb++;
		if($ip==$_SERVER['REMOTE_ADDR']) $voted=true;
	}
	if($nb>0) $score=floor(10*$score/$nb)/10;

	if($compact) return ((($score==0)?'0':'').(int)($score*10)).(($voted)?1:0).$nb;
	return array('score'=>$score,'nb'=>$nb,'hasVoted'=>$voted);
}

/**
 * Store in meta data info that file has been viewed once
 *
 * @param string $completeFilename
 * @return nb of times file has been viewed
 */
function efFileSetViewedOnce($completeFilename){
	if(!file_exists($completeFilename)) return false;
	$viewed=((int)efFileMetaGet($completeFilename,'viewed'))+1;
	efFileMetaSet($completeFilename,'viewed',$viewed);
	return $viewed;
}

/*
***************************************************************************************************************************
* Download functions
***************************************************************************************************************************
*/

/**
 * @desc transforms a value into file size format (O/Ko/Mo/Go)
 *
 * @param string $file : file size (octets)
 * @return string formated file size
 */
function efFileSizeFormat($fileSize){
	if ($fileSize<1024) return efNumberFormat($fileSize, 0)." &nbsp;B";
	if ($fileSize<1024*1024) return efNumberFormat($fileSize/1024, 2)." KB";
	if ($fileSize<1024*1024*1024) return efNumberFormat($fileSize/1024/1024, 2)." MB";
	else return efNumberFormat($fileSize/1024/1024/1024,3)." GB";
}

/**
 * @desc read tokens in tokens file (transferTokens.txt)
 *
 * @return array: tokens
 */
function efTokensRead(){

	$removed=false;
	$tokensFile=cfAppDataDir().'/transferTokens.txt';

	// Load from memory
	if(!($tokenList=cfMGetVar('weezoTransferTokens'))){
		if(!file_exists($tokensFile)) return array();
		$tokenList=cfParse_ini_file($tokensFile,true,false);
		if(!is_array($tokenList)) $tokenList=array();
	}

	// Remove expired tokens
	$removed=array();

	foreach ($tokenList as $key=>$value) if(!efTokenIsValid($value)) $removed[]=$key;

	if(count($removed)) {
		foreach ($removed as $k) unset($tokenList[$k]);
		cfWriteIniFile($tokenList,$tokensFile,true);
	}
	return $tokenList;
}

/**
 * @desc Save token array to file (replace file content, don't append !)
 *
 * @param array $tokenList : array of tokens
 */
function efTokensWrite($tokenList){
	cfMSetVar('weezoTransferTokens',$tokenList);
	cfWriteIniFile($tokenList,cfAppDataDir().'/transferTokens.txt',true);
}
/**
 * @desc Check if a token is perempted or has reached its downloads limit
 *
 * @param array $token
 * @return boolean
 */
function efTokenIsValid(&$token){
	static $t=0;
	static $dTL=0;
	if($t==0){$t=time();$dTL=cfGGetVar('downloadTokenLifetime');}
	if(!isset($token['type'])) return false;
	if(isset($token['limitNbOriginal']) && isset($token['limitNb']) && $token['limitNb']<=0) return false;
	if(isset($token['expiry']) && $token['expiry'] && time()>$token['expiry']) return false;
	if($token['type']=='download' && (!isset($token['userId']) || ($dTL>0 && ($t-$token['creationTime'])>$dTL))) return false;
	return true;
}

/**
 * @desc process finished download's token :
 * 		decrement limitNb or unset if necessary
 *
 * @param string $tokenId
 */
function efTokenDlFinished($tokenId){
	$tokenList=efTokensRead();
	if(isset($tokenList[$tokenId])){
		if(!isset($tokenList[$tokenId]['limitNb']) || $tokenList[$tokenId]['limitNb']==1) unset($tokenList[$tokenId]);
		else $tokenList[$tokenId]['limitNb']--;
		efTokensWrite($tokenList);
	}
}

/**
 * @desc closed session download resume
 * 		check download existence in token database
 *
 */
function efResumeDownload($tokenId){
	require_once(INCLUDE_DIR.'initFunctions.php');
	if(substr($tokenId,0,7)=='dlToken') $tokenId=substr($tokenId,7);
	// Load stored tokens and Check token existence
	$tokenList=efTokensRead();
	if(!$tokenList || !isset($tokenList[$tokenId])) {require_once(INCLUDE_DIR.'outputFunctions.php');outDisplayErrorPage(cfCaption('directLinkNotValid'));}

	// Check file existence
	if(!file_exists($tokenList[$tokenId]['filename'])) efDisplayError(404);

	// Load user data (downloads only)
	if($tokenList[$tokenId]['type']=='download'){
		$userFiles=glob(cfAppDataDir().'/groups/*.usr');
		foreach ($userFiles as $value){
			$userData=cfParse_ini_file($value, true);
			if(isset($userData['id']) && $userData['id']==$tokenList[$tokenId]['userId']){
				$_SESSION['user']=$userData;
				$_SESSION['user']['id']=$tokenList[$tokenId]['userId'];
				break;
			}
		}
		// If user not found
		if(!isset($_SESSION['user'])) efDisplayError(403);
	}
	// Direct link
	elseif ($tokenList[$tokenId]['type']=='directLink')	{
		// Bittorrent transfer: send .torrent file to output
		if(@$tokenList[$tokenId]['torrent']){
			// Show message box on server
			cfServerShowMessageForm(cfCaption('genDirectDownload').' (torrent)',cfUTF8Encode(basename($tokenList[$tokenId]['filename'])),$_SERVER['REMOTE_ADDR'],cfAppDocRoot().'/gfx/directLinkTorrent32.gif','#85d285','#aae0aa');
			$metainfo=base64_decode($tokenList[$tokenId]['torrent']);
			header('Content-Type: application/x-bittorrent');
			header('Content-Disposition: attachment; filename="'.cfUTF8Encode(basename(cfFileWithoutExtension($tokenList[$tokenId]['filename']))).'.torrent"');
			header('Content-Length: '.strlen($metainfo));
			die($metainfo);
		}
	}
	elseif ($tokenList[$tokenId]['type']=='publishDownload')	{}
	// Else... error
	else efDisplayError(403);

	// Show message box on server
	if(cfGGetVar('visualNotification') && $tokenList[$tokenId]['type']=='directLink'){
		if($tokenList[$tokenId]['downloadFormat']=='zip') cfServerShowMessageForm(cfCaption('genDirectDownload'),cfUTF8Encode(basename($tokenList[$tokenId]['filename'])),$_SERVER['REMOTE_ADDR'],cfAppDocRoot().'/gfx/directLink32.gif','#F00','#F80');
	}
	if(cfGGetVar('visualNotification') && $tokenList[$tokenId]['type']=='publishDownload'){
		cfServerShowMessageForm(cfCaption('publishDownloadLabel'),cfUTF8Encode(basename($tokenList[$tokenId]['filename'])),$_SERVER['REMOTE_ADDR'],cfAppDocRoot().'/gfx/directLink32.gif','#F00','#F80');
	}

	// Start download

	// Zipped folder
	if($tokenList[$tokenId]['type']=='directLink' && is_dir($tokenList[$tokenId]['filename'])){
		// Generate zip and send it to output
		$outputFilename=basename($tokenList[$tokenId]['filename']);
		if(is_dir($tokenList[$tokenId]['filename'])) $outputFilename.='.zip';
		efZipDownload(array($outputFilename => $tokenList[$tokenId]['filename']),basename($tokenList[$tokenId]['filename']).'.zip',cfGGetVar('directLinkDownloadSpeedLimit'));
		// Unset / modify token
		efTokenDlFinished($tokenId);
		exit();
	}

	// Single file
	if(!efForceDownload($tokenList[$tokenId]['filename'], $tokenList[$tokenId], $tokenId)) efDisplayError(403);
}

/**
 * @desc proceed to $completeFilename download (first checks access rights for new transfers)
 * 		Send HTTP headers and stream file
 *
 * @param string $completeFilename : path & filename of string
 * @param array $token (optional - if not present, create new token) : download token
 * @param string $tokenId (optional - if not present, create new token) : download token Id
 * @return boolean : false if file not authorized or doesn't exist
 */
function efForceDownload($completeFilename, $token=false, $tokenId=false){
	$range=0;
	$completeFilename=cfCleanPathName($completeFilename);
	$fileRightsOK=false;

	// Resource specific file source and rights function : bypass common file's existence and access rights
	// and (may) replace given file name by real file name
	if(substr($completeFilename,0,14)=='*resSpecific*/'){
		if(cfRGetVar('extAccessFunction')) $extAccessFunction = create_function('$completeFilename', cfRGetVar('extAccessFunction'));
		if(!($completeFilename=$extAccessFunction($completeFilename))) return false;
		$fileRightsOK=true;
	}

	// File is actually a directory: zip it
	if(is_dir($completeFilename)) {
		if(cfRGetVar('multipleDownloadAllowed'))
			efZipDownload(array($completeFilename),basename($completeFilename).'.zip',cfUGetVar('downloadSpeedLimit'));
		exit;
	}

	// Generate token if needed
	if(!$token){
		// Token id sent by client
		if(isset($_GET['data5']) && substr($_GET['data5'],0,7)=='dlToken'){
			$tokenId = eregi_replace('[^0-9]','',substr($_GET['data5'],7));
			if(!$fileRightsOK && !cfFileRights($completeFilename,'download')) return false; // if download not authorized, exit;
			$tokenList=efTokensRead();
			$tokenList[$tokenId]['type']='download';
			$tokenList[$tokenId]['filename']=$completeFilename;
			$tokenList[$tokenId]['creationTime']=time();
			$tokenList[$tokenId]['userId']=$_SESSION['userId'];
			$tokenList[$tokenId]['limitNb']="1";
			$token=$tokenList[$tokenId];
			efTokensWrite($tokenList);
		}
		// Else, simply download file, with no resume
		else $token['type']='download';
	}
	// get HTTP header
	$clientHeaders=getallheaders();
	if(isset($clientHeaders['Range']) && strtolower(substr($clientHeaders['Range'],0,6)=='bytes=')){
		$range=trim(substr($clientHeaders['Range'],6));
	}

	// process range request
	if(isset($range) && strpos($range,'-')!==false){
		$range=trim($range);
		$fileSize=cfFileSize($completeFilename);

		// From
		$from=substr($range,0,strpos($range,'-'));
		if(!strlen($from)) $from=0;
		if(!is_numeric($from) || $from>$fileSize) $rangeErr=true;

		// To
		$rto=substr($range,strpos($range,'-')+1);
		if(!strlen($rto)) {
			$to = $fileSize-1;
		}
		else $to=$rto;
		if($to+1>$fileSize || $to<$from) $rangeErr=true;
		$content_size = 1 + $to - $from;

		// Incorrect range
		if(isset($rangeErr)){
			$range=false;
			header('HTTP/1.1 416 Requested Range Not Satisfiable');
			header("Content-Length: $content_size");
		}
		else{
			header('HTTP/1.1 206 Partial Content');
			header("Content-Range: bytes $from-$to/$fileSize");
			header("Content-Length: $content_size");
			$range=array('from'=>$from,'to'=>1+$to);
		}
	}
	// no range request
	else {
		header('Content-Length: '.cfFileSize($completeFilename));
	}


	// Proceed to download
	header("HTTP/1.0 200 OK");
	header("Accept-Ranges: bytes");
	//header("Pragma: public");
	header("Cache-Control: must-revalidate");
	header('Content-Type: '.efFileMimeType($completeFilename));
	header('ETag: "'.md5($completeFilename).'"');
	header('Content-Disposition: attachment; filename="'.cfUTF8Encode(basename($completeFilename),false,false).'"');
	header('Last-Modified: '.gmdate("D, d M Y H:i:s",@filemtime($completeFilename)).' GMT');
	header('Connection: Keep-Alive');

	// Log in viewed content history
	cfLogContentAccess($completeFilename);

	// close session and stream file
	if($token['type']=='download' || $token['type']=='publishDownload')	cfStreamFile($completeFilename,true,'auto',$range);
	elseif($token['type']=='directLink') cfStreamFile($completeFilename,true,cfGGetVar('directLinkDownloadSpeedLimit'),$range);
	// Clear download token
	efTokenDlFinished($tokenId);
	exit();
}

/**
 * @desc start remote download of $url file into $dir directory
 * if $url valid && direct download authorized in $dir, send command and create corresponding transfers array
 *
 * @param unknown_type $dir
 * @param unknown_type $url
 * @return unknown
 */
function efRemoteDownload($dir, $url){
	if(!cfFileRights($dir,'remoteDownload')) return false;
	require_once(INCLUDE_DIR.'transferFunctions.php');
	return tAddRemoteDownload($dir, $url);
}



/*
***************************************************************************************************************************
* Multiple downloads & playlist
***************************************************************************************************************************
*/

/**
 * @desc : add a file to multiple download file list
 *
 * @param string $fileName : name (without path) of file to add (note : path is efCurrentDirectory)
 */
function efMultipleDownloadAdd($fileName){
	cfAsyncHeader();
	$completeFilename=cfJoinPathFile(cfRGetVar('efCurrentDirectory'),$fileName);

	// List items and into base directory: try to find file or directory
	if(cfSharedMode('list') && cfRGetVar('efCurrentDirectory')=='*resourceBasePath*'){
		foreach (cfSharedItems() as $cfn=>$type) if(basename($cfn)==$fileName){
			$completeFilename=$cfn;
			break;
		}
	}

	// If file doesn't exists or user doesn't have access rights, do nothing
	if((is_file($completeFilename) && cfFileRights($completeFilename,'download')) || (is_dir($completeFilename) && cfFileRights($completeFilename,'read'))) {
		if(!cfRGetVar('mDownloadList',$fileName)){// if a file with same name (but possibly different dir...) is present, abort
			// Update total file number and size
			if(cfHGetVar('mDownloadPrecomputeSize')){
				list($nbFiles,$totalSize)=cfRGetVar('mDownloadTotals');
				if(is_file($completeFilename)){
					$nbFiles++;
					$totalSize+=cfFileSize($completeFilename);
					// Store total files nb and size in mDownloadListTotals
					cfRSetVar('mDownloadListTotals',$fileName, array(1,cfFileSize($completeFilename)));
				}
				else {
					list($nbFilesD,$totalSizeD)=efDirGetFilesSize($completeFilename);

					// Store total files nb and size in mDownloadListTotals
					cfRSetVar('mDownloadListTotals', $fileName, array($nbFilesD,$totalSizeD));
					$nbFiles+=$nbFilesD;$totalSize+=$totalSizeD;
				}
				cfRSetVar('mDownloadTotals',array($nbFiles,$totalSize));
			}
			// Store added file/dir in mDownloadList
			cfRSetVar('mDownloadList',$fileName, $completeFilename);

			// Update buttons
			echo efMultipleDownloadUpdateBt();

			// Update webpage's list
			echo cfAsyncXMLInnerHTMLbyId(outMultipleDownloadListInnerHTML(),'mDlList');
		}
	}
	echo cfAsyncXMLJSaction('mProcessing(false)');// Update reloading icon
	die(cfAsyncFooter());
}

/**
 * @desc : add all current directory files (but no folder) to multiple download file list
 *
 */
function efMultipleDownloadAddAll(){
	$fileAdded=false;
	cfAsyncHeader();
	if(!isset($_ENV['weezoResEfFiles'])) efDirUpdate();

	// Update total file number and size
	if(cfHGetVar('mDownloadPrecomputeSize')){
		list($nbFiles,$totalSize)=cfRGetVar('mDownloadTotals');
		$totals=true;
		$nbFilesD=0;
		$totalSizeD=0;
	}
	else $totals=false;

	$mDownloadList=cfRGetVar('mDownloadList');
	if($totals) $mDownloadListTotals=cfRGetVar('mDownloadListTotals');


	// For each file in current directory
	foreach ($_ENV['weezoResEfFiles'] as $value) if(isset($value['type']) && $value['type']=='file') {
		$completeFilename=$value['completeFileName'];

		// If file download authorized
		if(cfFileRights($completeFilename,'download',false)){

			// Add to mDownload list
			if(!isset($mDownloadList[$value['name']])){// if a file with same name (but possibly different dir...) is present, abort
				$mDownloadList[$value['name']]=$completeFilename;
				if($totals){
					$nbFilesD++;
					$totalSizeD+=cfFileSize($completeFilename);
					// Store files nb and size in mDownloadListTotals
					$mDownloadListTotals[$value['name']]=array(1,cfFileSize($completeFilename));
				}
				$fileAdded=true;
			}
		}
		cfPingUpdate(100); // Ping app one
	}

	// Mobile view
	if(cfIsMobile() && $fileAdded){
		cfRSetVar('mDownloadList',$mDownloadList);
		echo cfAsyncXMLInnerHTMLbyId(mefMultipleDownloadsInnerHTML(),'mdlF');
	}
	// Normal view, update only if new files
	elseif($fileAdded){
		cfRSetVar('mDownloadList',$mDownloadList);
		if($totals) cfRSetVar('mDownloadListTotals',$mDownloadListTotals);

		if(cfHGetVar('mDownloadPrecomputeSize')){
			// Update totals
			cfRSetVar('mDownloadTotals',array($nbFiles+$nbFilesD,$totalSize+$totalSizeD));
		}
		// Update mdl buttons state
		echo efMultipleDownloadUpdateBt();
		// Update webpage's list
		echo cfAsyncXMLInnerHTMLbyId(outMultipleDownloadListInnerHTML(),'mDlList');
	}
	if(!cfIsMobile()) echo cfAsyncXMLJSaction('mProcessing(false)');// Update reloading icon

	die(cfAsyncFooter());
}

/**
 * @desc : remove a file from multiple download file list
 *
 * @param string $fileName : name (without path) of file to remove (note : path is efCurrentDirectory)
 */
function efMultipleDownloadSupp($fileName){
	header('Content-Type:text/xml');
	echo '<?xml version="1.0" encoding="utf-8"?><response>'."\n";
	if(is_array(cfRGetVar('mDownloadList')) && cfRGetVar('mDownloadList',$fileName)){// if passed file is in list

		// Remove file/folder from list
		$mdl=cfRGetVar('mDownloadList');
		$completeFilename=$mdl[$fileName];
		unset($mdl[$fileName]);
		cfRSetVar('mDownloadList',$mdl);

		// Update total file number and size
		if(cfHGetVar('mDownloadPrecomputeSize')){
			// Remove nb files and size info from list
			$mdl=cfRGetVar('mDownloadListTotals');
			list($nbFilesD,$totalSizeD)=$mdl[$fileName];
			unset($mdl[$fileName]);
			cfRSetVar('mDownloadListTotals',$mdl);
			unset($mdl);

			// Update totals
			list($nbFiles,$totalSize)=cfRGetVar('mDownloadTotals');
			cfRSetVar('mDownloadTotals',array($nbFiles-$nbFilesD,$totalSize-$totalSizeD));
		}

		// Update mdl buttons state
		echo efMultipleDownloadUpdateBt();
		// Update webpage's list
		echo cfAsyncXMLInnerHTMLbyId(outMultipleDownloadListInnerHTML(),'mDlList');
	}
	echo cfAsyncXMLJSaction('mProcessing(false)');// Update reloading icon
    cfAsyncFooter();

}

/**
 * @desc : clear multiple download file list
 *
 */
function efMultipleDownloadClear(){
	cfRUnsetVar('mDownloadList');
	cfRSetVarArray('mDownloadList');
	cfRUnsetVar('mDownloadListTotals');
	cfRSetVarArray('mDownloadListTotals');
	cfAsyncHeader();

	// Update totals
	cfRSetVar('mDownloadTotals',array(0,0));

	// Update mdl buttons state
	echo efMultipleDownloadUpdateBt();

	// Update webpage's list
	echo cfAsyncXMLInnerHTMLbyId(outMultipleDownloadListInnerHTML(),'mDlList');
	echo cfAsyncXMLJSaction('mProcessing(false)');// Update reloading icon
	cfAsyncFooter();
}

/**
 * @desc return async HTML code for updating multiple download buttons
 *
 * @return string : HTML code
 */
function efMultipleDownloadUpdateBt(){
	if(cfHGetVar('mDownloadPrecomputeSize')){
		list($nbFiles,$totalSize)=cfRGetVar('mDownloadTotals');
		$output=cfAsyncXMLInnerHTMLbyId('('.cfCaption('explorerFiles',$nbFiles).' / '.efFileSizeFormat($totalSize).')','mDlTotals');
	}
	if(cfRGetVar('mDownloadList')) $output.=cfAsyncXMLJSaction('mToggleButtons(true)'); else $output.=cfAsyncXMLJSaction('mToggleButtons(false)');
	return $output;
}

/**
 * @recursively add dir and files to zip (used by efMultipleDownloadDownload)
 *
 * @param zipfile $zip
 * @param srting $completeFilename : complete file name of directory to add
 * @param string $baseName : path name beetween zip root and current dir
 */
function mdlAddDir(&$zip, $completeFilename, $baseName){
	global $protectSensitiveFiles;
	global $protectedFileExtension;
	global $protectedDirectories;

	$isDlToken=cfCmpLeft($_SERVER['REQUEST_URI'],'/downloads/dlToken');

	if(!($handle = opendir($completeFilename))) return;
	$zip->add_dir($completeFilename,$baseName);

	while (($file = @readdir($handle))!==false) {
		//set_time_limit(10);
		if($file!='.' && $file!='..'){
			$f=$completeFilename.'/'.$file;
			// Check individual file access rights
			// (disabled if not linked to a resource or that resource doesn't have a path property)
			if(is_file($f) && ($isDlToken || !cfRGetVar('path') || cfFileRights($f,'download'))){
				if(!$protectSensitiveFiles || in_array(cfFileExtension($file),$protectedFileExtension)===false) $zip->add_file($f, $baseName.'/'.$file);
			}
			elseif (is_dir($f) && !cfIsGeneralProtected($f,$protectedDirectories)) mdlAddDir($zip, $f, $baseName.'/'.$file);
		}
	}
}

/**
 * @desc launch multiple download containing cfRGetVar('mDownloadList') file list
 * 		No buffering, directly send zip to output
 *
 */
function efMultipleDownloadDownload(){
	global $protectSensitiveFiles;
	if(!is_array(cfRGetVar('mDownloadList'))) exit; // If list is empty (shouldn't happend), exit

	// Create resource specific file convertion / rights function if needed
	if(cfRGetVar('extAccessFunction')) $extAccessFunction = create_function('$completeFilename', cfRGetVar('extAccessFunction'));

	$zips=array();
	foreach (cfRGetVar('mDownloadList') as $fileName=>$completeFilename) {
		$fileRightsOK=false;
		if(substr($completeFilename,0,14)=='*resSpecific*/') {
			if(($completeFilename=$extAccessFunction($completeFilename))!==false) {
				$fileName=basename($completeFilename);
				$fileRightsOK=true;
			}
		}
		if(is_file($completeFilename) && ($fileRightsOK || cfFileRights($completeFilename,'download'))){
			$outputFileName=$fileName;
			$zips[$fileName]=cfCleanPathName($completeFilename);
		}
		elseif (is_dir($completeFilename) && ($fileRightsOK || cfFileRights($completeFilename,'read'))){
			$outputFileName=$fileName;
			$zips[$fileName]=cfCleanPathName($completeFilename);
		}
	}
	// If list is empty (shouldn't happend), exit
	if(!count($zips)) exit;

	// Name output zip file
	if(count($zips)==1) $outputFileName.='.zip'; else $outputFileName=cfcaption('multipleDownloadFileName');

	// Generate and send zip to output
	efZipDownload($zips, $outputFileName, cfUGetVar('downloadSpeedLimit'));
	exit;
}

/**
 * @desc Generate .torrent file from files contained in cfRGetVar('mDownloadList'), and send .torrent file to output
 *
 */
function efMultipleDownloadCreateTorrent(){
	// If list is empty (shouldn't happend), exit
	if(!is_array(cfRGetVar('mDownloadList'))) exit;

	if(!cfGGetVar('bittorrentEnabled')||!cfRGetVar('bittorrentEnabled')) return false;

	require_once(INCLUDE_DIR.'transferFunctions.php');

	// Create transfer and .torrent file
	$tr=tAddBittorrentDownload(array_values(cfRGetVar('mDownloadList')),$metainfo);

	// Failure
	if(!$metainfo) outDisplayErrorPage('Bittorrent file generation error<br/>'.outButton(cfCaption('genBack'),'javascript:history.back()',outIcon('back')));

	// .torrent file header
	header('Content-Type: application/x-bittorrent');
	header('Content-Disposition: attachment; filename="'.cfUTF8Encode($tr->destinationFile).'"');
	header('Content-Length: '.strlen($metainfo));
	echo $metainfo;
	exit;
}


/**
 *  Generate .torrent file from a single file, and send .torrent file to output
 *
 * @param string $completeFilename
 * @return false on failure
 */
function efSinglDownloadCreateTorrent($completeFilename){
	if(!file_exists($completeFilename) || !cfFileRights($completeFilename,'download')) return false;
	if(!cfGGetVar('bittorrentEnabled')||!cfRGetVar('bittorrentEnabled')) return false;

	require_once(INCLUDE_DIR.'transferFunctions.php');

	// Create transfer and .torrent file
	$tr=tAddBittorrentDownload(array($completeFilename),$metainfo);

	// Failure
	if(!$metainfo) return false;

	// .torrent file header
	header('Content-Type: application/x-bittorrent');
	header('Content-Disposition: attachment; filename="'.cfUTF8Encode($tr->destinationFile).'"');
	header('Content-Length: '.strlen($metainfo));
	echo $metainfo;
	exit;
}

/**
 * @desc Create torrent in transfers list, and return .torrent content
 *
 * @param array $files
 * @param string $tokenId: id of directLink or publishDownload
 * @param array $options: see tAddBittorrentDownload for details
 * @return array(transfer id, torrent file binary content)
 */
function efTorrentCreate($files, $tokenId=false, $options=array()){
	require_once(INCLUDE_DIR.'transferFunctions.php');

	// Create transfer and .torrent file
	$tr=tAddBittorrentDownload(array_values($files),$metainfo,$tokenId,$options);

	if(!$tr) return array(false,false);
	return array($tr->id,$metainfo);
}

/**
 * @desc generate and send a zip file to browser
 *
 * @param array $files : files and folders put in zip (filename1 => completeFilename1,
 * @param string $outputFileName : generated zip file name
 * @param int $speedLimit : download speed limit (KBytes/s)
 */
function efZipDownload($files, $outputFileName, $speedLimit=false, $isTransfer=true){
	require(INCLUDE_DIR.'zip.php');
	require(INCLUDE_DIR.'transferFunctions.php');

	// Store protected directories and files before closing session
	$protectSensitiveFiles=cfGGetVar('protectSensitiveFiles');
	$protectedDirectories=cfGGetVar('protectedDirs');

	// Headers
	wSession_write_close();
	set_time_limit(0);
	//header("Pragma: public");
	header('Content-Description: file Transfert');
	Header("Content-type: application/x-zip");
	Header('Content-disposition: attachment; filename="'.cfUTF8Encode($outputFileName).'"');
	header("Content-Transfer-Encoding: binary");
	header("Cache-Control: must-revalidate, post-check=0, pre-check=0, public");
	header("Expires: 0");
	header("Connection: Keep-Alive");


	// Create zip file
	$zip = new zipfile2();
	$zip->setSpeedLimit($speedLimit);
	if($isTransfer) $zip->t=tAddMultipleDownload($outputFileName); // Monitor multiple download as transfer
	if(cfHGetVar('mDownloadPrecomputeSize')) list($nbFiles,$zip->uncompressedSize)=cfRGetVar('mDownloadTotals'); else $zip->uncompressedSize=0;

	// Add files and dirs to zip
	foreach ($files as $fileName=>$completeFilename){
		if(is_file($completeFilename)) $zip->add_file($completeFilename, $fileName);
		elseif (is_dir($completeFilename)) mdlAddDir($zip, $completeFilename, basename($completeFilename));
	}
	// Send zip central directory
	$zip->file();

	if($isTransfer) $zip->t->updateStatus(4,true); // Set transfer as finished
}

/**
 * @desc : generate multiple downloads list from playlist
 *
 */
function efConvertPlayListToMDList(){
	//if(!cfRGetVar('multipleDownloadAllowed')) return;
	$mdl=array();
	foreach (cfRGetVar('playlist') as $value) $mdl[basename($value['completeFileName'])]=$value['completeFileName'];
	cfRSetVar('mDownloadList',$mdl);
}

/**
 * @desc Generate JS Array to send to plRefresh JS function for refreshing playlist display
 * @desc string : JS Array : Array(filename1,label1, filename2,label2,...)
 *
 */
function efPlayListContentArray(){
	$arrayMainPlaylist='Array(';
	$itemsFound=false;
	if(!is_array(cfRGetVar('playlist')) || count(cfRGetVar('playlist'))==0) {}
	// Playlist files
	else foreach (cfRGetVar('playlist') as $key=>$song) {
		if($itemsFound) $arrayMainPlaylist.=','; else $itemsFound=true;
		// File name
		if(substr($song['completeFileName'],0,14)=='*resSpecific*/') $arrayMainPlaylist.='"'.cfProtectFilename($song['completeFileName']).'",';
		else $arrayMainPlaylist.='"'.cfProtectFilename(cfResourceRelativePath($song['completeFileName'])).'",';
		// File label
		$arrayMainPlaylist.='"'.str_replace('"','\\"',stripslashes(cfUTF8Encode($song['label']))).'"';
	}
	return $arrayMainPlaylist.')';
}

/**
 * @desc Return true if supported playlist format
 *
 * @param string $n: filenam
 * @return bool
 */
function efIsPlaylist($n){
	$e=cfFileExtension($n);
	if($e=='m3u' || $e=='pls') return true;
}




/*
***************************************************************************************************************************
* Misc functions
***************************************************************************************************************************
*/


/**
 * @desc  Return path where images and video thumbnails should be placed, create directory if needed
 *
 * @return string
 */
function efThumbnailsPath(){
	if(cfSharedMode('list')){
		$thumbnailsPath=cfJoinPathFile(cfRGetVar('resourceDataDir').'/list/',substr(cfResourceRelativePath(cfRGetVar('efCurrentDirectory')),18));
		if(substr($thumbnailsPath,-1)=='/') $thumbnailsPath=cfStrShorten($thumbnailsPath,1);
	}
	else{
		if(cfRGetVar('path')!='computerRoot') $thumbnailsPath=cfRGetVar('resourceDataDir').substr(cfRGetVar('efCurrentDirectory'), strlen(cfCleanPathName(cfRGetVar('path'))));
		else $thumbnailsPath=cfRGetVar('resourceDataDir').'/'.str_replace(':/','/',cfRGetVar('efCurrentDirectory'));
	}
	// Create thumbnails dir if non-existant
	if(!is_dir($thumbnailsPath)) cfMkdir($thumbnailsPath,null,true,null);
	return $thumbnailsPath;
}

/**
 * @desc Return complete file name for a file's thumbnail
 * @param string $completeFilename: file path & name
 * @return string thumbnail's filename
 */
function efThumbnailFilePath($completeFilename){
	static $thumbnailsPath='';
	static $tnS=0;

	// thumbnails path & size
	if(!$tnS){
		$thumbnailsPath=efThumbnailsPath();
		$tnS=cfRGetVar('thumbnailsSize');
	}
	$fileExt=cfFileExtension($completeFilename);

	// .ico files: return file itself
	if($fileExt=='ico') return $completeFilename;

	return $thumbnailsPath.'/'.$tnS.basename($completeFilename).'.jpg';
}

/**
 * @desc Return thumbnails max width
 *
 * @return integer
 */
function efThumbnailsMaxWidth(){
	$s=cfRGetVar('thumbnailsSize');
	$sf=cfRGetVar('thumbnailsSizeFactor');
	if($sf==2) return (cfRGetVar('cropThumbnails'))?floor($s*2/3):$s;
	return floor(min(4,(pow($sf,2)+1))*$s/4);
}

/**
 * @desc Generate video thumbnail
 *
 * @param string $source: source complete filename
 * @param string $destination: jpg destination complete filename, falses to send to output
 * @param int $w: dest max width
 * @param int $h: dest max height
 * @param int $q: dest JPG quality (1-100)
 * @param int $videoThumbnailOffset: offset from start (seconds)
 */
function efMakeVideoThumbnail($source,$destination,$w=0,$h=0,$q=0,$videoThumbnailOffset=0){
	if(!$w) $w=cfRGetVar('thumbnailsSize');
	if(!$h) $h=cfRGetVar('thumbnailsSize');
	if(!$q) $q=DEFAULT_JPG_QUALITY;
	$w=2*(floor($w/2));
	$h=2*(floor($h/2));

	if($videoThumbnailOffset==0) $videoThumbnailOffset=(int)cfRGetVar('videoThumbnailOffset');

	// Generate image from video
	$cl=' -ss '.(int)$videoThumbnailOffset.' -i "'.$source.'" -vframes 1 -an "'.cfAppDataDir().'/temp/temp%0d.jpg"';
	cfFfmpegLog($cl,'Thumbnail generation');
	cfFfmpegExec($cl);

	// If failure, try to generate image from frame 0
	if(!file_exists(cfAppDataDir().'/temp/temp1.jpg')){
		$cl='-ss 0 -i "'.$source.'" -vframes 1 -an "'.cfAppDataDir().'/temp/temp%0d.jpg"';
		cfFfmpegLog($cl,'Gen video thumbnail');
		cfFfmpegExec($cl);

		if(!file_exists(cfAppDataDir().'/temp/temp1.jpg')) return false;
	}

	// Resize thumbnail from image
	cfCreateResizedJPG(cfAppDataDir().'/temp/temp1.jpg',0,0,$destination,$w,$h,4,$q);

	@unlink(cfAppDataDir().'/temp/temp1.jpg');
	return true;
}

/**
 * @desc Display a progress bar and generate video thumbnails
 *
 * @param array $thumbnails
 */
function efMakeVideoThumbnails($thumbnails){
	// IE doesn't support real-time async processing: if more than 5 thumbnails, sync reload page
	if(cfIsAsync() && count($thumbnails)>2 && !cfBGetVar('asyncHTTPStreaming')) {
		echo cfAsyncXMLJSaction('wl.goURL()');
		die(cfAsyncFooter());
	}

	// Display monitoring stuff
	cfRTScript('var tMN=D.createElement("DIV");tMN.id="tnMonitoringDiv";tMN.style.display="'.((cfIsAsync())?'none':'').'";tMN.innerHTML=\'<div class="mask" id="tnMonitoringMask">&nbsp;</div><div style="position:absolute;top:50%;left:50%;z-index:999999"><div class="popup" style="width:24em;left:-12em;top:-4em">'.outFrameHeaderTable('popupHeader',outImage(outIcon('loading'),false,false,'vertical-align:middle;margin-right:0.5em').cfCaption('genThumbnails'),outImage(outIcon('fi/icovid'),false,false,'vertical-align:middle')).'<br><center>'.outProgressBar(0,'80%','0/'.count($thumbnails),'tnProg').'</center><br></div></div>\';D.body.appendChild(tMN);');

	set_time_limit(0); $nb=0;
	foreach ($thumbnails as $completeFilename=>$destination){
		// Update progress bar
		cfRTScript('tnUpdt("'.$nb.' / '.count($thumbnails).'",'.floor(100*$nb/count($thumbnails)).')');
		// Generate thumbnail
		efMakeVideoThumbnail($completeFilename,$destination,cfRGetVar('thumbnailsSize'),cfRGetVar('thumbnailsSize'));
		$nb++;
	}
	cfRTScript('removeNode(dgi("tnMonitoringDiv"))');
	set_time_limit(ini_get('max_execution_time'));
}

/**
 * @desc Display a progress bar and generate photo thumbnails
 * 		Note: cfRTScript is disabled for mobiles
 *
 * @param array $path: source directory
 * @param array $thumbnailsPath: destination directory
 */
function efMakePhotoThumbnails($thumbnails){
	// IE doesn't support real-time async processing: if more than 5 thumbnails, sync reload page
	if(cfIsAsync() && count($thumbnails)>5 && !cfBGetVar('asyncHTTPStreaming')) {
		echo cfAsyncXMLJSaction('wl.goURL()');
		die(cfAsyncFooter());
	}

	// Base thumbnails should be better encoded as they are bound to be reecoded one more time => set min quality to 90%
	$thumbnailJPGQuality=max(90,cfHGetVar('thumbnailJPGQuality',DEFAULT_JPG_QUALITY));

	// Called by cfStreamProc on data reception
	function efMakePhotoThumbnailsCB($buff){
		static $nbPhotoThumbnails=0;
		static $nbPhotoThumbnailsProcessed=0;
		static $remBuff='';
		static $lastUpdate;

		// Init total number of thumbnails to be processed
		if(is_integer($buff)) {
			$nbPhotoThumbnails=$buff;
			return;
		}

		$buff=$remBuff.$buff;
		$remBuff=substr($buff,strrpos($buff,'OK')+2);
		// Count newly created thumbnails
		str_replace('OK','',$buff,$cnt);
		// If any
		if($cnt){
			$nbPhotoThumbnailsProcessed+=$cnt;
			// Update progress bar
			if(microtime(true)>$lastUpdate+0.5){
				cfRTScript('tnUpdt("'.$nbPhotoThumbnailsProcessed.' / '.$nbPhotoThumbnails.'",'.floor(100*$nbPhotoThumbnailsProcessed/$nbPhotoThumbnails).');');
				$lastUpdate=microtime(true);
			}
		}
	}

	// Display synchronous monitoring stuff
	cfRTScript('var tMN=D.createElement("DIV");tMN.id="tnMonitoringDiv";tMN.style.display="'.((cfIsAsync())?'none':'').'";tMN.innerHTML=\'<div class="mask" id="tnMonitoringMask">&nbsp;</div><div style="position:absolute;top:50%;left:50%;z-index:999999"><div class="popup" style="width:24em;left:-12em;top:-4em">'.outFrameHeaderTable('popupHeader',outImage(outIcon('loading'),false,false,'vertical-align:middle;margin-right:0.5em').cfCaption('genThumbnails'),outImage(outIcon('fi/icoImage'),false,false,'vertical-align:middle')).'<br><center>'.outProgressBar(0,'80%','0/'.count($thumbnails),'tnProg').'</center><br></div></div>\';D.body.appendChild(tMN)');

	set_time_limit(0); $nb=0;
	$tnSize=cfRGetVar('thumbnailsSize');

	// Disabled use of createJPG: PHP-generate
	if(cfGGetVar('disableCreateJPG') || (cfSharedMode('list') && cfRGetVar('efCurrentDirectory')=='*resourceBasePath*')){
		foreach ($thumbnails as $completeFilename=>$destination){
			// Update progress bar
			cfRTScript('tnUpdt("'.$nb.' / '.count($thumbnails).'",'.floor(100*$nb/count($thumbnails)).');');
			// Generate thumbnail (no more cropping on TN as it's done when actually displaying TNs)
			cfCreateResizedJPG($completeFilename,0,0,$destination,$tnSize,$tnSize,DEFAULT_RESIZE_METHOD,$thumbnailJPGQuality,0,0/*cfRGetVar('cropThumbnails')*/);
			$nb++;
		}
	}
	// Use CreateJPG to generate thumbnails
	else{
		// Init efMakePhotoThumbnailsCB function
		efMakePhotoThumbnailsCB(count($thumbnails));
		$cl='"'.cfRGetVar('efCurrentDirectory').'" "'.dirname(reset($thumbnails)).'" '.$tnSize.' '.$tnSize.' images';
		/*if(cfRGetVar('cropThumbnails')) $cl.=' square 100'; else */$cl.=' no 100'; // (no more cropping on TN as it's done when actually displaying TNs)
		$cl.=' '.$thumbnailJPGQuality;

		// Launch createJPG and monitor stdout with efMakePhotoThumbnailsCB
		$output=array('stdout'=>'efMakePhotoThumbnailsCB');
		cfStreamProc('"'.cfAppBinDir().'/createJPG.exe" '.$cl,$output,array('bufferSize'=>10,'haltOnSeq'=>chr(12)));
	}
	cfRTScript('removeNode(dgi("tnMonitoringDiv"))');
	set_time_limit(ini_get('max_execution_time'));
}

/**
 * @desc Return best-suited image as a cover for an audio file, from all images located in directory
 *
 * @param string $completeFilename: audio complete filename
 * @param integer $score: matching criterias score
 * @return string: cover image complete filename
 */
function efGetCoverFromFolder($completeFilename, &$score=0){
	$score=-1;

	if($completeFilename=='*resourceBasePath*') $folder='';
	elseif(is_dir($completeFilename)){
		$filename=false;
		$folder=$completeFilename;
	}
	else{
		$folder=dirname($completeFilename);
		$filename=basename($completeFilename);
	}

	$images=array();
	foreach (cfGlob($folder.'/*') as $cfn) if(in_array(strtolower(cfFileExtension($cfn)),array('jpg','jpeg','gif','png','tiff'))) $images[]=$cfn;
	// Default image
	if(!count($images)) return cfAppDocRoot().'/gfx/cover.png';
	$score=0;
	if(count($images)==1) return $images[0];

	// Browse images to find the best-matching
	$max=0;
	$bestImage='';
	foreach ($images as $imageCompleteFilename){
		$imageFilename=basename($imageCompleteFilename);
		$imageFilenameLC=strtolower($imageFilename);
		$score=0;
		if(strpos($imageFilenameLC,'cover')!==false) $score+=5;
		if(strpos($imageFilenameLC,'front')!==false) $score+=3;
		if(strpos($imageFilenameLC,'back')!==false) $score+=2;
		if(strpos($imageFilenameLC,'.jpg')) $score+=1;
		$imageFilename=str_ireplace('cover','',str_ireplace('front','',cfFileWithoutExtension($imageFilename)));
		similar_text(cfFileWithoutExtension($filename),cfFileWithoutExtension($imageFilename),$perc);
		if($perc>50) $score+=$perc/10;
		if($score>=$max) {
			$max=$score;
			$bestImage=$imageCompleteFilename;
		}
	}
	$score=$max;
	//cfDbg($completeFilename.'=>'.$bestImage.' ('.$score,1);
	return $bestImage;
}

/**
 * @desc search COMMENTS_FILE (located in $thumbnailsPath) for a line starting with "$imageName=" and return the right part of the line
 * @return mixed : false if error or $imageName not found, comments if image found
 * @param string $thumbnailsPath
 * @param string $imageName
*/
function efReadComments($completeFilename){
	$completeFilename=strtolower($completeFilename);
	$imageName=basename($completeFilename);
	$subPath=substr(cfDirName($completeFilename),strlen(cfRGetVar('path'))); // path between album base directory and current image's directory
	$commentsFilePath=cfJoinPathFile(cfAppResourceDir(),$subPath); // Path in which comments text file is stored
	if(!file_exists($commentsFilePath)) return efReadCommentsExif($completeFilename); // If this path doesn't exist, return exif value
	$commentsFileName=cfJoinPathFile($commentsFilePath, COMMENTS_FILE); // Comments text file complete file name
	if(!file_exists($commentsFileName) || !($handle= fopen($commentsFileName,"r"))) return efReadCommentsExif($completeFilename); // If comments file doesn't exist or cannot be opened, return exif value
	if(strlen($imageName)<3) return false;
	if(substr($imageName,0,1)=='/') $imageName=substr($imageName,1);

	while(!feof($handle)){
		$textLine = trim(fgets($handle));
		if(strtolower(substr($textLine,1,strlen($imageName)))==$imageName && substr($textLine,strlen($imageName)+1,1)=='"' && (substr(ltrim(substr($textLine,strlen($imageName)+2)),0,1)=="=")) {fclose($handle); return substr(ltrim(substr($textLine,strlen($imageName)+2)),1);}
	}
	fclose($handle);
	return efReadCommentsExif($completeFilename);
}

/**
 * @desc Return "title" exif property of image (if any) if exif data access allowed, or "" else
 *
 * @param string $completeFilename
 * @return string
 */
function efReadCommentsExif($completeFilename){
	if(!cfRGetVar('showExifData')) return;
	if(!is_array($raw=@exif_read_data($completeFilename))) return '';
	foreach ($raw as $k=>$v) if(strtolower($k)=='title') return $v;
	return '';
}

/**
 * @desc : write user comments on file $completeFilename, in comments text file
 * Comments file is named COMMENTS_FILE and is located in $thumbnailsPath.
 * Written string is "$imageName=$comments"
 *
 * @param string $completeFilename : path and file name of commented file
 * @param string $comments : comments
 * @param bool $append : If append is true, append this line at the end of file, else replace existing line
 * @return boolean : result of operation
 */
function efWriteComments($completeFilename, $comments, $append){
	$fileName=basename($completeFilename);
	$subPath=substr(cfDirName($completeFilename),strlen(cfRGetVar('path'))); // path between album base directory and current image's directory
	$commentsFilePath=cfJoinPathFile(cfAppResourceDir(),$subPath); // Path in which comments text file is stored
	if(!file_exists($commentsFilePath)) if(!mkdir($commentsFilePath)) return false; // If this path doesn't exist, try to create it
	$commentsFileName=cfJoinPathFile($commentsFilePath, COMMENTS_FILE); // Comments text file complete file name

	// If comments file doesn't exist : create it and write line
	if(!file_exists($commentsFileName)){
		if(!($handle= fopen($commentsFileName,"w"))) return false;
		fwrite($handle, '"'.$fileName.'"='.$comments);
		fclose($handle);
		return true;
	}


	// find image's entry in file
	if(!($handle= fopen($commentsFileName,"r"))) return false;
	$lines=array();
	$lines[0]='"'.$fileName.'"='.$comments."\n";
	$nbLines=1; $found=false;
	while(!feof($handle)){
		$textLine = trim(fgets($handle));
		if(!$found && substr($textLine,1,strlen($fileName))==$fileName && substr($textLine,strlen($fileName)+1,1)=='"' && (substr(ltrim(substr($textLine,strlen($fileName)+2)),0,1)=="=")) $found=true;
		elseif(strlen($textLine)>8) $lines[$nbLines++]=$textLine."\n";
	}
	fclose($handle);

	// erase and rewrite file
	if(!($handle= fopen($commentsFileName,"w"))) {unset($lines); return false;}
	for($i=0;$i<$nbLines;$i++) fwrite($handle, $lines[$i]);
	fclose($handle);
	unset($lines);
	return true;
}

/**
 * @desc unlink (send to recycled bin) $passedDir/$passedFile (first checks access rights if force is not set to true)
 * if $passedFile is a non-empty directory, recursively deletes sub files and folders
 *
 * @param string $completeFilename : file or dir name
 * @param boolean $force: true to force file deletion, don't check rights
 * @return boolean : result of unlink
 */
function efUnlink($completeFilename,$force=false){
	// if no general modify and write authorizations, return false
	if(cfGGetVar('generalModifyProtection') || !(cfRGetVar('modifyAllowed') || cfGGetVar('generalWriteProtection') || !(cfRGetVar('writeAllowed')))) return false;
	if(!file_exists($completeFilename) || !cfFileRights($completeFilename,'modify')) return false;
	// Destroy files (unlink)
	if(cfRGetVar('destroyDeletedFiles') || !function_exists('customSendToRecycled')){
		if(is_dir($completeFilename)) return cfUnlinkDir($completeFilename);else return unlink($completeFilename);
	}
	// Send files to recycle bin
	else {
		if(!is_dir($completeFilename)){
			if(!($fp=@fopen($completeFilename,'w'))) return false;
			fclose($fp);
		}
		return customSendToRecycled($completeFilename);
	}
}

/**
 * @desc return a number formated according to local settings
 *
 * @param number $num: number to format
 * @param int $decimal: number of fraction figures
 * @return string
 */
function efNumberFormat($num,$decimal=0){
	if(!isset($_ENV['locale'])){
		setlocale(LC_NUMERIC,cfCaption('_ISO_639'));
		$_ENV['locale']=localeconv();
		if(ord($_ENV['locale']['thousands_sep'])==160) $_ENV['locale']['thousands_sep']=' ';
		setlocale(LC_NUMERIC,'eng');
	}
	return number_format($num,$decimal,$_ENV['locale']['decimal_point'],$_ENV['locale']['thousands_sep']);
}

/**
 * @desc : return dimension of an image ($srcWidth x $srcHeight) resized to fit in $maxWidth x $maxHeight square
 *
 * @param integer $srcWidth
 * @param integer $srcHeight
 * @param integer $maxWidth
 * @param integer $maxHeight
 * @param boolean $noStretch: true not to stretch image if destination larger than source
 * @return array : output width, height
 */
function efGetOutputImageDimensions($srcWidth, $srcHeight, $maxWidth, $maxHeight=0, $noStretch=false){
	if(!$srcHeight ||!$srcWidth) return array($maxWidth,$maxHeight);
	if(!$maxWidth && !$maxHeight) $maxWidth=100;
	$tnMW=$maxWidth;if(!$tnMW) $tnMW=$maxHeight;
	$tnMH=$maxHeight;if(!$tnMH) $tnMH=$maxWidth;
	$tnRatio=$tnMW/$tnMH;$imgRatio=$srcWidth/$srcHeight;
	if($imgRatio>=$tnRatio) $tnMH=floor($tnMW/$imgRatio); else $tnMW=floor($tnMH*$imgRatio);
	if($noStretch && $tnMW>$srcWidth) return array($srcWidth,$srcHeight);
	return array(floor($tnMW),floor($tnMH));
}

/**
 * @desc Update preview image in image gallery explorer
 * Generate HTML page with javascript updating parent frame preview image :
 * src img with id="previewImage" is changed
 * image title with id="previewTitle" is changed
 * image file name with id="previewFileName" is changed
 * next and previous images with id "previewNext" and "previewPrevious"
 *
 * @param string $completeFilename : path and name of preview image
 * @param integer $maxWidth : max output image width (overridden by cfRGetVar('previewMaxWidth') if not cfRGetVar('previewAutoSize')
 * @param integer $maxHeight : max output image height (overridden by cfRGetVar('previewMaxHeight') if not cfRGetVar('previewAutoSize')
 */
function efUpdatePreview($completeFilename, $maxWidth=false, $maxHeight=false){
	// Check file download rights
	$completeFilename=cfCleanPathName($completeFilename);
	$passedFile=basename($completeFilename);
	if(!cfFileRights($completeFilename,'download')) exit;

	// Log in viewed content history
	cfLogContentAccess($completeFilename);

	if($maxWidth && !is_numeric($maxWidth)) exit;
	if(!$maxHeight) $maxHeight=$maxWidth*3/4;

	// Start output
	$js= 'with(preview){';

	// Image file name
	$js.='setFilename("'.cfUTF8Encode(basename($completeFilename)).'");';

	// Preview rotation
	$rotation=cfRGetVar('previewAngle',false,false,null);

	// Exif rotation
	if((cfFileExtension($completeFilename)=='jpg' || cfFileExtension($completeFilename)=='jpeg') && cfGGetVar('EXIFAutoRotation') && $rotation===null){
		$eD=@exif_read_data($completeFilename);

		if($angle=@$eD['Orientation']){
			// If exif data indicates a +-90 rot but height is > width then image has probably been rotated by user but exif data have not been updated
			if(($angle==5 || $angle==6 || $angle==7 || $angle==8) && cfConstant('EXIF_FIX_ROTATION') && isset($eD['COMPUTED']) && @$eD['COMPUTED']['Width']<@$eD['COMPUTED']['Height']) $rotation=0;
			elseif($angle==3) $rotation=180;
			elseif($angle==6 || $angle==7) $rotation=90;
			elseif($angle==5 || $angle==8) $rotation=270;
			cfRSetVar('previewAngle',$rotation);
		}
	}


	// Output image size
	if($maxWidth>0 && $maxHeight>0){
		if(cfFileExtension($completeFilename)=='itc' || cfFileExtension($completeFilename)=='itc2') $width=$height=0;
		else list($width, $height, $type, $attr) = @getimagesize($completeFilename);
		if(!isset($width)) $width=0; if(!isset($height)) $height=0;

		// 90 / 270 rotation: swap width & height
		if(cfRGetVar("previewAngle")==270 || cfRGetVar("previewAngle")==90) cfSwap($width,$height);
		// Compute
		list($w,$h)=efGetOutputImageDimensions($width, $height, $maxWidth, $maxHeight);

		// Stretch / don't stretch small images
		if((!cfRGetVar('stretchSmallImages')) && ($w>$width || $h>$height)) {$w=$width; $h=$height;}

		// Image src
		$js.='updateSrc("'.cfExtImage($completeFilename, $w, $h, true, (int)$rotation).'",'.((int)$w).','.((int)$h).');';
		//cfDbg($js);
	}

	// Image title
	$js.= 'setTitle("'.cfUTF8Encode(cfFileWithoutExtension(str_replace('"','',$passedFile))).'");';

	// Image file name
	$js.= 'filename="'.cfProtectFilename($passedFile).'";';

	$np=efGetNextAndPrevious($passedFile);

	// Next image name
	$js.= 'next="'.cfUTF8Encode($np['next']).'";';

	// Previous image name
	$js.= 'prev="'.cfUTF8Encode($np['previous']).'";';

	// Image comments
	$comments=efReadComments($completeFilename);
	$js.= 'commentsSet("'.cfUTF8Encode(str_replace('"','\\"',$comments),false,false).'");';

	// Image rating
	$rating=efFileGetRating($completeFilename,false,true);
	if(cfRGetVar('ratingAllowed') && cfGGetVar('ratingAllowed')) $js.= 'ratingSet("'.$rating.'");';

	// image current angle
	$js.= 'angle='.(int)$rotation.';';

	$js.='}';

	// Indicate file has been viewed once
	efFileSetViewedOnce($completeFilename);

	// Send async response
	cfAsyncHeader();
	echo cfAsyncXMLJSaction($js);
	cfAsyncFooter();
}

/**
 * @desc Return true if browser has drag-outside-of-the-browser download capability (Chrome 5+ so far)
 *
 * @return boolean
 */
function efHasDragDownload(){return strpos($_SERVER['HTTP_USER_AGENT'],'Chrome/');}

/*
***************************************************************************************************************************
*
* efFiles files array functions
*
***************************************************************************************************************************
*/

/**
 * @desc (re)init explorer data
 *
 */
function efInit(){

	// If configuration updated or resource not initalized, reset config
	if((!cfRGetVar('explorerInitialized')) || cfRGetVar('memUpdate')!=cfRGetVar('sessionUpdate')){

		cfRInitVar('actionsPanel',true);

		// Remove previous databases if present
		if(is_file(cfAppDataDir().'/audioCacheV2.db')) @unlink(cfAppDataDir().'/audioCacheV2.db');
		if(is_file(cfAppDataDir().'/audioCacheV3.db')) @unlink(cfAppDataDir().'/audioCacheV3.db');
		if(is_file(cfAppDataDir().'/audioCacheV4.db')) @unlink(cfAppDataDir().'/audioCacheV4.db');

		// array of extensions of displayed files. Set to false if no filter on file extension
		cfRInitVar('extensionFilter',false);
		// As extensionFilter is read from .ini file, turn it into an array
		if(cfRGetVar('extensionFilter') && !is_array(cfRGetVar('extensionFilter'))){
			$ext=array();
			foreach (explode(',',cfRGetVar('extensionFilter')) as $v) $ext[]=trim($v);
			cfRSetVar('extensionFilter',$ext);
			unset($ext);
		}

		if(cfSharedMode('list')){
			// Build filename=>"d" || "f" array
			if(1 || !is_array(cfRGetVar('sharedItems'))){
				cfChronoAddPoint();
				cfRSetVar('treeViewAvailable',false);
				$sharedItems=array();
				foreach (explode('|',cfRGetVar('sharedItems')) as $v)
					$sharedItems[$v]=((is_dir($v))?'d':'f');

				foreach ($sharedItems as $type) if($type=='d') {
					cfRSetVar('treeViewAvailable','mainTree');
					break;
				}

				cfRSetVar('sharedItemsList',$sharedItems);
				cfRSetVar('subFoldersIncluded',true);
			}
			// Reset directory
			if(!cfRGetVar('efCurrentDirectory') || !cfFileRights(cfRGetVar('efCurrentDirectory'),'read')) cfRSetVar('efCurrentDirectory','*resourceBasePath*');

			// If it's a list, and there is a single directory, directly go in this directory
			if(cfRGetVar('efCurrentDirectory')=='*resourceBasePath*'){
				$nbFiles=0;
				$nbDirs=0;

				foreach ($sharedItems as $name=>$type) {
					if($type=='d') $nbDirs++;
					if($type=='f') $nbFiles++;
				}
				if($nbDirs==1 && $nbFiles==0) cfRSetVar('efCurrentDirectory','*resourceBasePath*/'.basename($name));
			}
		}
		else{
			if(!cfRGetVar('efCurrentDirectory') || !cfFileRights(cfRGetVar('efCurrentDirectory'),'read')) cfRSetVar('efCurrentDirectory',cfCleanPathName(cfRGetVar('path')));
			if(!cfRGetVar('efCurrentDirectory')) {$errorMessageCritical=cfCaption('explorerErrorNoValidDir'); return false;}

			// Treeview available
			if(cfRGetVar('subFoldersIncluded')) cfRSetVar('treeViewAvailable','mainTree'); else cfRSetVar('treeViewAvailable',false);
		}

		// Common parameters
		cfRInitVar('searchFiles',true); 					// Default: enable file search
		cfRInitVar('searchFilesContentAllowed',false);	 	// Default: disable file content search for resources
		cfRInitVar('searchFilesContentDefault',false); 		// Default: if file content search is allowed, disable by default
		cfRInitVar('filesLayout','resource'); 				// 'list', 'icons', 'resource'
		cfRInitVar('thumbnailsSize',240); 					// 0: small, 1: medium, 2: big
		cfRInitVar('thumbnailsSizeFactor',1); 				// 0: small, 1: medium, 2: big

		cfRInitVar('itemsPerPage',50);						// thumbnails per page - 0: unlimited, 10, 20, 50

		cfRInitVar('bittorrentEnabled',true);				// Allow bittorrent download, except if disabled at general level

		// Photo explorer default items per page, per layout (layout 3 displays everything)
		cfRInitVar('itemsPerPageLayout1',20); 				// Layout 1
		cfRInitVar('itemsPerPageLayout2',50); 				// Layout 2

		// Video offset for thumbnail generation
		cfRInitVar('videoThumbnailOffset',cfConstant('VIDEO_THUMBNAIL_OFFSET'));

		// Wii: fix flash video chunk length to 4mn, as Wii freeze after 5mn
		if(cfIsWII()) cfRSetVar('flashChunkLength',120);


		// Browser specific parameters
		if(!cfBGetVar('download'))	{
			cfRSetVar('multipleDownloadAllowed',false);
			cfRSetVar('downloadShortcut',false);
			cfRSetVar('uploadAllowed',false);
			cfRSetVar('displayMultipleDownload',false);
		}
		cfRSetVar('explorerInitialized',true);
		cfRSetVar('sessionUpdate',cfRGetVar('memUpdate'));

		// Set user-level defined rights
		cfRInitUserRightsVar('downloadAllowed');
		cfRInitUserRightsVar('multipleDownloadAllowed');
		cfRInitUserRightsVar('writeAllowed');
		cfRInitUserRightsVar('modifyAllowed');
		cfRInitUserRightsVar('editionAllowed');
		cfRInitUserRightsVar('createFolderAllowed');
		cfRInitUserRightsVar('shareAllowed');
		cfRInitUserRightsVar('remoteDownloadAllowed');
		cfRInitUserRightsVar('mailoutAllowed');
		cfRInitUserRightsVar('ratingAllowed');

		cfRInitUserRightsVar('executeAllowed');
		cfRInitUserRightsVar('uploadAllowed');

		cfRInitUserRightsVar('fileViewAllowed');
		cfRInitUserRightsVar('downloadShortcut');

		cfRInitUserRightsVar('showExifData');
		cfRInitUserRightsVar('commentsAllowed');

		// Video
		cfRInitUserRightsVar('allowOriginalVideo');

		// Ensure all read/write rights are coherent
		if(!cfRGetVar('writeAllowed')){
			cfRSetVar('modifyAllowed',false);
			cfRSetVar('editionAllowed',false);
			cfRSetVar('uploadAllowed',false);
			cfRSetVar('createFolderAllowed',false);
		}
		if(!cfRGetVar('modifyAllowed')) cfRSetVar('editionAllowed',false);
		if(!cfRGetVar('downloadAllowed')){
			cfRSetVar('multipleDownloadAllowed',false);
			cfRSetVar('editionAllowed',false);
			cfRSetVar('downloadShortcut',false);
		}
	}
	//cfDbg(cfRGetVar('userRights_fileViewAllowed'));
	// Define resource-vars that will be saved if user is singleUser
	cfSetSavedRVar('efCurrentDirectory',
		'efSortBy',
		'efSortByPrev',
		'efSortOrder',
		'efSortOrderPrev',
		'actionsPanel',
		'displayTransfers',
		'treeViewDisplayed',
		'sortField',
		'scrollMainFileTable',
		'displayMultipleDownload',
		'layout',
		'filesLayout',
		'thumbnailsSizeFactor',
		'combinedLayout',
		'thumbnailsIndex',
		'itemsPerPage',
		'itemsPerPagePos',
		'searchFilesContentDefault',
		'CLSShow');
}

/**
 * @desc Return an array containing files and subdirectories of a given directory
 * array also contains properties of files
 *
 * @param string $dir : directory
 * @param bool $includeSubDirs : if true, include subdirectories of $dir
 * @param string $extensionFilter : array of extensions that will be listed. Files with an extension not matching those criterias are excluded
 * @param array $fileTypeFilter : array of extensions that will be listed. Files with a type not matching those criterias are excluded
 * @param bool $sort: if true, sort array using cfRGetVar('efSortBy') and cfRGetVar('efSortOrder') parameters
 * @param  array $options: extra options:
 * 		   'includeRights' : set this option to include file rights to file data (used not to call cfFileRights twice in explorer/std)
 * 		   'noExtra': set this option not to get file's extra info
 * 		   'iconSize': exe/ico extracted icon size (16,32)
 * 		   'noRightsCheck': set to true not to check files rights
 * @return array : array of files and subdirs (see description inside function)
 */
function efGetDirContent($dir, $includeSubDirs=true, $extensionFilter=false, $fileTypeFilter=false,$sort=false,$options=false){
	global $errorMessage;
	global $errorMessageCritical;
	// returned array :

	// if a critical error message has already been raised, exit
	if($errorMessageCritical) return false;

	// Common fields (files, dirs, drives)
	// effDir[fileNumber]['name'] : file name
	// effDir[fileNumber]['type']  : file type ('dir', 'file')
	// effDir[fileNumber]['rights']  (if options set 'includeRights') : file rights (from cfFileRights function)

	// Drives only fields
	// effDir[fileNumber]['freeSpace'] : file size (bytes)
	// effDir[fileNumber]['totalSpace'] : file size (bytes)

	// Files and dirs fields
	// effDir[fileNumber]['date'] : file last modification date

	//Files only fields
	// effDir[fileNumber]['size'] : file size (bytes)
	// effDir[fileNumber]['extra1'] : default extra file info displayed in std explorer
	// effDir[fileNumber]['subType']  : file type ('image', 'music', ...)

	$iconSize=isset($options['iconSize'])?$options['iconSize']:((efFilesListType()=='icons')?32:16);

	if($fileTypeFilter && !is_array($fileTypeFilter)) $fileTypeFilter=array($fileTypeFilter);

	// clean $dir
	$dir=cfCleanPathName($dir);

	// if access to $dir is not authorized to logged user
	if(!($dir) || (cfFileRights($dir,'state')!='authorized' && !@$options['noRightsCheck'])) {
		// If forbidden directory, try to reset to base path (might happend when base path is changed while user is connected&browsing)
		$dir=((cfSharedMode('list'))?'*resourceBasePath*':cfRGetVar('path'));
		cfRSetVar('efCurrentDirectory',$dir);
		if(!($dir) || cfFileRights($dir,'state')!='authorized') return false;
	}

	// Include or not complete file rights
	if(isset($options['includeRights']) && $options['includeRights']) $includeRights=true; else $includeRights=false;

	$efFiles=array();
	$nbFiles=0;	$nbSubDirs=0; $n=0;
	$efFiles['dir']=$dir;

	// If $dir is computerRoot
	if($dir=='computerRoot'){
		if(WEnv::host()->isWindows()){
			// Browse drives
			for ($i=66;$i<91;$i++) {
				$driveName=chr($i).":/";
				if (is_dir($driveName)){
					$efFiles[$nbSubDirs]=efDriveInfo($driveName);
					$nbSubDirs++;
				}
			}
			$efFiles['nbSubDirs']=$nbSubDirs;$efFiles['nbFiles']=0;$efFiles['nbTotal']=$nbSubDirs;

			return $efFiles;
		}
		// Linux / OSX
		$dir='/';
	}
	// Else, if dir is not computer root

	// var caching
	$folderIcon=outIcon('fi/folder');
	$fileIcons=array();
	$useFilesCache=false; // Set to true to use data/filesCache.db for file information cache

	if (substr($dir,-1)!="/") $dir=$dir.'/';

	// Look for shared files and dirs
	$files=array();

	if(cfSharedMode('list') && $dir=='*resourceBasePath*/'){
		foreach (cfSharedItems() as $cfn=>$type) $files[]=$cfn;
	}
	else{
		if(($handle = @opendir($dir))==false) 	return false;
		while (($file = @readdir($handle))!==false) if($file !== "." && $file !== "..") $files[]=$dir.$file;
		closedir($handle);
	}

	// Get files cache data if needed
	if(1 /*!@$options['noExtra'] && cfGGetVar('cacheFilesData')*/){

		// Add meta data stored in meta data database into "meta" property of files
		$filesCache=efFileMetaGet($files,false);

		$useFilesCache=true;
	}


	// Browse found files & dirs
	foreach ($files as $completeFilename){
		$n=$nbSubDirs+$nbFiles;
		$file=basename($completeFilename);

		// Get file access rights
		if($includeRights) {
			$rights=cfFileRights($completeFilename,false,false);
			$state=$rights['state'];
		}
		else $state=cfFileRights($completeFilename,'state',false);

		// SUBDIRECTORY
		if(is_dir($completeFilename)){
			if ($includeSubDirs && ($state=='authorized' || !cfGGetVar('generalHideUnallowedFiles'))) {
				$efFiles[$n]=array('name'=>$file,'completeFileName'=>$file,'size'=>0,'type'=>'dir','icon'=>$folderIcon,'date'=>@filemtime($completeFilename));
				if($includeRights) $efFiles[$n]['rights']=$rights;
				$nbSubDirs++;
			}
		}
		//FILE
		else {
			$fileExt=strtolower(substr($file,strrpos($file,'.')+1));
			if(
				((!$fileTypeFilter && !$extensionFilter)
				|| (is_array($fileTypeFilter) && !(in_array(efFileType($file), $fileTypeFilter)===false))
				|| ($extensionFilter && !(array_search($fileExt, $extensionFilter)===false))
				)
					&& ($state=='authorized' || !cfGGetVar('generalHideUnallowedFiles'))
				){
				// Get icon
				if($fileExt=='exe'||$fileExt=='ico'||!array_key_exists($fileExt,$fileIcons)){
					$fileIcons[$fileExt]=efIcon($completeFilename,isset($options['noExeIcon'])?false:'exe',$iconSize);
				}

				// Main properties
				$size=cfFileSize($completeFilename);
				$efFiles[$n]=array('name'=>$file,'completeFileName'=>$completeFilename,'size'=>$size,'type'=>'file','icon'=>$fileIcons[$fileExt],'date'=>@filemtime($completeFilename),'extra1'=>'');
				if($includeRights) $efFiles[$n]['rights']=$rights;

				// Rating
				if(isset($filesCache[$completeFilename]['rating'])) $efFiles[$n]['rating']=efFileGetRating($completeFilename,$filesCache[$completeFilename]['rating'],true);
				else $efFiles[$n]['rating']='0000';
				//else $efFiles[$n]['rating']=array('score'=>'00','hasVoted'=>0,'nb'=>'0');

				// Number of times a file has been viewed
				$efFiles[$n]['viewed']=@$filesCache[$completeFilename]['viewed'];

				$st=$efFiles[$n]['subType']=efFileType($file);

				//Set file specific extra data
				if(!isset($options['noExtra']) || !$options['noExtra']) {
					$cacheNeedsUpdate=false;

					// If file is in cache and is not outdated, use cache
					if($useFilesCache && isset($filesCache[$completeFilename]['mediaInfo']) && $filesCache[$completeFilename]['mediaInfo']['mtime']==$efFiles[$n]['date']){
						$efFiles[$n]+=$filesCache[$completeFilename]['mediaInfo']['data'];
					}
					else switch($st){// Get file type (i.e image, text, music...)
						// image file
						case "image":
							if($fileExt=='itc' || $fileExt=='itc2' || $fileExt=='ico') $width=$height=0;
							else {
								// Extract dimensions from files
								list($width, $height, $type, $attr) = @getimagesize($completeFilename);
								$extra['extra1']=$width.'x'.$height;
								$extra['width']=$width;
								$extra['height']=$height;
								$efFiles[$n]+=$extra;

								$cacheNeedsUpdate=1;
							}
							break;
						// audio file
						case "audio":
							if(cfRGetVar('needAudioInfo')){
								$id3=efGetAudioInfo($completeFilename);
								if($id3 && isset($id3['formatedBitrate'])) $efFiles[$nbFiles+$nbSubDirs]['extra1']=$id3['formatedBitrate']; else $efFiles[$nbFiles+$nbSubDirs]['extra1']="";
								if(isset($id3['bitrate'])) $extra['bitrate']=$id3['bitrate'];
								if(isset($id3['artist'])) $extra['artist']=$id3['artist'];
								if(isset($id3['album'])) $extra['album']=$id3['album'];
								if(isset($id3['track'])) $extra['track']=$id3['track'];
								if(isset($id3['title'])) $extra['title']=$id3['title'];
								if(isset($id3['year'])) $extra['year']=$id3['year'];
								if(isset($id3['genre'])) $extra['genre']=$id3['genre'];
								$cacheNeedsUpdate=1;
								$efFiles[$n]+=$extra;
							}
							break;
							// video file
						case 'video':

							// other : no extra data
						default:
					}

					// If new meta data extracted from file, update meta data database
					if($cacheNeedsUpdate){
						if(isset($filesCache[$file])) $meta=$filesCache[$file]; else $meta=array();
						$meta['mediaInfo']=array('mtime'=>$efFiles[$n]['date'],'data'=>$extra);
						efFileMetaSet($completeFilename,false,$meta);
					}
				}
				$nbFiles++;
			}
		}
		cfPingUpdate(100);
	}

	$efFiles['nbSubDirs']=$nbSubDirs;$efFiles['nbFiles']=$nbFiles;$efFiles['nbTotal']=$n;

	// SORT FILES AND SUBDIRECTORIES LIST
	if($sort && cfRGetVar('efSortBy') && cfRGetVar('efSortOrder')) efDirSort($efFiles, cfRGetVar('efSortBy'), cfRGetVar('efSortOrder'));

	return $efFiles;
}

/**
 * @desc Sort a files Array on name, size or date
 *
 * @param array $filesArray : array of files
 * @param string $sortBy : primary field to use for sorting : 'name', 'size' or 'date'
 * @param string $sortOrder : ascending or descending sort ('asc' or 'desc')
 * @param string $sortBy2 : secundary field to use for sorting : 'name', 'size' or 'date'
 * @param string $sortOrder2 : secundary sort order ('asc' or 'desc')
 */
function efDirSort(&$filesArray, $sortBy='name', $sortOrder='asc', $sortBy2='name', $sortOrder2='asc'){
	global $globalSortBy, $globalSortOrder, $globalSortBy2, $globalSortOrder2;

	if(!function_exists('efFileSortCompare')){
		/**
		 * @desc "sub function" of efDirSort
		 *
		 * @param array $a
		 * @param array $b
		 * @return integer -1 / 0 / +1 : result of comparison
		 */
		function efFileSortCompare($a,$b){
			global $globalSortBy, $globalSortOrder, $globalSortBy2, $globalSortOrder2;

//cfDbg(@$a['rating'].'<>'.@$b['rating'].'=>'.efFileSortCompareRatings(@$a['rating'],@$b['rating'],'desc'),'');
			// Non file/dir: set result to 0 to "exclude" from sorting
			if(!isset($a['type']) || !isset($b['type'])) return 0;

			if(!isset($a['rating'])) $a['rating']='0000';
			if(!isset($b['rating'])) $b['rating']='0000';

			// Equality on 1st sort criteria: use 2nd
			if (!isset($a[$globalSortBy]) || !isset($b[$globalSortBy]) || ($globalSortBy=='rating' && substr($a['rating'],0,2)==substr($b['rating'],0,2)) || $a[$globalSortBy] == $b[$globalSortBy]) {
				if($globalSortBy2=='rating') return efFileSortCompareRatings(@$a['rating'],@$b['rating'],$globalSortOrder2);
				if ($a[$globalSortBy2] == $b[$globalSortBy2]) return 0;
				if($globalSortOrder2=='asc') return (strtolower($a[$globalSortBy2]) < strtolower($b[$globalSortBy2])) ? -1 : 1;
				else return (strtolower($a[$globalSortBy2]) < strtolower($b[$globalSortBy2])) ? 1 : -1;
			}

			// Sort on main criteria
			if($globalSortBy=='rating') return efFileSortCompareRatings($a['rating'],$b['rating'],$globalSortOrder);
			if($globalSortOrder=='asc') return (strtolower($a[$globalSortBy]) < strtolower($b[$globalSortBy])) ? -1 : 1;
			else return (strtolower($a[$globalSortBy]) < strtolower($b[$globalSortBy])) ? 1 : -1;
		}
		// Compare 2 ratings
		function efFileSortCompareRatings($a,$b,$sortOrder){
			if($a==='0000' && $b==='0000') return 0;
			if($a==='0000') return ($sortOrder=='asc')?-1:1;
			if($b==='0000') return ($sortOrder=='asc')?1:-1;
			$a=1*substr($a,0,2)+0.001*substr($a,3);
			$b=1*substr($b,0,2)+0.001*substr($b,3);

			if($a==$b) $res=0;
			elseif(($a>$b && $sortOrder=='asc') || ($a<$b && $sortOrder=='desc')) $res=1;
			else $res=-1;

			return $res;
		}
	}


	$globalSortBy="";
	$globalSortOrder="";

	// Secondary sort criteria
	if(cfRGetVar('efSortByPrev')) $globalSortOrder2=cfRGetVar('efSortByPrev'); else $globalSortBy2='name';
	if(cfRGetVar('efSortOrderPrev')) $globalSortOrder2=cfRGetVar('efSortOrderPrev'); else $globalSortOrder2='asc';

	$globalSortBy=$sortBy; $globalSortOrder=$sortOrder; $globalSortBy2=$sortBy2; $globalSortOrder2=$sortOrder2;
	uasort($filesArray, "efFileSortCompare");
}

/**
 * @desc Get number of files in a files array (excludes drives and folders)
 *
 * @param array $filesArray : array of files
 * @return integer : number of files
 */
function efDirGetNbOfFiles($filesArray){
	if(!is_array($filesArray)) return 0;
	$nb=0;
	foreach ($filesArray as $value) if(isset($value['type']) && $value['type']=='file') $nb++;
	return $nb;
}

/**
 * @desc Get number of subDirs in a files array
 *
 * @param array $filesArray : array of files
 * @return integer : number of subdirectories
 */
function efDirGetNbOfSubDirs($filesArray){
	$nb=0;
	foreach ($filesArray as $value ) if(isset($value['type']) && $value['type']!='file') $nb++;
	return $nb;
}

/**
 * @desc return key index of $fileNumber's $fileType of $filesArray array
 *
 * @param array $filesArray
 * @param number of wanted file $fileNumber
 * @param string $fileType : type of file
 * @return string : array's key of wanted file
 */
function efDirGetKeyFromFileNumber($filesArray, $fileNumber, $fileType){
	$nb=0;
	foreach ($filesArray as $key => $value){
		if($value['type']==$fileType) if($fileNumber==$nb) return $key; else $nb++;
	}
}

/**
 * @desc Generate a m3u file from $completeFileNames and send it to browser
 *
 * @param string $completeFileNames : name(s) of file(s) generating playlist:
 * 			$completeFileNames can be an array of file names, a single audio file name or a m3u file name
 * 			exit() at the end of script
 *
 */
function efSendM3UFile($completeFileNames){

	$m3uFileContent="#EXTM3U\n";
	// if input is *playlist*.m3u (music resource internal playlist), convert it to playlist's songs array
	if($completeFileNames=="*playlist*.m3u"){
		if(cfRGetVar('shufflePlaylist')) $playlist=cfRGetVar('playlistShuffled'); else $playlist=cfRGetVar('playlist');
		foreach ($playlist as $value){
			if(cfRGetVar('outputBitRate')!='original') $externalFileName=cfExtAudio(($value['completeFileName']), "audio/".cfRGetVar('outputBitRate')."/",'absolute');
			else $externalFileName=cfExtAudio($value['completeFileName'],'','absolute');

			if(is_file($value['completeFileName']))	$id3=efGetAudioInfo($value['completeFileName']);
			elseif (cfRGetVar('extAccessFunction')){
				$extAccessFunction = create_function('$completeFilename', cfRGetVar('extAccessFunction'));
				if(($value['completeFileName']=$extAccessFunction($value['completeFileName']))) $id3=efGetAudioInfo($value['completeFileName']);
			}

			if(!isset($value['title'])) $value['title']=$value['name'];
			$value['label']=cfUTF8Encode($value['label']);
			if(isset($id3['playtime_seconds']))
			$m3uFileContent.="#EXTINF:".(int)$id3['playtime_seconds'].','.$value['label']."\n";
			else
			$m3uFileContent.="#EXTINF:,".$value['label']."\n";
			$m3uFileContent.=$externalFileName."\n";
		}

		$m3uFileName='playlist.m3u';
	}
	//
	// M3U
	// if $completeFileNames is the name of a m3u files, extract links and convert it into an array of audio file names
	//
	elseif(!is_array($completeFileNames) && cfFileExtension($completeFileNames)=='m3u'){
		$m3uFileName=$completeFileNames;
		if(($handle = fopen ($m3uFileName, "r"))){
			while (!feof ($handle)) {
				$m3uFile = trim(fgets($handle, 4096));
				if(substr($m3uFile,0,1)!="#"){
					$completeFilename=cfDirName($m3uFileName).'/'.$m3uFile;
					if(file_exists($completeFilename) &&  efFileType($completeFilename)=='audio'){
						if(cfRGetVar('outputBitRate')!='original') $externalFileName=cfExtAudio($completeFilename, "audio/".cfRGetVar('outputBitRate')."/");
						else $externalFileName=cfExtAudio($completeFilename);
						$id3=efGetAudioInfo($completeFilename);
						if(isset($id3['playtime_seconds']))
						$m3uFileContent.="#EXTINF:".(int)$id3['playtime_seconds'].','.basename($completeFilename)."\n";
						else
						$m3uFileContent.="#EXTINF:,".basename($completeFilename)."\n";
						$m3uFileContent.=cfGGetVar('hostName').$externalFileName."\n";
					}
				}
			}
			fclose ($handle);
		}
	}
	//
	// Single audio file name
	//
	else{
		if(cfRGetVar('outputBitRate')!='original') $externalFileName=cfExtAudio(($completeFileNames), "audio/".cfRGetVar('outputBitRate')."/",'absolute');
		else $externalFileName=cfExtAudio(($completeFileNames),'','absolute');
		$id3=efGetAudioInfo($completeFileNames);
		$m3uFileContent.="#EXTINF:".((isset($id3['playtime_seconds']))?(int)$id3['playtime_seconds']:'').','.basename($completeFileNames)."\n";
		$m3uFileContent.=$externalFileName."\n";

		$m3uFileName=cfFileWithoutExtension(basename($completeFileNames)).'.m3u';
	}

	// Send generated m3u file to output
	header('Content-Type:audio/x-mpegurl');
	header('Content-Description: file Transfert');
	header('Content-Disposition: attachment; filename="'.cfUTF8Encode($m3uFileName)).'"';
	header('Content-Length: '.strlen($m3uFileContent));
	header("Cache-Control: must-revalidate, post-check=0, pre-check=0, public");
	header("Pragma:"); // Do not remove, required by IE in SSL mode
	header("Expires: 0");
	echo $m3uFileContent;
	exit();
}

/**
 * @desc Update $_ENV['weezoResEfFiles'] array with content of "cfRGetVar('efCurrentDirectory')" directory
 * use cfRGetVar('subFoldersIncluded') for sub directories inclusion
 * use cfRGetVar('efExtensionFilter') for filtering on files extension
 * use cfRGetVar('efFileTypeFilter') for filtering on files type
 *
 * @param  array $options: extra options:
 * 		   'includeRights' : set this option to include file rights to file data (used not to call cfFileRights twice in explorer/std)
 * 		   'noThumbnails' : set this option to explcitly not generate photo/video thumbnails
 * 		   'noExtra' : set this option not to get file's extra info
 *
 * @return true if OK, false if NOK. If not OK, global variable $errorMessageCritical is set
 */
function efDirUpdate($options=false){
	global $errorMessage;
	global $errorMessageCritical;

	$_ENV['weezoResEfFiles']=array('nbFiles'=>0,'nbSubDirs'=>0);

	// Init multiple download data
	if(!is_array(cfRGetVar('mDownloadList'))) cfRSetVarArray('mDownloadList');
	if(!is_array(cfRGetVar('mDownloadTotals'))) cfRSetVar('mDownloadTotals',array(0,0));

	// If a critical error has been raised, don't process
	if($errorMessageCritical) return false;

	// (Re)set current directory if not OK
	if(!cfRGetVar('efCurrentDirectory') || cfFileRights(cfRGetVar('efCurrentDirectory'),'state')!=='authorized'){
		if(cfSharedMode('list')) cfRSetVar('efCurrentDirectory','*resourceBasePath*');
		else cfRSetVar('efCurrentDirectory',cfCleanPathName(cfRGetVar('path')));
		if(!cfRGetVar('efCurrentDirectory')) {$errorMessageCritical=cfCaption('explorerErrorNoValidDir'); return false;}
	}

	/**
	 * Get sorted files and subfolders array
	 */
	$_ENV['weezoResEfFiles']=efGetDirContent(cfRGetVar('efCurrentDirectory'),cfRGetVar('subFoldersIncluded'),cfRGetVar('extensionFilter'),cfRGetVar('fileTypeFilter'),true,$options);

	if(!$_ENV['weezoResEfFiles']) {
		$_ENV['weezoResEfFiles']=array('nbFiles'=>0,'nbSubDirs'=>0);
		return false;
	}

	// Set thumbnails JPG quality
	$thumbnailJPGQuality=((cfRGetVar('thumbnailJPGQuality'))?cfRGetVar('thumbnailJPGQuality'):((cfGGetVar('thumbnailJPGQuality'))?cfGGetVar('thumbnailJPGQuality'):DEFAULT_JPG_QUALITY));


	// thumbnails path & size
	$thumbnailsPath=efThumbnailsPath();
	$tnS=cfRGetVar('thumbnailsSize');

	// Photo thumbnails
	if(cfRGetVar('photoThumbnailsNeeded') && cfRGetVar('thumbnailsSize') && !isset($options['noThumbnails'])){
		// Browse images
		$missingThumbnails=0;
		foreach($_ENV['weezoResEfFiles'] as $key => $value ) if(isset($value['type']) && $value['type']=='file' && $value['subType']=='image'){
			$fileExt=cfFileExtension($value['completeFileName']);
			$_ENV['weezoResEfFiles'][$key]['thumbnail']=efThumbnailFilePath($value['completeFileName']);

			// Inexisting thumbnail
			if(!file_exists($_ENV['weezoResEfFiles'][$key]['thumbnail'])){
				// ITC/ITC2 files (apple) : generate thumbnail with PHP
				if($fileExt=='itc' || $fileExt=='itc2')	cfCreateResizedJPG($value['completeFileName'],0,0,$_ENV['weezoResEfFiles'][$key]['thumbnail'],$tnS,$tnS);
				// Thumbnail generation monitored: add to list processed by efInsertExplorerHeadAndBody
				elseif(cfRGetVar('monitorThumbnailGen'))
					$_ENV['thumbnailPhotoNeeded'][$value['completeFileName']]=$thumbnailsPath.'/'.$tnS.$value['name'].'.jpg';
				// PHP-generate thumbnail
				elseif(cfGGetVar('disableCreateJPG')){
					cfCreateResizedJPG($value['completeFileName'],0,0,$_ENV['weezoResEfFiles'][$key]['thumbnail'],$tnS,$tnS,DEFAULT_RESIZE_METHOD,false,0,false /*cfRGetVar('cropThumbnails')*/);
				}
				// Generate thumbnail through createJPG
				else $missingThumbnails++;
			}
		}

		// createJPG-generate thumbnails if needed
		if($missingThumbnails){
			$cl='"'.cfRGetVar('efCurrentDirectory').'" "'.$thumbnailsPath.'" '.$tnS.' '.$tnS.' images';
			/*if(cfRGetVar('cropThumbnails')) $cl.=' square 100'; else*/ $cl.=' no 100';
			$cl.=' '.$thumbnailJPGQuality;
			cfExecNoFork(cfAppBinDir().'/createJPG.exe','SW_HIDE',$cl); // Generate thumbnails
		}
	}


	// Video thumbnails
	$_ENV['thumbnailVideoNeeded']=array();
	if(cfRGetVar('videoThumbnailsNeeded') && !isset($options['noThumbnails'])){
		// Browse files, check if thumbnail exists, and if it doesn't, create it
		foreach ($_ENV['weezoResEfFiles'] as $key=>$value) if(isset($value['completeFileName']) && is_file($value['completeFileName']) && $value['subType']=='video'){

			$_ENV['weezoResEfFiles'][$key]['thumbnail']=efThumbnailFilePath($value['completeFileName']);

			if(!file_exists($_ENV['weezoResEfFiles'][$key]['thumbnail'])) {
				// Generate thumbnail
				if(!cfRGetVar('monitorThumbnailGen')){
					efMakeVideoThumbnail(
					$value['completeFileName'],
					$_ENV['weezoResEfFiles'][$key]['thumbnail'],
					$tnS,
					$tnS,
					$thumbnailJPGQuality
					);
					cfPingUpdate(10); // Inform server than client is still alive
				}
				// Mem that thumbnail has to be generated (generation called in efInsertExplorerHeadAndBody)
				else $_ENV['thumbnailVideoNeeded'][$value['completeFileName']]=$_ENV['weezoResEfFiles'][$key]['thumbnail'];
			}
		}
	}

	// Store nb files & subdirs (to be used on next request by efProcessPost)
	cfRSetVar('efFilesStats',array('nbFiles'=>$_ENV['weezoResEfFiles']['nbFiles'],'nbSubDirs'=>$_ENV['weezoResEfFiles']['nbSubDirs']));

	return true;
}

/**
 * @desc Return an array with same properties that array generated by efDirUpdate
 *
 * @param string $cfn: file complete filename
 * @return array
 */
function efFileProperties($cfn,$unsafeDir=true){
	$prop['name']=basename($cfn);
	$prop['completeFileName']=$cfn;
	$prop['size']=cfFileSize($cfn);
	$prop['type']=(is_file($cfn)?'file':'dir');
	$prop['icon']=efIcon($cfn,true);
	$prop['date']=@filemtime($cfn);
	$prop['rights']=cfFileRights($cfn,false,$unsafeDir);
	$prop['subType']=efFileType($cfn);
	$prop['extra1']=false;
	return $prop;
}

/**
 * @desc Return a label for a given directory, removing resourceBasePath, and setting caption for computerRoot
 *
 * @param string $dir
 * @return string
 */
function efDirLabel($dir=false){
	if(!$dir) $dir=cfRGetVar('efCurrentDirectory');
	if($dir=='computerRoot') {
		if(cfGGetVar('OSFamily')=='windows') return cfCaption('explorerComputerRoot');
		else return '/';
	}
	$bp=cfSharedMode('list')?'':basename(cfRGetVar('path'));
	$dir=str_replace('*resourceBasePath*',$bp,cfResourceRelativePath($dir));

	if(!$dir) $dir='/';
	return $dir;
}

/**
 * @desc return JS array item corresponding to a file description
 *
 * @param integer $id: id of item
 * @param array $value: properties of file
 * @param string $options:
 * 				'relativeDir': display file's name & path relatively to this directory, 'mediumIcon': use medium sized file icon (from /gfx/fi/med),
 * 				'thumbnail'=>thumbnail width: include thumbnails url
 * 				'includeType'=>1 to include mime (sub)type
 * 				'crop'=>1 to make croped square thumbnails
 * @return string
 */
function efFilePropertiesToJSArray($id,$value,$options=false){
	// JS Array format :
	//
	// 00 - id,
	// 01 - "d" for directories or "f" for files,
	// 02 - name,
	// 03 - draggable (0 or 1),
	// 04 - receiver (0 or 1),
	// 05 - icon path, (for directories : 0 for folder icon, 1 for locked folder icon)
	// 06 - editable (0 or 1), (0 for directories)
	// 07 - downloadable (0 forbidden, 1 ok without shortcut, 2 ok+shortcut), (0 for directories)
	// 08 - mdownloadable (0 or 1), (0 for directories)
	// 09 - file size,
	// 10 - modified date,
	// 11 - extra info,
	// 12 - unlinkable (0 or 1),
	// 13 - renamable (0 or 1),
	// 14 - excutable (0 or 1),
	// 15 - (files only) rating XXYZZ: XX is ratingx10, Y is 1 if user has voted or 0 if not, ZZ is the number of users who have voted. Rating is set to "-" if no one has voted
	// 16 - (files only) number of view

	// 17 - thumbnail URL (optional)
	// 18 - thumbnail w (optional)
	// 19 - thumbnail h (optional)
	// 20 - 1st letter of file mime (sub)type

	// Cache data
	static $downloadShortcut;
	static $multipleDownloadAllowed;
	static $editionAllowed;
	static $modifyAllowed;
	static $downloadShortcut;

	if(!isset($downloadShortcut)){
		$multipleDownloadAllowed=cfRGetVar('multipleDownloadAllowed');
		$editionAllowed=cfRGetVar('editionAllowed');
		$modifyAllowed=cfRGetVar('modifyAllowed');
		$downloadShortcut=cfRGetVar('downloadShortcut');
	}

	// No rights set (everything but File1 explorer): set download-only rights
	if(!isset($value['rights'])) $value['rights']=array('state'=>'authorized','download'=>1,'modify'=>0,'write'=>0,'execute'=>0);

	static $dateFormat=0;	if(!$dateFormat) $dateFormat=cfCaption('_DATE_FORMAT');

	// Set most used icons
	static $ico_noX=0; if(!$ico_noX) $ico_noX=outIcon('noX');

	$type=(($value['type']=='dir' || $value['type']=='drive')?'d':'f');
	$o='['.$id.',"'.$type.'",';
	if(isset($options['relativeDir'])) {
		// Relative dir is computer root: display full filename
		if($options['relativeDir']=='computerRoot') $o.='"'.str_replace('"','\\"',cfProtectFilename($value['completeFileName'])).'",';

		// Relative dir is resource base path:
		else if(substr($options['relativeDir'],0,18)=='*resourceBasePath*'){
			$value['completeFileName']=cfResourceRelativePath($value['completeFileName']);
		}
		// Relative dir
		$o.='"'.str_replace('"','\\"',cfProtectFilename(substr($value['completeFileName'],strlen($options['relativeDir'])+1))).'",';
	}
	else $o.='"'.str_replace('"','\\"',cfProtectFilename($value['name'])).'",';

	// Thumbnails options
	$tnOpts=(isset($options['crop']))?array('sq'=>1):array();

	// Dirs
	if($type=='d'){
		// Authorized subdir
		if($value['rights']['state']=='authorized') {
			// draggable && receiver
			if($value['rights']['modify'] || ($multipleDownloadAllowed && @$value['rights']['read'])){
				if($value['rights']['write'] && $modifyAllowed) $o.='1,1,';	else $o.='1,0,';
			}
			elseif($value['rights']['write']) $o.='0,1,';
			else $o.='0,0,';
			// Icon
			if($value['type']=='drive'){
				$ico=(isset($options['mediumIcon'])&&substr($value['icon'],0,4)=='/gfx')?outIcon('fi/med/'.basename($value['icon'])):$value['icon'];
				$o.="'".addslashes($ico)."',";
			}
			else
				$o.='0,';
		}
		else $o.='0,0,1,';


		// editable, downloadable, mdownloadable, size
		$o.='0,';
		if($value['rights']['download'] && $multipleDownloadAllowed) $o.='1,1'; else $o.='0,0';

		$o.=',"",';
		// date
		if(isset($value['date']))
			$o.="'".addslashes((($value['date']!='')?cfUTF8Encode(date($dateFormat, $value['date'])):''))."',";
		else
			$o.="'',";
		// extra info
		$o.='"",';
		// Suppress
		if($value['rights']['modify'] && $value['rights']['write']) $o.='1,'; else $o.='0,';
		// Rename
		if($value['rights']['modify']) $o.='1,'; else $o.='0,';
		// Execute
		$o.='0';
	}
	// Files
	else{
		// Draggable, receiver
		if($value['rights']['modify'] || ($multipleDownloadAllowed && $value['rights']['download'] && $downloadShortcut)) $o.='1,0,'; else $o.='0,0,';
		// File extension's icon
		$ico=(isset($options['mediumIcon'])&&substr($value['icon'],0,4)=='/gfx')?outIcon('fi/med/'.basename($value['icon'])):$value['icon'];
		$o.="'".addslashes($ico)."',";

		// "Edit file" icon
		if($value['rights']['download'] && $value['rights']['modify'] && $editionAllowed && cfFileExtension($value['name']) && strpos(EDITABLE_FILES,';'.cfFileExtension($value['name']).';')) $o.='1,'; else $o.='0,';

		// Download file icon
		if($value['rights']['download']){
			if($downloadShortcut) $o.='2,'; else $o.='1,';
		}
		else $o.='0,';

		// Multiple download icon
		if($value['rights']['download'] && $multipleDownloadAllowed) $o.='1,'; else $o.='0,';

		// Size
		$o.="'".addslashes(str_replace(' ','&nbsp;',efFileSizeFormat($value['size'])))."',";
		// Date
		$o.="'".addslashes((($value['date']!='')?cfUTF8Encode(date($dateFormat, $value['date'])):''))."',";

		// Extra info
		$o.="'".addslashes((($value['extra1'])?cfUTF8Encode($value['extra1']):''))."',";
		// Suppress
		if($value['rights']['modify']) $o.='1,'; else $o.='0,';
		// Rename
		if($value['rights']['modify']) $o.='1,'; else $o.='0,';
		// Execute
		if($value['rights']['execute']) $o.='1,'; else $o.='0,';

		// Rating
		//if(isset($value['rating'])) $o.="'".((($value['rating']['score']==0)?'0':'').(int)($value['rating']['score']*10)).(($value['rating']['hasVoted'])?1:0).$value['rating']['nb']."',"; else $o.='0,';
		if(isset($value['rating'])) $o.="'".$value['rating']."',"; else $o.='0,';

		// Nb of views
		if(isset($value['viewed'])) $o.=(int)($value['viewed']); else $o.='0';

		// Thumbnail
		if(isset($options['thumbnail']) && ($value['subType']=='image'||$value['subType']=='video'||($value['subType']=='audio'&&!efIsPlaylist($value['name'])))){
			$adjust=($value['subType']=='audio')?3/4:1; // Ratio applied to thumbnail

			/**
			 * Compute thumbnail dimensions
			 */

			// Video thumbnails: as there is no source width & height info in $value, retreive this info from generated thumbnail image
			if(!isset($options['crop']) && isset($value['thumbnail']) && !isset($value['width'])) {
				@list($value['width'],$value['height'])=@getimagesize($value['thumbnail']);
			}

			// Crop & square image
			if(isset($options['crop'])) $w=$h=$options['thumbnail']*$adjust;
			// Images: use source image width & height to compute actual thumbnail ratio and size
			elseif(isset($value['width'])) list($w,$h)=cfThumbnailDimensions($value['width'],$value['height'],$options['thumbnail']*$adjust,$options['thumbnail']*3/4,true);
			// No info on source (videos only I think...)
			else {$w=$options['thumbnail']*$adjust;$h=floor(3*$options['thumbnail']/4);}


			/**
			 * Insert thumbnail link and dimensions info
			 */
			// If a thumbnail exists, base smaller thumbnail on it
			if(isset($value['thumbnail'])) $o.=',"'.cfExtImage(isset($value['thumbnail'])?$value['thumbnail']:$value['completeFileName'],$w,$h,true,0,$tnOpts).'",'.$w.','.$h;
			// Else, base thumbnail on actual file (disabled for videos as it would take too much time
			elseif ($value['subType']!='video') $o.=',"'.cfExtImage($value['completeFileName'],$w,$h,true,0,($tnOpts+(($value['subType']=='audio')?array('noDC'=>1):array()))).'",'.$w.','.$h;

		}
		else /*if(isset($options['thumbnail']))*/ $o.=',0,0,0';
		// Mime type
		/*if(isset($options['includeType']))*/	$o.=',"'.substr($value['subType'],0,1).'"';

	}
	return $o.']';
}

/**
 * @desc Return a JS array filled with files properties.
 * 				See efFilePropertiesToJSArray for list of properties
 *
 * @param string $type: 'all', 'files', 'dirs'
 * @param string options: see efFilePropertiesToJSArray
 * @param array $totals: array('file'=>0,'dir'=>0,'size'=>0)
 * @return string
 */
function efFilesJSArray($type='all',$options=false,&$totals=false){
	$i=0; $totals=array('file'=>0,'dir'=>0,'size'=>0);
	if(!$_ENV['weezoResEfFiles']) return 'Array()';

	/*
	* SUBDIRS
	*/
	$out='[';

	if($type!='files') foreach($_ENV['weezoResEfFiles'] as $key=>$value) if(isset($value['type']) && ($value['type']=='dir' || $value['type']=='drive')){
		$out.=(($i==0)?'':',').efFilePropertiesToJSArray($i,$value,$options);

		$i++;
	}
	$totals['dir']=$i;

	/*
	* FILES
	*/
	$nbFiles=0;
	if(isset($options['useInterval']) && cfRGetVar('itemsPerPage')>0){
		$min=(int)cfRGetVar('thumbnailsIndex');
		$max=$min+cfRGetVar('itemsPerPage');
	}
	else {
		$min=0;$max=9999;
	}
	if($type!='dirs') if($_ENV['weezoResEfFiles']) foreach($_ENV['weezoResEfFiles'] as $key=>$value) if(isset($value['type']) && $value['type']=='file'){


		if($nbFiles>=$min && $nbFiles<$max){
			$out.=(($i==0)?'':',').efFilePropertiesToJSArray($i,$value,$options);
			$totals['size']+=$value['size'];
			$i++;
		}
		$nbFiles++;
	}

	$out.=']';
	$totals['files']=$i-$totals['dir'];
	return $out;
}

/**
 * @desc Rename $passedDir/$passedFile to $passedDir/$newName (first checks access rights)
 *
 * @param string $passedDir : current directory
 * @param string $passedFile : file to rename
 * @param string $newName : new name of file
 * @param boolean copy : copy instead of rename
 * @return string : new path & file name
 */
function efRename($passedDir,$passedFile,$newName,$copy=false){
	$source=cfCleanPathName(cfJoinPathFile($passedDir,$passedFile));

	if(substr($newName,0,18)!='*resourceBasePath*' && substr($newName,0,2)!='//' && substr($newName,1,1)!=':'){
		$dest=cfJoinPathFile($passedDir,$newName);
	}
	else{
		$dest=$newName;
	}

	if(!file_exists($source) || (!$copy && !cfFileRights($source,'modify'))) return false; //if source doesn't exist or cannot be modified

	if(!($destDir=cfCleanPathName(cfDirName($dest)))) return false;//if dest dir doesn't exist

	if(!cfFileRights($destDir,'write')) return false; // destination dir is protected
	$destFile=basename($dest);

	$dest=cfJoinPathFile($destDir,$destFile);

	// Do copy
	if($copy) {
		if(is_dir($source)) return cfCopyDir($source,$dest); else return @copy($source,$dest);
	}

	// Move (if not in same directory, copy & delete)
	if(substr($source,0,1)!=substr($dest,0,1) && is_dir($source)) return cfCopyDir($source,$dest,true);
	else return @rename($source, $dest); // Do rename
}
function efCopy($passedDir,$passedFile,$newName){return efRename($passedDir,$passedFile,$newName,true);}

/**
 * @return mixed new dir if ok, false if not ok
 * @param string $dir : current dir
 * @param string $file : subdir
 * @param string $action ('goDirect' or 'go')
 * @desc changes 'efCurrentDirectory' to $dir."/".$file (first checks access rights)
*/
function efChangeCurrentDir($dir, $file, $action){
	// List mode
	if(cfSharedMode('list')){
		if($action=='goDirect' && substr($file,0,1)=='/') $dir='*resourceBasePath*';
		$completeFilename=cfCleanPathName(cfJoinPathFile($dir,$file));
		// If it's not a directory or if it's not allowed, exit
		if($completeFilename!=='*resourceBasePath*' && !is_dir($completeFilename)) return false;
		if(cfFileRights($completeFilename,'state')!=='authorized') return false;
	}

	// Folder mode
	else{
		//if $action=='goDirect', check if it is relative path, and if so, modify $dir and $file so it works as a regular 'go' action
		if($action=='goDirect' && $dir!='computerRoot' && !(cfCleanPathName(cfJoinPathFile($dir,$file)))) {$dir=$file;$file=".";}

		// Action = back (previous directory)
		if($action=='goBack'){
			$file=cfRGetVar('efPreviousDirectories',(count(cfRGetVar('efPreviousDirectories'))-1)); $dir='computerRoot';
			cfRUnsetVar('efPreviousDirectories',count(cfRGetVar('efPreviousDirectories'))-1);
		}

		//if current dir is x:/ and $file is "..", go to computer root
		//if current dir is 'computerRoot' and $file is ".", go to computer root
		if(((strlen($dir)<=3 && $file=="..") || ($dir=='computerRoot' && $file=='.')) && cfFileRights('computerRoot','state')=='authorized'){
			$completeFilename='computerRoot';
		}
		else{
			if($dir=='computerRoot')
			$completeFilename=cfCleanPathName($file);
			else
			$completeFilename=cfCleanPathName(cfJoinPathFile($dir,$file));
		}
		// if passed dir exists && is a directory && authorized, OK
		if($completeFilename && ($completeFilename=='computerRoot' || $completeFilename=='*resourceBasePath*' || is_dir($completeFilename)) && (cfFileRights($completeFilename,'state')=='authorized')){}
		// else exit
		else return false;
	}

	// if directory has changed
	if(cfRGetVar('efCurrentDirectory')!==$completeFilename){
		if(!cfRGetVar('efPreviousDirectories')) cfRSetVarArray('efPreviousDirectories');
		if($action!='goBack') cfRSetVar('efPreviousDirectories',count(cfRGetVar('efPreviousDirectories')), cfRGetVar('efCurrentDirectory')); // Memorize dir in navigation history
		cfRSetVar('efCurrentDirectory',$completeFilename); //Sets new directory
	}
	return $completeFilename;
}

/**
 * @desc execute a file on server (can be a non-exe file)
 *
 * @param string $passedDir : path
 * @param string $passedFile : filename
 * @param string $arguments : optional arguments
 * @param bool/mixed $async: not false if async response
 */
function efExecuteFile($completeFilename,$arguments=false,$async=false){
	// if this file may not be executed
	if(!cfFileRights($completeFilename,'execute')){
		cfLog('Unauthorized execution attempt on file '.cfJoinPathFile($dir,$file),LOG_ERROR);
		outDisplayErrorPage(cfCaption('explorerErrorExecute'));
	}
	// Execute file
	$ext=cfFileExtension($completeFilename);
	if($ext=='exe'||$ext=='com'||$ext=='bat') cfExecFork($completeFilename,'SW_SHOWNORMAL', $arguments);
	else customShellExecute($completeFilename,'open');

	// Display page
	if($async){
		cfAsyncHeader();
		die(cfAsyncFooter());
	}
	cfInsertHEAD(true);
	outShadowBefore('100%');
	echo '<div class="frame1"><div class="frame1Header">'.cfCaption('explorerExecute').'</div>';
	echo '<br/><br/><center>'.cfCaption('explorerExecuted',cfUTF8Encode(basename($completeFilename))).'</center><br/>';
	echo '<center>'.outButton(cfCaption('genClose'),'javascript:close()',outIcon('close')).'</center>';
	echo '<br/><br/>';
	echo '</div>';
	outShadowAfter();
	echo '</body></html>';
	exit;
}

/**
 * @desc create new folder in $dir
 *
 * @param string $dir
 * @param string $folderName
 * @return boolean : result of creation
 */
function efCreateFolder($dir,$folderName){
	$dir=cfCleanPathName($dir);
	if(!$dir || $dir=='computerRoot' || !cfFileRights($dir, 'write') || file_exists(cfJoinPathFile($dir,$folderName))) return false;
	return mkdir(cfJoinPathFile($dir,$folderName));
}

/**
 * @desc Async search for a file
 *
 * @param string $searchString
 * @param boolean $searchContent: true to search in files
 */
function efSearchFiles($passedDir,$searchString,$searchContent){
	cfAsyncHeader();
	$result=array();
	flush();
	$contentString=$searchString;

	// Add wildcards if none set
	if(!strpos($searchString,'*') && !strpos($searchString,'?')) $searchString='*'.$searchString.'*';

	// Set-up cancel message reception
	cfMSetVar('weezoCancel'.cfUGetVar('id').'-'.cfMGetVar('id'),1);
	wSession_write_close();

	// If share mode is file list and searched directory is upper dir, process all top-level directories
	if($passedDir=='*resourceBasePath*' && cfSharedMode('list')){
		$result=array();
		foreach (cfSharedItems() as $cfn=>$type) {
			if($type==='d') $result=array_merge(efSearchFilesRec($cfn,$searchString,$searchContent,$contentString),$result);
			elseif($type==='f') {
				foreach (cfGlob(dirname($cfn).'/'.$searchString) as $cfn2){
					if($cfn2==$cfn){
						$result=array_merge(array(efFileProperties($cfn,false)),$result);
						break;
					}
				}
			}
		}
	}

	// Else, do regular recursive directory search (only check directory access rights so far)
	elseif(cfFileRights($passedDir,'read')) {
		$result=efSearchFilesRec($passedDir,$searchString,$searchContent,$contentString);
	}

	cfMUnsetVar('weezoCancel'.cfUGetVar('id').'-'.cfMGetVar('id'),1);

	// Set JS files array properties
	$filesProperties=array();
	// Add thumnails if required by resource
	if(cfRGetVar('filesLayout')=='icons' || cfRGetVar('filesLayout')=='resource' || efFilesListType()=='customIcons') $filesProperties['thumbnail']=efThumbnailsMaxWidth();
	if($passedDir) $filesProperties['relativeDir']=$passedDir;

	// Browse results
	if(count($result)){
		$i=0;
		$output='[';
		$totalSize=0;
		foreach ($result as $res) {
			if(cfFileRights($res['completeFileName'])){
				// If thumbnails are needed and a thumbnail already exists,
				// put it's path into file's ['thumbnail'] property so efFilePropertiesToJSArray includes it (required for videos)
				if(@($filesProperties['thumbnail']) && is_file(efThumbnailFilePath($res['completeFileName']))) $res['thumbnail']=efThumbnailFilePath($res['completeFileName']);
				$output.=(($i)?',':'').efFilePropertiesToJSArray($i++,$res,$filesProperties);
				$totalSize+=$res['size'];
			}
		}
		$output.=']';
		echo cfAsyncXMLJSaction('searchFileComplete('.$output.',"'.addslashes(efFileSizeFormat($totalSize)).'");');
	}
	else{
		echo cfAsyncXMLJSaction('searchFileComplete(false,"")');
	}

	die(cfAsyncFooter());
}

/**
 * Async search recursive function
 *
 * @param string $dir
 * @param string $searchString
 * @return array of files with properties
 */
function efSearchFilesRec($dir,$searchString,$searchContent,$contentString){
	$return=array();
	$tmp=array();

	// Non-detection of async connection aborted workaround: use a 2nd message to cancel search
	if(!cfMIssetVar('weezoCancel'.cfUGetVar('id').'-'.cfMGetVar('id'))) die();


	// Search in files names
	foreach (cfGlob($dir.'/'.$searchString) as $cfn) {
		$prop=efFileProperties($cfn,false);
		if($prop['rights']['download']) $return[]=$prop;

		$tmp[$cfn]=1;
	}
	// Search in files content
	if($searchContent){
		foreach (cfGlob($dir.'/*') as $cfn) if(!isset($tmp[$cfn]) && is_file($cfn)){
			// 1st check access rights
			$prop=efFileProperties($cfn,false);
			if(!$prop['rights']['download']) continue;

			$buff='';
			// Search in file
			if($fp=fopen($cfn,'r')){
				while (!feof($fp)){
					$buff.=fread($fp,32384);
					if(strpos($buff,$contentString)!==false){
						$return[]=$prop;
						break;
					}
					$buff=substr($buff,32384-strlen($contentString));
				}
				fclose($fp);
			}
		}
	}

	// Search subdirectories
	foreach (cfGlob($dir.'/*') as $cfn) if(is_dir($cfn)&&cfFileRights($cfn,'read')) $return=array_merge(efSearchFilesRec($cfn,$searchString,$searchContent,$contentString),$return);
	return $return;
}

/**
 * @desc Cancel current search
 *
 */
function efSearchFilesCancel(){
	cfMUnsetVar('weezoCancel'.cfUGetVar('id').'-'.cfMGetVar('id'));
	cfAsyncHeader();
	die(cfAsyncFooter());
}

/**
 * @return array of string : 'previous' and 'next' : file name of previous and next files
 * @param $fileName : filename of file to search
 * @desc return next and previous files in $_ENV['weezoResEfFiles'] array
*/
function efGetNextAndPrevious($fileName){
	if(!isset($_ENV['weezoResEfFiles'])) efDirUpdate();
	$fileName=strtolower($fileName);
	$first=false;$last=false;$prev=false; $next=false; $position=0; $filePosition=false;
	foreach ($_ENV['weezoResEfFiles'] as $key => $value ) if(is_array($value) && $value['type']=='file'){
		if(strtolower($value['name'])==$fileName) $filePosition=$position;
		if($first===false) $first=$value['name'];
		if($filePosition===false) $prev=$value['name'];
		if($filePosition!==false && $filePosition!=$position && $next===false) $next=$value['name'];
		$last=$value['name'];
		$position++;
	}
	if($filePosition===false) return array('previous'=>false, 'next'=>false);
	if($prev===false) $prev=$last;
	if($next===false) $next=$first;
	return array('previous'=>$prev, 'next'=>$next);
}


/**
 * @desc Reture array (complete filename, directory, file name) from comForm POST data
 *
 * @param bool $verifyFile: true to verify passed dir/file exists
 * @return array complete file name, dir, file
 */
function efGetPassedFileComponents($verifyFile=true,$pD=null,$pF=null){

	// Get passed dir and file
	if($pD!==null && $pF!==null){
		$passedDir=$pD;
		$passedFile=$pF;
	}
	else{
		$passedDir=(isset($_POST['data1']))?cfUnprotectFilename($_POST['data1']):false;
		$passedFile=(isset($_POST['data2']))?cfUnprotectFilename($_POST['data2']):false;
	}

	// Exception for playlist
	if($passedFile=='*playlist*.m3u') $completeFilename=$passedFile;

	// shared list: check passed file / folder is in list and convert to actual filename
	if(cfSharedMode('list')){
		//cfDebugSingle(strtoupper($action).': Received args: dir='.$passedDir.' file='.$passedFile.' cfn=>???');
		$completeFilename='';

		if ($passedFile=='*playlist*.m3u'){
			$completeFilename=$passedFile;
		}
		elseif ($passedFile=='/'||$passedFile=='*resourceBasePath*'){
			$completeFilename=$passedDir='*resourceBasePath*';
			$passedFile='.';
		}
		elseif(cfCmpLeft($passedFile,'*resourceBasePath*')) { // Dir passed into file
			$completeFilename=cfSharedCompleteFilename($passedFile);
			$passedDir=cfDirName($completeFilename);
			$passedFile=cfBasename($completeFilename);
		}
		elseif($passedDir){
			$completeFilename=cfSharedCompleteFilename(cfJoinPathFile($passedDir,$passedFile));
			$passedDir=cfDirName($completeFilename);
			$passedFile=cfBasename($completeFilename);
		}
		//cfDebugSingle(strtoupper($action).': Processed args: dir='.$passedDir.' file='.$passedFile.' cfn=>'.$completeFilename);
	}
	// shared folder: replace *resourceBasePath* by actual path
	else{
		// Get directory
		if(strpos($passedDir,'*resourceBasePath*')!==false && cfRGetVar('path')) $passedDir=str_replace('*resourceBasePath*',cfRGetVar('path'),$passedDir);
		// Get file
		if(strpos($passedFile,'*resourceBasePath*')!==false && cfRGetVar('path')) $passedFile=str_replace('*resourceBasePath*',cfRGetVar('path'),$passedFile);

		// Merge & clean
		$completeFilename=cfCleanPathName(cfJoinPathFile($passedDir,$passedFile),$verifyFile);
	}
	// cfDbg($completeFilename.'='.$passedDir.'+'.$passedFile);
	return array($completeFilename,$passedDir,$passedFile);
}

/*
***************************************************************************************************************************
*
* Explorer related HTML stuff...
*
***************************************************************************************************************************
*/

/**
 * @return string name of command processed, or false if no command processed
 * @desc Process commands received through comForm or HTTPXlmRequest
*/
function efProcessPOST(){
	global $errorMessage;
	global $errorMessageCritical;
	global $asyncResponse;
	global $tree;

	// If a critical error has been raised, don't process
	if($errorMessageCritical) {if(isset($_POST['asyncRequest'])) $asyncResponse='error'; return false;}

	//Async file share form insertion and processing
	if(cfRGetVar('shareAllowed')) {
		require_once(cfAppDocRoot().'/local/directLink.php');
		dlDirectLinkRemoteForm();
	}

	/**
	 * Common explorer panels / controls vars
	 */
	cfAsyncXMLChangePhpVarValue('actionsPanel'); // process hide / show right panel
	cfAsyncXMLChangePhpVarValue('displayMultipleDownload'); // process hide / show multiple download frame dynamic update
	cfAsyncXMLChangePhpVarValue(SLIDESHOW_TIMER_CONTROL_NAME,array(1,2,3,4,5,10,15,30)); // process slideshow timeout value change

	// If form has been submited using get method, transform into POST
	if(isset($_GET['data3'])){
		if(isset($_GET['data1'])) $_POST['data1']=$_GET['data1'];
		if(isset($_GET['data2'])) $_POST['data2']=$_GET['data2'];
		$_POST['data3']=$_GET['data3'];
		if(isset($_GET['data4'])) $_POST['data4']=$_GET['data4'];
		if(isset($_GET['data5'])) $_POST['data5']=$_GET['data5'];
	}

	// If no command sent, return
	if(!isset($_POST['data3'])) return;

	$d1=@$_POST['data1'];
	$d2=@$_POST['data2'];
	$action=cfUTF8Decode($_POST['data3']);
	if(isset($_POST['data4'])) $d4=$_POST['data4'];
	if(isset($_POST['data5'])) $d5=$_POST['data5'];

	/**
	 * Get passed dir and file
	 */
	list($completeFilename,$passedDir,$passedFile)=efGetPassedFileComponents();

	// If POST request has been sent by XMLHttpRequest, inform resource that an async response is expected
	if(cfIsAsync()) $asyncResponse=$action; else $asyncResponse=false;

	/*
	* Common explorer actions
	*/

	// file emailing (mailout) form insertion and processing
	if($action=='mailoutShowRemoteForm' || $action=='mailoutSend') efMailoutProcess($action, $completeFilename);

	// Folder navigation
	//if authorized $passedDir/$passedFile, set  efCurrentDirectory to $passedDir/$passedFile
	if(($action=='go' || $action=='goDirect' || $action=='goBack') && $passedDir) {
		// Direct access to upper dir by typping "/" (except for non-windows where "/" is root directory
		if($action=='goDirect' && substr($passedFile,0,1)=='/' && substr($passedFile,0,2)!='//' && WEnv::host()->isWindows()) {
			$passedDir=((cfSharedMode('list'))?'*resourceBasePath*':cfRGetVar('path'));
			$passedFile=str_pad(substr($passedFile,1),1,'.');
		}
		if(!efChangeCurrentDir($passedDir, $passedFile, $action)) $errorMessage=cfCaption('explorerErrorChangeDir');
		else{
			//reset images and subDirs arrays, reset thumbnails and preview position
			cfRSetVar('thumbnailsIndex','0');cfRSetVar('activePreview',0);
			cfRSetVar('itemsPerPagePos',0);
		}
	}
	// Tree unfold
	if($action=='treeUnfoldTo' && isset($d4) && cfRGetVar('treeViewDisplayed')){
		cfAsyncHeader();
		if(is_dir($completeFilename) && cfFileRights($completeFilename,'read')){
			echo cfAsyncXMLJSaction($tree->updateScript($completeFilename));
		}
		else{
			echo cfAsyncXMLJSaction('treeTmp = new treeNode("'.$d5.'");treeTmp.setError()');
		}
		echo cfAsyncXMLJSaction('treeProcessing=0;');
		cfAsyncFooter();
	}
	// Toggle tree frame displayed / hidden
	if($action=='toggleTreeFrame'){
		cfRSetVar('treeViewDisplayed',($d4==1)?true:false);
		if(($d4==0) && cfRGetVar('layout')==0) cfRSetVar('layout',1);
		if(($d4==1) && cfRGetVar('layout')==1) cfRSetVar('layout',0);
		if(($d4==0) && cfRGetVar('layout')==2) cfRSetVar('layout',3);
		if(($d4==1) && cfRGetVar('layout')==3) cfRSetVar('layout',2);
	}
	// Change files layout
	if($action=='filesLayoutSet'&&($d4=='list'||$d4=='icons'||$d4=='resource'||$d4=='customIcons') && !cfRGetVar('filesLayoutLocked')) cfRSetVar('filesLayout',$d4);

	// remote download
	if(($action=='rdl')){
		if(!efRemoteDownload($passedDir, $passedFile)) {
			$errorMessage=cfCaption('explorerErrorDirectDownload');
		}
		elseif ($asyncResponse){
			cfAsyncHeader();
			echo cfAsyncXMLJSaction('if(dgi("transfersIFrame")) window.setTimeout("frames[\'transfersIFrame\'].refresh()",1000)');
			cfAsyncFooter();
		}
	}

	// file download
	if($action=='download' && $completeFilename){
		if(!efForceDownload($completeFilename)) $errorMessage=cfCaption('explorerErrorDownload');
	}

	// Rate file
	if($action=='rate'){
		if($completeFilename && is_numeric($_POST['data4']) && $_POST['data4']>=1 && $_POST['data4']<=5){
			if(cfRGetVar('ratingAllowed') && cfGGetVar('ratingAllowed') && cfFileRights($completeFilename,'state')=='authorized') efFileRate($completeFilename,(int)$_POST['data4']);
		}
		cfAsyncHeader();
		cfAsyncFooter();
	}

	// multiple file download functions
	if($action=='mAdd' && $passedFile && !cfRGetVar('playlistDisplayed')) efMultipleDownloadAdd($passedFile);// add file to download list (dl)
	if($action=='mSupp' && $passedFile && !cfRGetVar('playlistDisplayed')) efMultipleDownloadSupp($passedFile);// remove file from dl list
	if($action=='mSuppAll' && !cfRGetVar('playlistDisplayed')) efMultipleDownloadClear();// clear dl list
	if($action=='mAddAll' && !cfRGetVar('playlistDisplayed')) efMultipleDownloadAddAll();// add all current dir to dl list
	if($action=='mDownload') efMultipleDownloadDownload();// start zip dl
	if($action=='mBtDownload') efMultipleDownloadCreateTorrent();// start bittorrent dl

	// Single bittrorrent file download
	if($action=='btDl' && $completeFilename) {
		if(!efSinglDownloadCreateTorrent($completeFilename)) $errorMessage=cfCaption('explorerErrorDownload');
	}

	// file VIEW (view fullscreen doesn't work)
	if(($action=='view' ||$action=='viewFullscreen')){
		if($passedFile=='*playlist*.m3u') $completeFilename=$passedFile;
		if($completeFilename){
			require(INCLUDE_DIR.'viewFunctions.php');
			vfViewFile($completeFilename,(isset($_POST['data3']))?(cfUTF8Decode(($_POST['data3']))):(''),
			(isset($d4))?(cfUTF8Decode(($d4))) : (''),
			(isset($d5)?(cfUTF8Decode(($d5))):('')));

		}
		exit();
	}

	// file edit
	if(($action=='edit') || $action=='editSave' || $action=='editSaveQuit'){
		if($completeFilename){
			require(INCLUDE_DIR.'editWindow.php');
			ewDisplayEditor($completeFilename, $action, (isset($d4)?$d4:''));
		}
		exit();
	}

	// FILE OR FOLDER RENAME
	elseif($action=='ren'||$action=='cpy') {
		//$dest=str_replace('*resourceBasePath*',cfRGetVar('path'),cfUnprotectFilename($d4));
		list($dest,$foo,$foo1)=efGetPassedFileComponents(false,$d4,'');
		if(!efRename($passedDir,$passedFile,$dest,$action=='cpy')) $errorMessage=cfCaption('explorerErrorRename');
	}


	// FOLDER CREATION
	elseif(($action=='newFolder')) {if(!efCreateFolder($passedDir,$passedFile)) $errorMessage=cfCaption('explorerErrorCreateFolder');}

	// FILE OR FOLDER SUPPRESS
	elseif(($action=='sup')) {
		if(!efUnlink(cfJoinPathFile($passedDir,$passedFile))) $errorMessage=cfCaption('explorerErrorSuppress');
	}

	// FILE EXECUTION
	elseif(($action=='exe')) {efExecuteFile($completeFilename,(isset($d4))?$d4:false,(isset($d5))?$d5:false);}

	// FILE SEARCH
	elseif(($action=='searchFiles') && isset($d4) && cfRGetVar('searchFiles')) {
		efSearchFiles($completeFilename,$d4,((isset($d5) && cfRGetVar('searchFilesContentAllowed'))?$d5:false));
	}
	elseif(($action=='searchFileCancel')) efSearchFilesCancel();

	// FILE SORT METHOD AND ORDER
	elseif(($action=='srt')) {
		$by=$_POST['data1'];
		$order=$_POST['data2'];

		if($order=='asc' || $order=='desc'){
			if($by!=cfRGetVar('efSortBy')) cfRSetVar('efSortOrderPrev',cfRGetVar('efSortOrder')); // Store previous sort order, for use as a secondary sort method
			cfRSetVar('efSortOrder',$order);
		}
		if($by=='name' || $by=='size' || $by=='date' || ($by=='rating' && cfRGetVar('ratingAllowed') && cfGGetVar('ratingAllowed'))) {
			if($by!=cfRGetVar('efSortBy')) cfRSetVar('efSortByPrev',cfRGetVar('efSortBy')); // Store previous sort method, for use as a secondary sort method
			cfRSetVar('efSortBy',$by);
		}

	}

	// Check if a file exists before overwritting with upload
	elseif ($action=='uploadCheckOverwrite'){
		cfAsyncHeader();
		$completeFilename=cfJoinPathFile($passedDir,basename($passedFile));
		$uploadAction='proceed';

		if(file_exists($completeFilename)){
			if(cfFileRights($completeFilename,'modify')) $uploadAction='confirm'; else $uploadAction='forbidden';
		}
		elseif (!cfFileRights($passedDir,'write') || !is_writable($passedDir)) {
			$uploadAction='forbidden';
		}

		if($uploadAction=='confirm') $caption=cfCaption('genFileAlreadyExistReplace');
		elseif($uploadAction=='forbidden') $caption=cfCaption('explorerErrorFileSend2');
		else $caption='';

		echo cfAsyncXMLJSaction('initUploadProceed("'.$uploadAction.'","'.$caption.'")');
		die(cfAsyncFooter());
	}

	// Display file information in tooltip
	elseif ($action=='getFileInfo'){
		cfAsyncHeader('',cfGetBrowser()!='ie');
		require(INCLUDE_DIR.'fileInfoFunctions.php');
		// Show thumbnail in tooltip
		if(isset($d4) && $d4) $getThumbnail=true; else $getThumbnail=false;
		// Show detailed info in tooltip
		if(isset($d5) && $d5=='nd') $noDetail=true; else $noDetail=false;

		if(cfFileRights($completeFilename,'download')) echo cfAsyncXMLJSaction('tooltipSetProperties('.fiPHPArrayToJSArray(fiGetInfo($completeFilename,$getThumbnail,!$noDetail)).')');
		else echo cfAsyncXMLJSaction('tooltipSetProperties({"icon":"'.outIcon('forbiddenBig').'","header":"&nbsp;"})');
		die(cfAsyncFooter());
	}
	elseif ($action=='setPos' && isset($d4)){
		if($d4=='next') cfRSetVar('itemsPerPagePos',cfRGetVar('itemsPerPagePos')+cfRGetVar('itemsPerPage'));
		elseif($d4=='prev') cfRSetVar('itemsPerPagePos',max(0,cfRGetVar('itemsPerPagePos')-cfRGetVar('itemsPerPage')));
	}
	// Change thumbnails size factor
	elseif ($action=='efTNSize' && isset($d4)){
		if(($d4==0||$d4==1||$d4==2) && !cfRGetVar('thumbnailsSizeFactorLocked')) cfRSetVar('thumbnailsSizeFactor',$d4);
	}

	/*
	* Photo albums related actions
	*/
	// Thumbnail shift
	elseif(($action=='goThumbnail')) {
		if(0+$passedDir < 0) cfRSetVar('thumbnailsIndex','0'); else cfRSetVar('thumbnailsIndex',$passedFile);
		cfRSetVar('activePreview', cfRGetVar('thumbnailsIndex'));
	}

	// Thumbnails per page
	elseif($action=='changeItemsPerPage' && (is_numeric($d2) && $d2>=0 && $d2<=100)) {
		cfRSetVar('itemsPerPage', $d2);
		if($st=cfRGetVar('efFilesStats')){
			$nbFiles=$st['nbFiles'];
			cfRSetVar('thumbnailsIndex',max(0,min($nbFiles-$d2,cfRGetVar('thumbnailsIndex'))));
		}
	}

	// New preview
	elseif($action=='preview'){
		if(isset($d4) && isset($d5))	efUpdatePreview($completeFilename,$d4,$d5);
	}
	// New comment
	elseif ($action=='commentsSave' && isset($d4) && cfRGetVar('commentsAllowed')) {

		// Get comments (non-administrator comments are HTML-cleaned)
		$comments=cfUTF8Decode($d4,true,true,!cfUGetVar('administrator'),2048);

		// Save comments
		efWriteComments($completeFilename,$comments,false);

		// Javascript code to go to next image
		cfAsyncHeader();
		echo cfAsyncXMLJSaction('preview.clickNext();dgi("efCommentsTextArea").focus()');
		cfAsyncFooter();

	}
	// Toggle comments edit mode
	elseif ($action=='toggleComments'){
		cfAsyncHeader();
		cfRSetVar('efEditMode',!cfRGetVar('efEditMode'));
		cfAsyncFooter();
	}
	// Preview image rotation
	elseif ($action=='rotate90' || $action=='rotate180' || $action=='rotate270' || $action=='rotate0'){
		cfRSetVar('previewAngle',substr($action,6));
		if(isset($d4)) efUpdatePreview($completeFilename, $d4); else efUpdatePreview($completeFilename);
	}
	/*
	* Playlist related actions
	*/

	// Add a file to playlist
	if($action=='plAdd' && file_exists($completeFilename) && cfFileRights($completeFilename,'download')){
		// Set file properties
		$id3=efGetAudioInfo($completeFilename);
		$arr=cfRGetVar('playlist');
		$arr[]=array('completeFileName'=>$completeFilename, 'name'=>$passedFile, 'label'=>$id3['label'], 'artist'=>@$id3['artist'], 'album'=>@$id3['album'], 'track'=>@$id3['track'], 'length'=>@$id3['length']);
		cfRUnsetVar('playlist');
		cfRSetVar('playlist',$arr);
		cfRSetVar('noFileRefresh',true);
		efConvertPlayListToMDList();
	}
	// Move playlist item up (=to lower index) or down (=to upper index)
	if($action=='plUp' || $action=='plDown') {
		$arr=cfRGetVar('playlist');
		if(isset($arr[$d4]) && $action=='plUp' && $d4>0 || $action=='plDown' && $d4<count($arr)-1){
			cfRUnsetVar('playlist');
			cfRSetVarArray('playlist');
			$i=0;
			foreach ($arr as $value) cfRSetVar('playlist',$i++,$value);
			if($action=='plUp') {
				cfRSetVar('playlist',$d4,$arr[$d4-1]);
				cfRSetVar('playlist',$d4-1,$arr[$d4]);
			}
			else{
				cfRSetVar('playlist',$d4+1,$arr[$d4]);
				cfRSetVar('playlist',$d4,$arr[$d4+1]);
			}
			cfRSetVar('noFileRefresh',true);
			efConvertPlayListToMDList();
		}
	}
	// Clear playlist
	if($action=='mSuppAll' && cfRGetVar('playlistDisplayed')) {
		cfRUnsetVar('playlist'); cfRSetVarArray('playlist'); cfRSetVar('noFileRefresh',true);
		efConvertPlayListToMDList();
	}
	// Remove a file from playlist
	if($action=='plSupp') {
		$arr=cfRGetVar('playlist');
		if(isset($arr[$d4])) unset($arr[$d4]);
		cfRUnsetVar('playlist');
		cfRSetVarArray('playlist');
		$i=0;
		foreach ($arr as $value) cfRSetVar('playlist',$i++,$value);
		cfRSetVar('noFileRefresh',true);
		efConvertPlayListToMDList();
	}
	// Add all current dir's files to playlist
	if($action=='mAddAll' && cfFileRights($passedDir=cfRGetVar('efCurrentDirectory'),'state')=='authorized' && cfRGetVar('playlistDisplayed')){
		$arr=cfRGetVar('playlist');

		$af=efGetDirContent($passedDir,false,false,'audio',false);
		foreach ($af as $key=>$value) if(is_numeric($key) && cfFileExtension($value['name'])!='m3u'){
			$completeFilename=cfJoinPathFile($passedDir,$value['name']);
			$id3=efGetAudioInfo($completeFilename);
			$arr[]=array('completeFileName'=>$completeFilename, 'name'=>$passedFile, 'label'=>$id3['label'], 'artist'=>@$id3['artist'], 'album'=>@$id3['album'], 'track'=>@$id3['track'], 'length'=>@$id3['length']);
		}
		cfRUnsetVar('playlist');
		cfRSetVar('playlist',$arr);
		cfRSetVar('noFileRefresh',true);
		efConvertPlayListToMDList();
	}
	// Toggle shuffle playlist
	if($action=='plToggleShuffle'){
		cfAsyncHeader();
		if(isset($d4) && $d4=='ns'){
			cfRSetVar('shufflePlaylist',false);
			echo cfAsyncXMLJSaction('dgi("plShuffleBtOn").style.display="none";dgi("plShuffleBtOff").style.display="inline";');
		}
		else {
			cfRSetVar('shufflePlaylist',true);
			echo cfAsyncXMLJSaction('dgi("plShuffleBtOn").style.display="inline";dgi("plShuffleBtOff").style.display="none";');
		}
		cfAsyncFooter();
	}
	// Other audio explorer
	if($action=='groupBy' && isset($d4)){
		if($d4=='groupAlbum') cfRSetVar('sortField','album');
		elseif($d4=='groupArtist') cfRSetVar('sortField','artist');
		elseif($d4=='groupArtistAlbum') cfRSetVar('sortField','artistAlbum');
		else cfRSetVar('sortField','no');
	}

	/**
 * Mobile specific actions
 */
	if($action=='setView' && isset($d4)){
		if($d4=='concat' || $d4=='folders' || $d4=='files' || $d4=='tools' || $d4=='search'){
			cfRSetVar('view',substr($d4,0,100));
			cfRSetVar('itemsPerPagePos',0);
		}
	}
}

/**
 * @desc Insert javascripts needed by explorers
 *
 * @return void
 */
function efInsertJScripts(){
	global $errorMessageCritical;
	require_once(INCLUDE_DIR.'mailFunctions.php');

	// If a critical error has been raised, don't process
	if($errorMessageCritical) return false;
?>
<script language="javascript" type="text/javascript">
<?php
	// Full-size view allowed
	echo cfJSRVar('fileViewAllowed');

	// Link sharing allowed
	echo cfJSRVar('shareAllowed');

	// Send by email allowed
	if(Mailout::isMainConfigurationSet()) echo cfJSRVar('mailoutAllowed');

	// Rating allowed
	if(cfRGetVar('ratingAllowed') && cfGGetVar('ratingAllowed')) echo "var ratingAllowed=1;\n";

	// File comments vars
	if(cfRGetVar('commentsAllowed')) echo 'var editMode='.((int)cfRGetVar('efEditMode')).";\n";
	// Image preview vars
	if(cfRGetVar('previewDisplay')) {
		echo "var currentImage=".(cfRGetVar('activePreview')+0).";\n";
		echo "var slideShowActive=0;\n";
		echo "var slideShowTimer;\n";
	}
	// search files vars
	if(cfRGetVar('searchFiles')) {
		echo 'var efSearchTitle="'.cfCaption('genFiles')."\";\n";
		echo 'var efSearchText="'.cfCaption('genSearch')."\";\n";
		echo 'var efSearchIcon="'.outIcon('search')."\";\n";
		echo 'var efSearchOkCaption="'.cfCaption('genOK')."\";\n";
		echo 'var efSearchCancelCaption="'.cfCaption('genCancel')."\";\n";
		if(cfRGetVar('searchFilesContentAllowed')) {
			echo cfJSCaptionVar('searchFilesContent');
			echo "var searchFilesContentAllowed=1;\n";
			echo "var searchFilesContentDefault=".(int)cfRGetVar('searchFilesContentDefault').";\n";
		}
	}
	// Shared types
	echo 'var fileTypeFilter='.json_encode(cfRGetVar('fileTypeFilter')).';';

	// Main files extensions
	if(!cfRGetVar('fileTypeFilter')) echo 'var viewableFiles='.json_encode($_ENV['mainFilesExtensions']).';';

	// All other vars needed by explorer.js functions
?>
var slideShowInterval=<?php echo (max(1,0+cfRGetVar('slideShowInterval'))*1000); ?>;
wl.capt.basePath="<?php echo cfUTF8Encode(efDirLabel());?>";
wl.capt.explorerEnterNewName="<?php echo cfCaptionJS('explorerEnterNewName');?>";
wl.capt.genConfirmMove="<?php echo cfCaptionJS('genConfirmMove');?>";
wl.capt.genConfirmCopy="<?php echo cfCaptionJS('genConfirmCopy');?>";
wl.capt.genConfirmSuppress="<?php echo cfCaptionJS('genConfirmSuppress');?>";
wl.capt.explorerEnterFolderName="<?php echo cfCaptionJS('explorerEnterFolderName');?>";
wl.capt.explorerEnterParameters="<?php echo cfCaptionJS('explorerEnterParameters');?>";
wl.capt.explorerAudioNoSongInPlayList="<?php echo stripslashes(cfCaptionJS('explorerAudioNoSongInPlayList'));?>";
<?php
if(efHasDragDownload()) echo 'wl.capt.dragDownloadHint=\''.outImageIcon('hint').stripslashes(cfCaptionJS('altDragHint')).'\';';
echo cfJSCaptionVar('viewRotateLeft');
echo cfJSCaptionVar('viewRotateRight');
?>

var efIcoUp='<?php echo str_replace("'","\\'",outImage(outIcon('up')));?>';
var efIcoDown='<?php echo str_replace("'","\\'",outImage(outIcon('down')));?>';

var phpRVarInlinePlayer=<?php echo (int)cfRGetVar('inlinePlayer');?>;
var efEditMode=<?php echo(cfRGetVar('efEditMode')?'1':'0');?>;
var dragDownload=<?php if (efHasDragDownload()) echo 1; else echo 0; // Ability to download files directly outside browser ?>;
</script>
<?php
	// Include explorer.js scripts
	echo cfScriptLink('explorer.js');
}

/**
 * @desc Insert HTML <head> and <body> parts for explorers, with commands communication form and "send" dialog box
 *		or insert XML header if asynchronous response is required
 *
 * @param string $bodyExtraHTML : extra HTML code to include in <body>
 * @return void
 */
function efInsertExplorerHeadAndBody($bodyExtraHTML=false, $extraOptions=false){
	global $errorMessage;
	global $errorMessageCritical;
	global $asyncResponse;

	// Facebook comments panel status update
	outCommentsPanelUpdate();

	// If an asynchronous response is required, just display XML header
	if($asyncResponse) {
		cfAsyncHeader();
		if($errorMessageCritical) {
			echo cfAsyncXMLJSaction('alert("Error : '.addslashes($errorMessage).'")');
			cfAsyncFooter();
		}
		if($errorMessage) echo cfAsyncXMLJSaction('alert("'.addslashes($errorMessage).'")');
		// Update standardComForm 'data1' with new current directory value
		echo cfAsyncXMLJSaction('D.comForm.data1.value="'.cfUTF8Encode(cfResourceRelativePath(cfRGetVar('efCurrentDirectory'))).'"');
		// Update current directory label
		echo cfAsyncXMLJSaction('wl.capt.basePath="'.addslashes(cfUTF8Encode(efDirLabel())).'"');
		return;
	}

	// If a critical error has been raised, display error page and exit
	if($errorMessageCritical) outDisplayErrorPage($errorMessageCritical,false,true);

	// Insert HTML Head stuff
	cfInsertHEAD(false);

	// Insert Javascripts
	efInsertJScripts();

	// Insert extra style
	if(isset($extraOptions['style'])) echo "\n<style type=\"text/css\">\n".$extraOptions['style']."\n</style>";
	echo "\n</head>\n";

	// Onload
	$onload=false;
	if($errorMessage) $onload.='alert(\''.$errorMessage.'\');';
	if(preg_match('/onload\s*=\s*"([^"]{0,})"/i',$bodyExtraHTML,$matches)){
		$bodyExtraHTML=str_replace($matches[0],'',$bodyExtraHTML);
		$onload.=$matches[1].';';
	}
	if(cfRGetVar('onload')) $onload.=(($onload)?';':'').cfRGetVar('onload');

	// <body>
	echo '<body style="overflow:hidden"'.
	(($bodyExtraHTML)?' '.$bodyExtraHTML:'').
	(($onload)?' onload="'.$onload.'"':'').
	((cfGetBrowser()=='webkit')?' onselectstart="return false"':'').
	((cfGetBrowser()=='ie'||cfGetBrowser()=='webkit')?' ondragstart="return false"':'').
	cfBodyRightClick().
	">\n";

	echo '<script type="text/javascript">$(D.body).e.disableSelection()</script>';

	// Video thumbnails generation & monitoring
	if(isset($_ENV['thumbnailVideoNeeded']) && count($_ENV['thumbnailVideoNeeded'])) efMakeVideoThumbnails($_ENV['thumbnailVideoNeeded']); unset($_ENV['thumbnailVideoNeeded']);

	// Images thumbnails generation & monitoring
	if(isset($_ENV['thumbnailPhotoNeeded']) && is_array($_ENV['thumbnailPhotoNeeded'])) efMakePhotoThumbnails($_ENV['thumbnailPhotoNeeded']); unset($_ENV['thumbnailPhotoNeeded']);

	// Drag'n drop scripts
	if(!isset($_ENV['noDragAndDrop'])) echo cfScriptLink('wz_dragdrop.js');

	// Load mask
	echo outLoadMask(false);

	// Comments panel
	require_once(INCLUDE_DIR.'facebookFunctions.php');
	facebookCommentsPanel();

	// top div layer (used for fixed-width templates, and to hide entire page when displaying full-size browser)
	echo '<div id="topDiv">'."\n";

	// Inserts standard communication FORM
	outInsertStandardComForm($_SERVER['PHP_SELF'],cfUTF8Encode(cfResourceRelativePath(cfRGetVar('efCurrentDirectory'))));
}

/**
 * @desc Insert (echo) hidden field including file name,
 *
 * @param string $fileName
 * @param integer $efSeqFileNumber : sequential file number
 * @param integer $hidden: true if already hidden (no need for dn)
 */
function efOutFileTag($fileName,$sequentialIndex,$hidden=0){
	return '<p id="id'.$sequentialIndex.'"'.(($hidden)?'':' class="dn"').'>'.cfProtectFilename($fileName)."</p>\n";
}

/**
 * @desc Insert (echo) file checkbox for use with javascripts
 *
 * @param integer $efSeqFileNumber : associated file sequential number
 */
function efOutFileCheckBox($efSeqFileNumber){if(cfRGetVar('sendAllowed')) return '<input id="cf'.$efSeqFileNumber.'" type="checkbox" name="cb">';}

/**
 * @desc Return true if action panel should be displayed
 *
 * @return bool
 */
function efActionPanelDisplayed(){
	return (cfRGetVar('multipleDownloadAllowed')||cfRGetVar('uploadAllowed')||(cfRGetVar('treeViewDisplayed')));

}

/**
 * @desc Output HTML code for a foldable action panel including mdl, upload and folders tree
 *
 * @param string $postion: 'right' or 'left'
 * @param string $width: width of panel
 * @param array $options: extra options:
 * 				'shadows': true to add shadows around blocks
 */
function efActionPanel($postion,$width=false,$options=false){
	global $tree;

	if(0) $tree=new treeViewFolder();
	$width='';
	$shadows=isset($options['shadows']);
	$panelExtraHTML=isset($options['panelExtraHTML'])?$options['panelExtraHTML']:'';

	$icons='<div style="'.outCssOpacity(50).';margin-top:0.4em">';
	if(cfRGetVar('multipleDownloadAllowed')) $icons.=outImage(outIcon('mdl'));
	if(cfRGetVar('uploadAllowed')) $icons.=outImage(outIcon('uploadMult'));
	if(cfRGetVar('treeViewDisplayed')) $icons.=outImage(outIcon('treeView'));
	$icons.='</div>';

	// Toggle collapse control
	if($postion=='right') echo outCollapsableTD('actionsPanel',$postion,array('innerHTML'=>$icons,'class'=>'vBanner'));

	/**
	 * Action panel TD
	 */
	echo '<td '.$panelExtraHTML.' style="vertical-align:top;padding:0;overflow:hidden'.((cfRGetVar('actionsPanel')?'':';display:none')).'" id="actionsPanel" class="vBanner">';

	echo '<div id="actionsPanel_colTDD1" style="overflow:hidden;position:relative"><div id="actionsPanel_colTDD2">';

	/*
	***************************************************************************************************************************
	* Multiple downloads
	***************************************************************************************************************************
	*/
	if(cfRGetVar('multipleDownloadAllowed')) {
		//if($shadows) outShadowBefore($width);
		echo outMultipleDownloadList('frame2',false,'vertical',false,'height:11em;margin-top:0;width:'.$width.(($shadows)?';margin-bottom:0':''));
		//if($shadows) outShadowAfter();
	}


	/*
	***************************************************************************************************************************
	* Upload file form
	***************************************************************************************************************************
	*/
	if(cfRGetVar('uploadAllowed')/* && cfFileRights(cfRGetVar('efCurrentDirectory'),'upload')*/){
		//if($shadows) outShadowBefore($width);
		echo outDivFrame('frame2','id="explorerUploadDiv"','width:'.$width.(($shadows)?';margin-bottom:0':'')).'<div class="frame2Header">'.outImage(outIcon('upload'),false,false,'margin-right:1em; vertical-align:middle');
		if(cfRGetVar('fileTypeFilter')==array('image')) echo cfCaption('explorerUploadImage');
		else echo cfCaption('uploadBoxTitle');
		echo '</div>';
		echo outUploadForm(11,'refreshPage','enabled',false,false,array('compact'=>true));
		echo "</div>\n";
		//if($shadows) outShadowAfter();
	}

	/*
	***************************************************************************************************************************
	* Directory browsing
	***************************************************************************************************************************
	*/
	if(cfRGetVar('treeViewDisplayed')) {
		//if($shadows) outShadowBefore($width);
		if(!efFilesListIsResource()) {
			$tree->syncSelectPath=false;
			$tree->sync='async';
		}
		echo $tree->HTML('boxSizing frame2',false,false,'height:100px;margin-bottom:0;width:100%'.(($shadows)?';margin-bottom:0':''));
		//if($shadows) outShadowAfter();
	}
	echo '</div></div></td>';

	// Toggle collapse control
	if($postion=='left')  echo outCollapsableTD('actionsPanel',$postion,array('innerHTML'=>$icons,'postToggleJS'=>"if(dgi('mDlList')){dd.elements.mDlList.del();ADD_DHTML('mDlList'+NO_DRAG+RECEIVER+AUTOSIZE)}"));

	if(!isset($options['fixedWidth'])){
?>
<script type="text/javascript">
$('actionsPanel').e.setResizable($('actionsPanel_colTD'),'width',{'firstChildWA':1,'min':100,'max':'winMe.getInnerWidth()/3','cursor':'pointer','onresize':'winResized();$(\'mainTree_outer\').style.width=\'100%\''});
</script>
<?php
	}
}

/**
 * @desc Remote file sharing form insertion and processing (all is async)
 *
 */
function efMailoutProcess($action, $completeFilename){
	// Only if user may share files
	if(!cfRGetVar('mailoutAllowed')) return;

	// Check file access rights
	if(!cfFileRights($completeFilename,'download')) $out=cfAsyncJSaction("javascript:alert('".str_replace("'",'',cfCaption('explorerErrorFileSend'))."')");
	$out='';

	// Insert email entry form
	if($action=='mailoutShowRemoteForm'){
		if(cfIsMobile())
			$out=mefMailoutForm($completeFilename);
		else
			$out=cfAsyncJSaction("var db=new DialogBox({'icon':'".outIcon('mailOut')."','title':'".cfCaption('sendByMail')."','text':'<br>".cfCaption('explorerFileSendReceiver')."','input':'".cfGGetVar('mailoutDefaultRecipient')."','buttons':{'ok':'OK','cancel':'CANCEL'},'mask':true},'mailoutProceed',{'inputValid':mailoutInputValid,'extraInput':{'mailoutTitle':\"".cfCaption('genTitle')."\",'mailoutMessage':\"".cfCaption('explorerFileSendMessage')."\"}});db.show()");
	}

	// Send email
	if($action=='mailoutSend'){
		$options=array();

		$destMail=$_POST['data4'];
		cfGSetVar('mailoutDefaultRecipient',$destMail);
		require_once(INCLUDE_DIR.'mailFunctions.php');

		wSession_write_close();
		ignore_user_abort(true);
		set_time_limit(0);

		// Parse subject / message
		@list($subject,$message)=@explode('|',cfUTF8Decode($_POST['data5']));
		$options=array('subject'=>@$subject,'message'=>@$message);

		// Do send mail
		$result=@Mailout::sendFile($completeFilename,$destMail,$options);

		// Process result
		if($result===true) $message=cfCaption('sendMailOk');
		else {
			$message=cfCaption('sendMailError',$result);
			$message=str_replace(array("\r","\n","\0","<p>","</p>"),' ',$message);
		}
		$out=cfAsyncJSaction('try{mailoutResult('.(($result===true)?'1':'0').',"'.$message.'")}catch(err){}');

	}

	cfAsyncHeader();
	echo $out;
	die(cfAsyncFooter());
}


/*
***************************************************************************************************************************
* Image preview class
***************************************************************************************************************************
*/
class efPreview{
	// See efUpdatePreview() for POST processing and async refresh


	/**
	 * @desc HTML code for hidden fields needed by preview update, next and previous scripts
	 *
	 * @param string $previewFileName : file name of currently displayed preview
	 */
	function JSDefinition(){
		$previewFileName=cfRGetVar('');
		if($previewFileName) $np=efGetNextAndPrevious($previewFileName);
		$out='var preview=new PreviewImage();';
		if(!cfRGetVar('fileViewAllowed')) $out.='preview.filename="'.cfUTF8Encode($previewFileName).'";';
		if(!cfRGetVar('fileViewAllowed')) $out.='preview.fileViewAllowed=0;';
		$out.='preview.commentsEdit='.(int)cfRGetVar('efEditMode').';';
		return $out;
	}

	/**
	 * @desc Display viewed file title
	 *
	 * @param string $previewFileName : file name of currently displayed preview
	 * @return string : HTML Code
	 */
	function title(){
		$previewFileName=@$_ENV['weezoResEfFiles'][efDirGetKeyFromFileNumber($_ENV['weezoResEfFiles'], 0+cfRGetVar('activePreview'),'file')]['name'];
		return '<span id="previewTitle">'.cfUTF8Encode(cfFileWithoutExtension($previewFileName))."</span>\n";
	}

	/**
	 * @desc Insert preview Image for explorer/photo resources
	 *
	 * @param string $previewCompleteFileName : filename of file to display
	 * 				(set to false not to preload image - image will be loaded on page load)
	 * @param string $autoSizeBaseWidth : width of outer frame when autosizing preview
	 * @param string $extraStyle: extra style of outer frame
	 *
	 * @return HTML code
	 */
	function image($extraStyle=''){
		// Outer frame, used as drop target, vertical centering, and actual image size for automatic size
		$out='<div id="previewOuterFrame" '.(($extraStyle)?' style='.$extraStyle.'" ':'').' onmousemove="preview.ctrlsShow()" onmouseout="preview.ctrlsMouseOut()">';

		$out.=outImage(outIcon('photoPrev'),false,'id="previewPrevBt" onclick="preview.clickPrev()"','position:absolute;top:50%;left:20px;margin-top:-32px;visibility:hidden;z-index:100');
		$out.=outImage(outIcon('photoNext'),false,'id="previewNextBt" onclick="preview.clickNext()"','position:absolute;top:50%;right:20px;margin-top:-32px;visibility:hidden;z-index:100');

		// Comments
		if(cfRGetVar('commentsAllowed')){
			$editMode=cfRGetVar('efEditMode');
			// Comments display frame
			$out.='<div id="efCommentsFrame" style="display:'.($editMode?'none':'block').';z-index:99;text-align:center"></div>';
			// Comments edit frame
			$out.='<div id="efCommentsForm" style="display:'.($editMode?'block':'none').'">';
			// Comments caption
			$out.='<div style="width:100%;text-align:left;color:black">'.cfCaption('genComments').'</div>';
			// Textarea
			$out.='<textarea id="efCommentsTextArea" class="input" style="height:4em;width:100%;overflow:auto" onkeydown="wl.stopPropagation(event)" onclick="wl.stopPropagation(event)"></textarea><br>';
			// Save button
			$out.=outButton(cfCaption('genSave'),'javascript:preview.commentsSave()',outIcon('save'));
			$out.="</div>\n";
		}
		// Rating
		$out.='<div id="previewRatingOuter" style="position:absolute;left:4px;bottom:5px;z-index:10"></div>';
		$out.="</div>\n";

		// Add image outer frame to draggable items list
		cfDragAddItem("previewOuterFrame",'+RECEIVER_ONLY','drec.elements.previewOuterFrame.highlightFunc=function(a,b){if(b) setAlpha(a,0.5); else setAlpha(a,1);}');

		return $out;
	}

	/**
	 * @desc Toggle slide show button
	 * @return string : HTML Code
	 */
	function toggleSlideshowBt($showCaption=true){
		$out = outButton((($showCaption)?cfCaption('explorerSlideShowStart'):''),'javascript:preview.toggleSlideshow();',outIcon('sshow'),((!$showCaption)?cfCaption('explorerSlideShowStart'):''),'slideShowStartButton');
		$out.= outButton((($showCaption)?cfCaption('explorerSlideShowStop'):''),'javascript:preview.toggleSlideshow();',outIcon('stop'),((!$showCaption)?cfCaption('explorerSlideShowStop'):''),'slideShowStopButton','style="display:none"');
		return $out;
	}

	/**
	 * @desc Preview image rotation buttons
	 * @param string $extraHTML : extra HTML
	 * @return string : HTML Code
	 */
	function rotateBts($extraHTML=false){
		return;
		$out='<span '.$extraHTML.'>'.outButton(false,'javascript:preview.rotateLeft();',outIcon('rotL'),cfCaption('viewRotateLeft'));
		return $out.outButton(false,'javascript:preview.rotateRight()',outIcon('rotR'),cfCaption('viewRotateRight')).'</span>';
	}

	/**
	 * @desc Insert previous preview button
	 *
	 * @param boolean $showCaption : show caption in button
	 * @return string : HTML Code
	 */
	function previousBt($showCaption=false){
		return outButton((($showCaption)?cfCaption('explorerPrevious'):''),'javascript:preview.goPrev()',outIcon('prev'),(($showCaption)?'':cfCaption('explorerPrevious')),'prevImg');
	}

	/**
	 * @desc Insert next preview button
	 *
	 * @param boolean $showCaption : show caption in button
	 * @return string : HTML Code
	 */
	function nextBt($showCaption=false){
		return outButton((($showCaption)?cfCaption('explorerNext'):''),'javascript:preview.goNext()',outIcon('next'),(($showCaption)?'':cfCaption('explorerNext')),'nextImg');
	}

	/**
	 * @desc View Image button
	 *
	 * @param boolean $showCaption : show caption in button
	 * @return string : HTML Code
	 */
	function viewBt($showCaption=false,$fullscreen=false){
		if(!cfRGetVar('fileViewAllowed')) return '';
		return outButton((($showCaption)?cfCaption('explorerClickToEnlarge'):''),"javascript:preview.view(".(($fullscreen)?"undefined,1":"").")",outIcon('fscreen'),(($showCaption)?'':cfCaption('explorerClickToEnlarge')));
	}

	/**
	 * Code for download and zip download add
	 *
	 * @param bool $showCaption: true to show captions in buttons
	 * @return string
	 */
	function downloadShortcut($showCaption=false){
		if(!cfRGetVar('downloadShortcut')) return '';
		return outButton((($showCaption)?cfCaption('photoDownload'):''),'javascript:preview.dl()',outIcon('dl'),(($showCaption)?'':cfCaption('photoDownload')));
	}

	/**
	 * @desc Download & zip download button
	 *
	 * @return string : HTML Code
	 */
	function downloadBts($showCaption=true){

		$out='';
		if(cfRGetVar('downloadShortcut')) {
			$out.=outButton((($showCaption)?cfCaption('genDownload'):''),'javascript:preview.dl()',outIcon('dl'),cfCaption('photoDownload')).'&nbsp;&nbsp;';
			if(cfGGetVar('bittorrentEnabled') && cfRGetVar('bittorrentEnabled')) $out.=outButton('','javascript:preview.btDl()',outIcon('bittorrent'),cfCaption('photoDownload').' ('.cfCaption('bittorrent').')');
		}
		if(cfRGetVar('multipleDownloadAllowed')) $out.=outButton('','javascript:preview.mAdd()',outIcon('mdl'),cfCaption('explorerMultipleDownloadAdd'));
		return $out;
	}

	/**
	 * @desc Toggle comments edition button
	 *
	  * @return string : HTML code
	 */
	function editCommentsBt(){
		if(!cfRGetVar('commentsAllowed')) return;
		//edit image's comments button
		if (cfRGetVar('efEditMode')) return outButton(cfCaption('explorerStopEditMode'),'javascript:preview.commentsToggle()',outIcon('editNo'),false,'editStopBt').outButton(cfCaption('explorerEditMode'),'javascript:preview.commentsToggle();',outIcon('edit'),false,'editStartBt','style="display:none"');
		// Stop edit comments button
		return outButton(cfCaption('explorerStopEditMode'),'javascript:preview.commentsToggle()',outIcon('editNo'),false,'editStopBt','style="display:none"').outButton(cfCaption('explorerEditMode'),'javascript:preview.commentsToggle();',outIcon('edit'),false,'editStartBt');
	}

	/**
	 * @desc Add code for keyboard shortcuts
	 *
	 */
	function setKeyboardShortcuts($scriptTags=false){
		return outKeyHandler('D',
			array('37'=>'preview.clickPrev()',
			'40'=>'preview.clickNext()',
			'37'=>'preview.clickPrev()',
			'34'=>'preview.clickNext()',
			'39'=>'preview.clickNext()',
			'32'=>'preview.clickNext()',
			'13'=>'preview.view()')
			,$scriptTags
		);
	}
}


/*
***************************************************************************************************************************
* list/icons files view functions
***************************************************************************************************************************
*/

/**
 * @desc Return HTMl code for files layout selection (list/icons/resource)
 * @param array $options resourceIcon, resourceCaption: icon & caption for resource layout button; noResourceLayout: set to skip resource layout selector
 *
 * @return string
 */
function efFilesListSelector($options=array()){
	$out='';
	$fL=cfRGetVar('filesLayout');

	// "list" layout
	$capt=cfCaption('genDisplay').' - '.cfCaption('filesAndFolders');
	$out.=outButton(false,'javascript:filesLayoutSet(\'list\')',outIcon('displayList'),$capt,'filesLayout_list',false,false,$fL==='list');

	// "icons" layout
	$capt=cfCaption('genDisplay').' - '.cfCaption('explorerIcons');
	$out.=outButton(false,'javascript:filesLayoutSet(\'icons\')',outIcon('displayIcons'),$capt,'filesLayout_icons',false,false,$fL==='icons');

	// "resource" layout
	if(!@$options['noResourceLayout']){
		$capt=cfCaption('genDisplay').' - '.((isset($options['resourceCaption']))?$options['resourceCaption']:cfCaption('genThumbnails'));
		$ico=(isset($options['resourceIcon']))?$options['resourceIcon']:outIcon('displayFull');
		$out.=outButton(false,'javascript:filesLayoutSet(\'resource\')',$ico,$capt,'filesLayout_resource',false,false,$fL==='resource');
	}
	return $out;
}

/**
 * @desc Insert Javascript vars (captions and icons)
 *
 */
function efFilesListJSVars(){
?>
var efBtDl=<?php echo (int)(cfGGetVar('bittorrentEnabled') && cfRGetVar('bittorrentEnabled'));?>,
filesListDisplayed="<?php echo efFilesListType();?>",
thumbnailsSizeFactor=<?php echo (int)cfRGetVar('thumbnailsSizeFactor');?>;
efCaptView="<?php echo stripslashes(cfCaptionJS('genView'));?>",
efCaptPlay="<?php echo stripslashes(cfCaptionJS('explorerAudioPlay'));?>",
efCaptRen="<?php echo stripslashes(cfCaptionJS('explorerRename'));?>",
efCaptEdit="<?php echo stripslashes(cfCaptionJS('genEdit'));?>",
efCaptDl="<?php echo stripslashes(cfCaptionJS('genDownload'));?>",
efCaptBtDl="<?php echo stripslashes(cfCaptionJS('genDownload').' ('.cfCaptionJS('bittorrent').')');?>",
efCaptMAdd="<?php echo stripslashes(cfCaptionJS('explorerMultipleDownloadAdd'));?>",
efCaptSup="<?php echo stripslashes(cfCaptionJS('genDelete'));?>",
efCaptExe="<?php echo stripslashes(cfCaptionJS('explorerExecute'));?>",
efCaptShare="<?php echo stripslashes(cfCaptionJS('doShare'));?>",
efCaptOpen="<?php echo stripslashes(cfCaptionJS('genOpen'));?>",
efCaptNoFile="<?php echo stripslashes(cfCaptionJS('explorerEmptyDir'));?>",
efCaptMailout="<?php echo stripslashes(cfCaptionJS('sendByMail'));?>",

efIcoDl="<?php echo outIcon('dl2');?>",
efIcoBtDl="<?php echo outIcon('bittorrent');?>",
efIcoView="<?php echo outIcon('view');?>",
efIcoPlay="<?php echo outIcon('play');?>",
efIcoSup="<?php echo outIcon('del');?>",
efIcoMAdd="<?php echo outIcon('mdl');?>",
efIcoRen="<?php echo outIcon('ren');?>",
efIcoEdit="<?php echo outIcon('edit');?>",
efIcoExe="<?php echo outIcon('exec');?>",
efIcoShare="<?php echo outIcon('share');?>",
efIcoMailout="<?php echo outIcon('mailOut');?>",

efOverMus="<?php echo outIcon('overMus');?>",
efOverVid="<?php echo outIcon('overVid');?>"
efFolderIcon="<?php echo outIcon('fi/folder');?>",
efFolderIconM="<?php echo outIcon('fi/med/folder');?>",
efFolderIconB="<?php echo outIcon('fi/big/folder');?>",
efFolderIconVB="<?php echo outIcon('fi/vBig/folder');?>",
function onrightclick(e){return fL.onrightclick(e);}
		<?php
}

/**
 * @desc Display list or icons files layout
 * @param nodeId: id of node to insert in, or false to document write, or null just to update files list and not redraw
 * @param options:
 * 			'commitDraw':false=no (done by resource), true=yes, 'onload'=body.onload
 * 			'noScriptTag':true not to include <script> in sync
 * 			'thumbnails':integer>0 to inclue thumbnails of given max width
 * 			'noSortControls':bool
 * 			'crop':integer>0 to square thumbnails by croping them
 * 			'resourceLayout':'useInterval' || 'all' : set to indicate resource uses this function to display items
 * 				(all=all items displayed, 'useInterval': use thumbnailIndex and itemsPerPage resource-values
 *
 */
function efFilesList($nodeId=false,$options=array()){
	if(efFilesListIsResource() && !cfRGetVar('resourceLayout')) return;

	// Default options
	$options=$options+array('commitDraw'=>true,'dir'=>false);

	// Set thumbnail size
	$tnw=efThumbnailsMaxWidth();

	// Sync display: insert required vars
	if(!cfIsAsync()){
		if(!isset($options['noScriptTag'])) {?><script type="text/javascript"><?php }
		efFilesListJSVars();
	}

	$nb=0;
	$drag=1;

	// Set JS Array options
	$efFilesJSArrayOptions=array();
	if(isset($options['thumbnails'])) $efFilesJSArrayOptions+=array('thumbnail'=>$options['thumbnails']); // Include URL to thumbnail (value = thumbnails width)
	if(isset($options['crop'])) $efFilesJSArrayOptions+=array('crop'=>$options['crop']); // Crop thubnails to square them
	if(cfRGetVar('filesLayout')=='icons') $efFilesJSArrayOptions+=array('mediumIcon'=>1,'thumbnail'=>$tnw); // Include
	if(efFilesListIsResource()||efFilesListType()=='customIcons') $efFilesJSArrayOptions+=array('thumbnail'=>$tnw);

	if(/*efFilesListIsResource() && */cfRGetVar('resourceLayout')=='useInterval') $efFilesJSArrayOptions+=array('useInterval'=>1);

	// Multimedia resource: include mime (sub)type in array to separate file types
	if(is_array(cfRGetVar('fileTypeFilter')) && count(cfRGetVar('fileTypeFilter'))>1) $efFilesJSArrayOptions+=array('includeType'=>1);

	// Get JS array containing files properties
	$files=efFilesJSArray(($options['dir'])?'all':'files',$efFilesJSArrayOptions);

	// Async update files list
	if(cfIsAsync()) {
		echo cfAsyncXMLJSaction('filesListDisplayed="'.efFilesListType().'";thumbnailsSizeFactor='.(int)cfRGetVar('thumbnailsSizeFactor'));
		echo cfAsyncXMLJSaction('fL=new FilesList('.$files.',"'.cfRGetVar('filesLayout').'","'.$nodeId.'",'.$tnw.',"'.cfUTF8Encode(cfResourceRelativePath(cfRGetVar('efCurrentDirectory'))).'")');
		if($nodeId!==null) echo cfAsyncXMLJSaction('fL.draw()');
	}
	// 1st sync draw: create FilesList bject
	else{
		echo 'var fL=new FilesList('.$files.',"'.cfRGetVar('filesLayout').'","'.$nodeId.'",'.$tnw.',"'.cfUTF8Encode(cfResourceRelativePath(cfRGetVar('efCurrentDirectory'))).'");';

		/**
		 * Sort controls
		 */
		if(!@$options['noSortControls']){

			echo "wl.sortControls='";
			// Empty space for left-side icons
			echo '<tr class="frame2Header"><td style="width:20px">&nbsp;</td>';
			echo '<td style="text-align:left">';
			// Rating
			if(cfRGetVar('ratingAllowed') && cfGGetVar('ratingAllowed')) echo '<div style="float:right;vertical-align:bottom">'.cfCaption('rating').outImage(outIcon('sup'),'javascript:srt(\\\'rating\\\',\\\'asc\\\');',false,'margin-left:5px;'.((cfGetBrowser()=='ie')?'position:relative;top:-6px':'')).outImage(outIcon('sdown'),'javascript:srt(\\\'rating\\\',\\\'desc\\\');',false,((cfGetBrowser()=='ie')?'position:relative;top:-6px':'')).'</div>';
			// File name
			echo cfCaption('explorerFilename').outImage(outIcon('sup'),'javascript:srt(\\\'name\\\',\\\'asc\\\');',false,'margin-left:5px;'.((cfGetBrowser()=='ie')?'position:relative;top:-6px':'')).outImage(outIcon('sdown'),'javascript:srt(\\\'name\\\',\\\'desc\\\');',false,((cfGetBrowser()=='ie')?'position:relative;top:-6px':''));
			echo '</td>';
			// File size
			echo '<td style="text-align:right;width:40px;white-space:nowrap">&nbsp;'.cfCaption('genSize').outImage(outIcon('sup'),'javascript:srt(\\\'size\\\',\\\'asc\\\');',false,'margin-left:5px;'.((cfGetBrowser()=='ie')?'position:relative;top:-6px':'')).outImage(outIcon('sdown'),'javascript:srt(\\\'size\\\',\\\'desc\\\');',false,((cfGetBrowser()=='ie')?'position:relative;top:-6px':'')).'</td>';

			// File modification date header
			echo '<td style="text-align:right;width:40px;white-space:nowrap">&nbsp;'.cfCaption('explorerDate').outImage(outIcon('sup'),'javascript:srt(\\\'date\\\',\\\'asc\\\');',false,'margin-left:5px;'.((cfGetBrowser()=='ie')?'position:relative;top:-6px':'')).outImage(outIcon('sdown'),'javascript:srt(\\\'date\\\',\\\'desc\\\');',false,((cfGetBrowser()=='ie')?'position:relative;top:-6px':'')).'</td>';
			echo "<tr>';";
		}


		if($options['commitDraw']==='onload' && $nodeId) echo 'connect(D.body,"load",function(e){fL.draw()}, true);';
		elseif($options['commitDraw']) echo 'fL.draw();';

		if(!isset($options['noScriptTag'])) echo '</script>';
	}
}

/**
 * @desc Return true if resource specific layout is displayed
 *
 * @return bool
 */
function efFilesListIsResource(){return cfRGetVar('filesLayout')=='resource';}

/**
 * @desc Return files layout type
 *
 * @return string: list, icons, resource
 */
function efFilesListType(){return cfRGetVar('filesLayout');}

/**
 * @desc Async update files list/icons display
 *
 */
function efFilesListAsyncUpdate(){
	if((efFilesListIsResource()&&!cfRGetVar('resourceLayout')) || !cfIsAsync() || (!isset($_POST['data3']) && !isset($_POST['setCombinedLayout']))) return;

	$action=isset($_POST['data3'])?$_POST['data3']:'setCombinedLayout';

	// Check if command is a supported asynchronous command
	$asyncUpdateCommands=cfArrayValuesToKeys(array('goDirect','filesLayoutSet','srt','efTNSize','goThumbnail','changeItemsPerPage','setCombinedLayout'));
	if(isset($asyncUpdateCommands[$action])){
		cfAsyncHeader();
		if($action=='goDirect'){
			// Video thumbnails generation & monitoring
			if(isset($_ENV['thumbnailVideoNeeded']) && count($_ENV['thumbnailVideoNeeded'])) efMakeVideoThumbnails($_ENV['thumbnailVideoNeeded']);

			// Images thumbnails generation & monitoring
			if(isset($_ENV['thumbnailPhotoNeeded']) && is_array($_ENV['thumbnailPhotoNeeded'])) efMakePhotoThumbnails($_ENV['thumbnailPhotoNeeded']);
		}

		// Update client-side thumbnails size
		if($action=='efTNSize') echo cfAsyncXMLJSaction('tnWidth='.efThumbnailsMaxWidth());

		// Async update tree
		if(($action=='goDirect'||$action=='srt') && cfRGetVar('treeViewDisplayed')) {
			global $tree;
			echo cfAsyncXMLJSaction('D.comForm.data1.value="'.cfUTF8Encode(cfResourceRelativePath(cfRGetVar('efCurrentDirectory'))).'"');
			echo cfAsyncXMLJSaction($tree->updateAndSelectScript(cfRGetVar('efCurrentDirectory')));
		}

		// Async update view (if not explicitely not-requested by JS)
		if(!@$_POST['filesListNoUpdate']){
			$options=cfRGetVar('cropThumbnails')?array('crop'=>cfRGetVar('cropThumbnails')):array(); // crop thumbnails
			/**
			 * Do update file list !
			 */
			efFilesList("noFileDiv",$options);
		}

		// Enable / disable upload form
		$dirPermissions=cfFileRights(cfRGetVar('efCurrentDirectory'));
		echo cfAsyncXMLJSaction('uploadFormEnable('.(int)((cfRGetVar('uploadAllowed') && $dirPermissions['upload'])).')');

		// Remove wait mask
		echo cfAsyncXMLJSaction('asyncSetPI(0)');

		// Refresh thumbnail interval selection box
		if($action=='changeItemsPerPage' || $action=='goThumbnail' || $action=='goDirect') echo outExplorerThumbnailsSelector(1,true);

		die(cfAsyncFooter());
	}
}
?>