<?php
/**
 * Sticky Notes functions (display, create, suppress)
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 2.0.0
 */

$SNOTES_COLORS=array('EE99B7','59B1C8','D1E94D','F5EE4F','BE82C0','FF6308');

/**
 * @desc Apply modifications to memory and to file
 *
 * @param array $sNotes: full sNotes definition
 */
function sNotesCommit($sNotes){
	cfMSetVar('weezoSNotes',$sNotes);
	cfWriteIniFile($sNotes,cfAppDataDir().'/stickyNotes.ini');
}

/**
 * @desc Update sticky notes position, delete sticky note or create a new one
 *
 */
function sNotesUpdate(){
	global $SNOTES_COLORS;

	// Set user rights (0 no, 1 view, 2 view+create, 3 view+create+delete
	$rights=(int)cfUGetVar('sNotes');

	if(!isset($_POST['sNotesAction'])) return ;
	cfAsyncHeader();

	// Retreive existing sticky notes
	$sNotes=cfMGetVar('weezoSNotes');
	// Process command
	if($rights>=2) switch ($_POST['sNotesAction']){
		case 'create':
			// Process args
			if(!($text=@$_POST['text'])) break;
			$text=htmlspecialchars($text);

			// Replace line breaks by <br>
			$text=str_replace(cfStrLBPattern($text),'<br>',$text);

			// Color
			$color=@$_POST['color']; if(!preg_match('/^[A-Za-z0-9]{6}$/',$color)) $color=$SNOTES_COLORS[0];

			// Position
			$x=@$_POST['x']; $y=@$_POST['y'];

			$id=$sNotes['maxId']+1;	$sNotes['maxId']=$id;
			$sNotes[$id]=array('text'=>$text,'color'=>$color,'x'=>$x,'y'=>$y);

			// Author
			$sNotes[$id]['author']=cfUGetVar('id');

			// Author name
			$sNotes[$id]['authorName']=cfUGetVar('name').((cfUGetVar('pseudo'))?' ('.cfUGetVar('pseudo').')':'');

			// timestamp
			$sNotes[$id]['time']=time();

			// Recipients
			if(@$_POST['groups']=='all') $sNotes[$id]['recipient']='all';
			elseif (@$_POST['groups']=='this') $sNotes[$id]['recipient']=cfUGetVar('id');
			else{
				$users=cfMGetVar('weezoUsers');
				if(cfUGetVar('administrator') && isset($users[@$_POST['groups']])) $sNotes[$id]['recipient']=$_POST['groups'];
				else $sNotes[$id]['recipient']='all';
			}

			// Save to memory & to file
			sNotesCommit($sNotes);

			// Finish browser-side creation
			echo cfAsyncXMLJSaction('sNotesCreated("'.$id.'",'.$x.','.$y.')');

			// Log to app
			cfLog(cfCaption('explorerFileSendMessage',false,false,false,true).' ('.cfUGetVar('name').') : '.cfStrTruncate(cfUTF8Decode(str_replace('<br>',' ',htmlspecialchars_decode($text)),true,false,false),40),LOG_INF);
			break;
		// Update text of an existing sticky note
		case 'updateText':
			// Check sNote exists and user may modify it (e.g. is creator)
			if((!@$sNotes[@$_POST['id']]) || $sNotes[$_POST['id']]['author']!=cfUGetVar('id')) break;
			// Process text
			if(!($text=@$_POST['text'])) break;
			//$text=htmlspecialchars($text);
			$sNotes[$_POST['id']]['text']=str_replace(cfStrLBPattern($text),'<br>',$text);
			// Save
			sNotesCommit($sNotes);
			break;
		// Delete sticky note
		case 'delete':
			if($rights!=3 || !@$sNotes[$_POST['id']]) break;
			unset($sNotes[$_POST['id']]);
			sNotesCommit($sNotes);
			break;
		// Update position
		case 'updatePos':
			if(!is_array($_POST['x'])) break;
			foreach ($_POST['x'] as $id=>$x) if(isset($sNotes[$id]) && $y=@$_POST['y'][$id]){
				if(!is_numeric($x)||!is_numeric($y)) continue;
				$sNote=$sNotes[$id];
				unset($sNotes[$id]);
				$sNote['x']=max(0,min(1,$x));
				$sNote['y']=max(0,min(1,$_POST['y'][$id]));
				$sNotes[$id]=$sNote;
			}
			sNotesCommit($sNotes);
	}
	die(cfAsyncFooter());
}

/**
 * @desc Display sticky notes and New sticky note control
 *
 */
function sNotesDisplay(){
	global $SNOTES_COLORS;

	if(!cfUGetVar('sNotes')) return ;

	// New sticky note control
	if(cfUGetVar('sNotes')>1){
		if(cfBGetVar('png')) {
	?>
	<div id="sNotesNewBt" onclick="sNotesNew()"></div>
	<?php }
		else echo outImage(outIcon('sNotesCreate'),'javascript:sNotesNew()','id="sNotesNewBt"');
	}
?>
<script type="text/javascript">
var sNotesC="<?php echo $SNOTES_COLORS[0];?>";
var sNotes={<?php
	$nb=0;
	$rights=(int)cfUGetVar('sNotes');
	$myId=cfUGetVar('id');

	// Generate a sticky notes JS array
	foreach (cfMGetVar('weezoSNotes') as $id=>$sNote) if(@$sNote['text']){
		// Check if this user may see note
		if(@$sNote['recipient']!=='all' && @$sNote['recipient']!==$myId) continue;
		// Check if this user may edit note
		$edit=(int)($rights>1 && $myId==$sNote['author']);

		echo (($nb++>0)?',':'').'"'.$id.'":Array(';
		echo '"'.addslashes($sNote['text']).'","'.addslashes(cfUTF8Encode(@$sNote['authorName'])).'<"+"br>'.addslashes(((@$sNote['time'])?date(cfCaption('_FILE_DATE_FORMAT'),$sNote['time']):'-')).'","'.$sNote['color'].'",'.$sNote['x'].','.$sNote['y'].','.$edit.')';
	}
?>}

// Move / moved functions
var sNotePosCmd={}
var sNoteDropT=0;
var sNoteZIWA=1;
function sNoteDrop(){
	if(wl.mask.shown) {
		wl.mask.moveAbove(dd.obj.div)
		return;
	}
	var wp=winPadding, id=dd.obj.id.substr(2),x,y;
	dd.obj.setZ(sNoteZIWA++)
	x=(dd.obj.x-wp.left)/(dd.getWndW()-wp.left-wp.right-dd.obj.w); x=Math.max(0,Math.min(1,x));
	y=(dd.obj.y-wp.top)/(dd.getWndH()-wp.top-wp.bottom-dd.obj.h); y=Math.max(0,Math.min(1,y));
	sNotes[id][2]=x;sNotes[id][3]=y;
	if(sNotePosCmd[id]) delete(sNotePosCmd[id]);
	sNotePosCmd[id]=Array(x,y);
	if(sNoteDropT) clearTimeout(sNoteDropT);
	sNoteDropT=setTimeout('sNoteDropCommit()',2000);
}
function sNoteDropCommit(){
	var cmd='sNotesAction=updatePos';
	for(var id in sNotePosCmd) cmd+='&x['+id+']='+sNotePosCmd[id][0]+'&y['+id+']='+sNotePosCmd[id][1];
	sendData(cmd)
	sNotePosCmd={};
}
function sNotesResetPos(){
	var x,y,wp=winPadding;
	for(var id in sNotes) {
		var x=sNotes[id][3],y=sNotes[id][4];
		dd.elements['sN'+id].moveTo(wp.left+(dd.getWndW()-wp.left-wp.right-dd.elements['sN'+id].w)*x,wp.top+(dd.getWndH()-wp.top-wp.bottom-dd.elements['sN'+id].h)*y);
	}
}

// sNode HTML item
function sNotesCreateItem(id,txt,auth,col,x,y,edit){
	var sN=D.createElement('div');
	var nid=((id)?'sN'+id:'sNotesNew');
	if(!auth) auth='';
	sN.setAttribute('id',nid);

	sN.setAttribute('class','sNotes'); sN.setAttribute('className','sNotes');
	sN.innerHTML='<'+'div class="sNotesCol" style="background-color:#'+col+'"><'+'/div><'+'div class="sNotesMask"><'+'/div>';
	if(id){
		if(<?php if(cfUGetVar('sNotes')==3) echo 1; else echo 0;?>) sN.innerHTML+='<'+'div class="close" onmousedown="sNotesDel('+id+')" title="<?php echo cfCaptionJS('genDelete');?>"><'+'/div>';
		sN.innerHTML+='<'+'div class="text" style="cursor:default">'+txt+'<'+'/div>';
		sN.innerHTML+='<'+'div class="author">'+auth+'<'+'/div>';
		if(edit) sN.innerHTML+='<'+'div class="edit" onmousedown="sNotesEdit('+id+')" title="<?php echo cfCaptionJS('genEdit');?>"><'+'/div>';
	}
	if(!id) {
		sN.innerHTML+='<'+'textarea id="sNotesNText" class="text"><'+'/textarea>';
		sN.innerHTML+='<'+'div id="sNotesCancel" class="close" onmousedown="sNotesCancel()" title="<?php echo cfCaptionJS('genCancel');?>"><'+'/div>';
		sN.innerHTML+='<'+'div id="sNotesCreateBt" class="create" onmousedown="sNotesCreate()" title="<?php echo cfCaptionJS('genOK');?>"><'+'/div>';
		sN.innerHTML+='<'+'img id="sNotesGroupsBt" class="groups" onmousedown="sNotesSelGroup()" src="<?php echo outIcon('users');?>" title="<?php echo cfCaptionJS('explorerFileSendReceiver');?>">';
		sN.innerHTML+='<'+'div id="sNotesColorPicker"><?php
		foreach ($SNOTES_COLORS as $v)	echo '<\'+\'fieldset style="background:#'.$v.'" onmousedown="sNotesSetC(\\\''.$v.'\\\')"><\'+\'b><\'+\'/b><\'+\'/fieldset>';
		?><'+'/div>';
	}
	D.body.appendChild(sN);
	ADD_DHTML(nid+ISICON+RESET_Z+(id?DRAGCLASS:'')+PICKDELAY+200);
	dd.elements[nid].setDropFuncInertia(sNoteDrop)
	var wp=winPadding;
	if(x||y) dd.elements[nid].moveTo(wp.left+(dd.getWndW()-wp.left-wp.right-dd.elements[nid].w)*x,wp.top+(dd.getWndH()-wp.top-wp.bottom-dd.elements[nid].h)*y);
	return sN;
}

// sNote creation functions
function sNotesCancel(kc){
	if(kc!=undefined && kc!=27) return;
	removeNode(dgi('sNotesNew'));
	wl.mask.hide(1);
	dd.elements['sNotesNew'].del();
	wl.removeKeycodeListener(D);
}
function sNotesSetC(c){dgi('sNotesNew').firstChild.style.backgroundColor='#'+c;sNotesC=c;dgi('sNotesNText').focus()}
function sNotesNew(){
	wl.mask.show(0,1);
	var sN=sNotesCreateItem(false,false,'',sNotesC);
	$(sN).e.center();	wl.mask.moveAbove(sN); fade(sN);wl.setKeycodeListener(D,sNotesCancel)
	dgi("sNotesNText").focus();
}
function sNotesCreate(){
	if(!(txt=dgi("sNotesNText").value)) return sNotesCancel();
	wl.mask.hide(1);

	var wp=winPadding;
	x=(dd.elements['sNotesNew'].x-wp.left)/(dd.getWndW()-wp.left-wp.right-dd.elements['sNotesNew'].w); x=Math.max(0,Math.min(1,x));
	y=(dd.elements['sNotesNew'].y-wp.top)/(dd.getWndH()-wp.top-wp.bottom-dd.elements['sNotesNew'].h); y=Math.max(0,Math.min(1,y));
	dd.elements['sNotesNew'].moveTo=false;
	dd.elements['sNotesNew'].del();
	removeNode(dgi('sNotesColorPicker'));
	removeNode(dgi('sNotesCreateBt'));
	removeNode(dgi('sNotesGroupsBt'));
	removeNode(dgi('sNotesCancel'));

	var txt2=dgi('sNotesNText').value.replace(new RegExp('<', "g"),'&lt;'); if(txt2.indexOf("\n")!=-1) txt2=txt2.replace(/[\n]/g,'<'+'br>'); else txt2=txt2.replace(/[\r]/g,'<br>');
	removeNode(dgi('sNotesNText'));
	dgi('sNotesNew').innerHTML+='<div class="text" style="cursor:default">'+txt2+'<'+'/div>';

	sendData('sNotesAction=create&text='+encodeURIComponent(txt)+'&color='+sNotesC+'&x='+x+'&y='+y+'&groups='+sNotesGroup);
}
function sNotesCreated(id,x,y){
	sNotes[id]=Array('','',x,y);
	if(<?php if(cfUGetVar('sNotes')==3) echo 1; else echo 0;?>) dgi('sNotesNew').innerHTML+='<'+'div class="close" onmousedown="sNotesDel('+id+')"><'+'/div><'+'div class="edit" onmousedown="sNotesEdit('+id+')"><'+'/div>';
	dgi('sNotesNew').id='sN'+id;
	ADD_DHTML('sN'+id+ISICON+RESET_Z+DRAGCLASS+PICKDELAY+200);
	dd.elements['sN'+id].setZ(sNoteZIWA++)
	dd.elements['sN'+id].setDropFuncInertia(sNoteDrop)
}
var sNotesGroup='all';
function sNotesSelGroup(){
	var cmi=[
		ContextMenuItem("<?php echo outIcon('users');?>","<?php echo cfCaption('genAll')?>","sNotesGroupSet('all')"),
		ContextMenuItem("<?php echo outIcon('user');?>","<?php echo htmlentities(cfUTF8Encode(cfUGetVar('name')),ENT_QUOTES)?>","sNotesGroupSet('this')")<?php
if(cfUGetVar('administrator')){
	echo ',ContextMenuItem("sep")';
	foreach (cfMGetVar('weezoUsers') as $id=>$user) if(!@$user['invisible'] && !@$user['administrator']){
		echo ',ContextMenuItem("'.cfExtGFX('/gfx/icons/'.$user['icon'],16,16).'","'.addslashes(cfUTF8Encode($user['name'])).'","sNotesGroupSet(\''.$id.'\',\''.cfExtGFX('/gfx/icons/'.$user['icon'],16,16).'\')")';
	}
}
?>
	];
	ContextMenuDropDown(dgi('sNotesGroupsBt'),cmi)
}
function sNotesGroupSet(groupId,ico){
	sNotesGroup=groupId;
	dgi('sNotesGroupsBt').src=(ico)?ico:((groupId=='all')?"<?php echo outIcon('users');?>":"<?php echo outIcon('user');?>");
}
// Edit
var sNotesEditedNode;
function sNotesEdit(id){
	var sN=dgi('sN'+id),ta=sN.getElementsByTagName('div');
	sNotesEditedNode=sN;
	wl.mask.show(0,1);	wl.mask.moveAbove(sN); wl.setKeycodeListener(D,sNotesEditKD)
	for(var e in ta) if(ta[e].className=='edit') removeNode(ta[e]);
	for(var e in ta) if(ta[e] && ta[e].className=='text') {
		var txt=ta[e].innerHTML.replace(/<br>/gi,"\n");
		removeNode(ta[e]);
		dgi('sN'+id).innerHTML+='<'+'textarea id="sNotesNText" class="text">'+txt+'<'+'/textarea><div id="sNotesUpdateBt" class="create" onmousedown="sNotesUpdate(0)" title="<?php echo cfCaptionJS('explorerAudioApply');?>"><'+'/div>';
		dgi('sNotesNText').focus();
	}
}
function sNotesEditKD(kc){if(kc==27) sNotesUpdate(1);}
function sNotesUpdate(cancel){
	wl.mask.hide();
	var txt2=dgi('sNotesNText').value.replace(new RegExp('<', "g"),'&lt;'); if(txt2.indexOf("\n")!=-1) txt2=txt2.replace(/[\n]/g,'<br>'); else txt2=txt2.replace(/[\r]/g,'<'+'br>');
	removeNode(dgi('sNotesNText'));
	removeNode(dgi('sNotesUpdateBt'));
	sNotesEditedNode.innerHTML+='<'+'div class="text" style="cursor:default">'+txt2+'<'+'/div><'+'div class="edit" onmousedown="sNotesEdit('+(sNotesEditedNode.id.substr(2))+')"><'+'/div>';
	dd.elements[sNotesEditedNode.id].setZ(sNoteZIWA++)
	sendData('sNotesAction=updateText&text='+encodeURIComponent(txt2)+'&id='+sNotesEditedNode.id.substr(2));
}

// delete
function sNotesDel(id){
	var n=dgi('sN'+id);
	wl.mask.hide();
	if(phpBrowser=='ie') sNotesDelProcess(id); else fade(n,1,0,5,'sNotesDelProcess('+id+')');
}
function sNotesDelProcess(id){
	removeNode(dgi('sN'+id));
	sendData('sNotesAction=delete&id='+id);
}


// Init
function sNotesInit(){
	if(dgi('sNotesNewBt') && W.winPadding){
		if(winPadding.bottom) dgi('sNotesNewBt').style.bottom=winPadding.bottom+10;
		if(winPadding.right) dgi('sNotesNewBt').style.right=winPadding.right+10;
	}
	var sN;
	for(var id in sNotes) {
		sN=sNotes[id];
		sNotesCreateItem(id,sN[0],sN[1],sN[2],sN[3],sN[4],sN[5]);
	}
}
sNotesInit();
</script>
<?php
}
?>