/**
 * Drag & Drop upload scripts
 * t: assoc. array of public functions
 * uploads: assoc. array of uploads (key is file name): for each upload:
 *			'HTMLID': unique id used for HTML controls,
 *			'file': file item
 *			'status': transfer status (see transferFunctions.php): 0 created, 3 running, 4 finished, 6 canceled, 8 failed)
 * status: see uploads status
 */

/**
 * Turn a node into upload drop receiver
 * @arg divId: id of node
 * @arg simple: set to true to directly upload single file
 */
function setUploadReceiver(divId,simple){
	var t={}, uploads={}, totalSize=0, totalSent=0, status=0, dropbox=$(divId), emptyDivHTML='', destDir, currentUploadId, isSimple=simple;
	var overwrite='ask'; // on file already existing: ask, skip, overwrite

	/**
	 * Upload item template
	 * @param uploadItem: upload item (see uploads array)
	 */
	var uploadItem = function(uploadItem){
		var label=uploadItem.file.name;
		var pb=dgi('uploadDDPBTemplate').innerHTML.replace(/CAPT/,wl.formatedSize(uploadItem.file.size)).replace(/PBID/g,'PB'+uploadItem.HTMLID);

		// If upload has started, update progress
		if(uploadItem.progress) pb=pb.replace(/width:0/,'width:'+uploadItem.progress)

		return '<tr id="TR'+uploadItem.HTMLID+'"><td><span style="position:relative"><img id="uploadFilesIco'+uploadItem.HTMLID+'" src="'+wl.fileExtIconSrc(label)+'" style="height:16px"><div class="uploadDDCancel" onclick="uploadDD[\''+divId+'\'].singleUploadCancel(\''+uploadItem.HTMLID+'\')" title="'+wl.capt.genCancel+'"></div></span>&nbsp;'+label+'</td><td>'+pb+'</td></tr>';
	}

	// Update HTML for uploads
	var updateUploadItemsHTML = function(){
		if(isSimple) return;
		var o='',gp=dgi('uploadDDTotalProgress');
		for(e in uploads) o+=uploadItem(uploads[e]);
		dropbox.innerHTML=(o)?'<table>'+o+'</table>':emptyDivHTML;
		if(gp) gp.innerHTML=(o)?(dgi('uploadDDPBTemplate').innerHTML.replace(/CAPT/,wl.formatedSize(totalSize)).replace(/PBID/g,'PBUploadDDTotal')):'';
	}

	// Update global progress bar
	var updateUploadTotalProgress = function(ts){
		if(isSimple) return;
		p=Math.round((ts*100)/totalSize);
		progressSetPerc('PBUploadDDTotal',p);
		progressSetText('PBUploadDDTotal',wl.formatedSize(ts)+' / '+wl.formatedSize(totalSize));
	}

	// Set upload status to canceled
	var cancelUpload = function(id){uploads[id].status=6}

	// Cancel all pending transfers
	t.cancelAll = function(){
		wl.asr.cancelAll();
		uploads={};
		totalSize=0; totalSent=0;
		updateUploadItemsHTML();
		t.updateButtonsState();
	}

	// Cancel a request
	t.singleUploadCancel = function(HTMLID){
		var e,st;
		$('TR'+HTMLID).e.remove();
		for(e in uploads) if(uploads[e].HTMLID==HTMLID){
			st=uploads[e].status;
			if(st==4) totalSent-=uploads[e].file.size; // If file was transmited, remove its size from total transmited
			totalSize-=uploads[e].file.size;
			delete uploads[e];
			if(st==3) processQueue(); // If upload was running, proceed with next upload
		}
		// No upload in list (may happend if user canceled the only upload), set default content
		if(count(uploads)==0) updateUploadItemsHTML();
	}

	// Hide Drag & Drop upload box, show regular upload box
	t.hide = function(){
		dgi('htmlUploadDDStuff').style.display='none';
		dgi('htmlUploadStuff').style.display='';
	}


	/**
	 * Start uploads
	 */
	t.start = function(){
		if(!count(uploads)) return;

		status=3; // Set global status to running
		t.updateButtonsState(); // Disable Send button

		updateUploadItemsHTML(); // Reset progress bars (usefull when files have already been transfered once)
		if(D.comForm) destDir=D.comForm.data1.value; // Set destination directory (so it doesn't change if user navigates to another directory)
		processQueue(); // Start uploads
		if(dgi('transfersIFrame')) setTimeout(function(){frames["transfersIFrame"].transfersRefresh(1)},500);

	}

	/**
	 * Browse uploads queue and start first not-started (/error/canceled) upload.
	 * If all downloads completed, set all transfers as finished;
	 */
	var processQueue = function(){
		for(var e in uploads) if(uploads[e].status==0) return setTimeout(function(){singleUploadStart(e)},1);

		// All downloads finished
		status=4;
		// Update cancel / start buttons
		t.updateButtonsState();
		// Reset status so uploads can restart
		for(var e in uploads) uploads[e].status=0;
	}

	/**
	 * Step 1/2: start uploading a file (send async request to check if upload is allowed, and to generate transfer object)
	 * @param uploadFileName: name of file to upload
	 */
	var singleUploadStart = function(uploadFileName,forceOverwrite){
		var ul=uploads[uploadFileName],args;

		ul.status=4;
		if(!isSimple) progressSetPerc('PB'+ul.HTMLID,0);
		currentUploadId=uploadFileName; // Store current download id (used to process after an overwrite confirmation)

		args={
			'uploadDD':divId,
			'createUploadTransfer':1,
			'overwriteConfirmChecked':overwriteConfirmChecked,
			'overwrite':(forceOverwrite)?'overwrite':overwrite,
			'uploadId':uploadFileName,
			'preUlDir':destDir,
			'preUlFile':uploadFileName
		}

		wl.asr.send(args,'uploadFlash.php');
	}


	/**
	 * Step 2/2: Proceed with upload
	 * transferId: set server-side transfer ID
	 * action: forbidden, skip, confirm
	 */
	t.singleUploadProceed = function(transferId,action,uploadFileName){
		var id=uploadFileName,ul=uploads[id];
		// Dest file protected
		if(action=='forbidden'){
			progressSetError(uploadId);
			ul.status=8;
			progressSetText('PB'+ul.HTMLID,wl.capt.transferError);
			t.processQueue();
		}
		// Dest file existing, skip
		if(action=='skip'){
			ul.status=6;
			progressSetText('PB'+ul.HTMLID,wl.capt.transferCanceled);
			t.processQueue();
		}
		// Confirm overwrite
		else if(action=='confirm'){
			var m=new DialogBox({'text': wl.capt.replaceFile,'title':ul.file.name,'icon':wl.ico.alert,'mask':1,'fade':1,'buttons':{'ok':wl.capt.genYes,'cancel':wl.capt.genNo},'check':overwriteConfirmChecked,'checkText':wl.capt.dontAskAnymore},'uploadDD["'+divId+'"].uploadOverwrite');
			m.show();
		}
		// cancel overwrite
		else if(action=='confirm' && !overwriteAsk && !overwriteDefault){
			ul.status=6;
			progressSetText('PB'+ul.HTMLID,wl.capt.transferCanceled);
			t.processQueue();
		}
		// proceed upload
		else{
			ul.status=3;
			ul.transferId=transferId;

			// Async upload file
			wl.asr.send(ul.file,'uploadFlash.php',undefined,{
				'uploadProgress':t.singleUploadProgress,
				'uploadCompleted':t.singleUploadCompleted,
				'uploadId':id,
				'requestHeaders':{
					'X-Upload-ID':id,
					'X-resId':resId,
					'X-Upload-transferId':transferId,
					'X-Upload-ulDir':destDir,
					'X-Upload-ulFile':ul.file.name
				}}
			);
		}
	}

	// Overwrite dialogBox result
	t.uploadOverwrite = function(result){
		overwriteConfirmChecked=(result['check'])?1:0;
		// User requested overwrite
		if(result.button=='ok') {
			if(overwriteConfirmChecked) overwrite='overwrite';
			singleUploadStart(currentUploadId,1);
		}
		// User refused overwrite
		else{
			if(overwriteConfirmChecked) overwrite='skip';
			cancelUpload(currentUploadId);
			if(dgi('PB'+currentUploadId)) progressSetText('PB'+currentUploadId,wl.capt.transferCanceled);
			processQueue();
		}
	}

	// XXX
	t.singleUploadError=function(errTxt){
		/*
		progressSetPerc('PB'+uploadId,0);
		progressSetError(uploadId);
		if(errTxt=='403') progressSetText('PB'+uploadId,"<?php echo cfCaption('explorerErrorFileSend2');?>");
		else progressSetText('PB'+uploadId,"<?php echo cfCaption('transfersFailed')?> ("+errTxt+")");
		uploadsArray[uploadId]['state']='error';
		if(errTxt=='IO err' && location.protocol=='https:') alert(httpsErrorCaption);
		else */
		processQueue();
	}

	// Called by async function on progress event
	t.singleUploadProgress = function(req,evt){
		if(isSimple) return;
		var id=req.opts.uploadId,gp=dgi('uploadDDTotalProgress'),p;
		if(evt.lengthComputable) {
			p=evt.loaded*100/evt.total;
			uploads[id].progress=p;
			progressSetPerc('PB'+uploads[id].HTMLID,Math.round(p));
			if(gp) {
				updateUploadTotalProgress(totalSent+evt.loaded);
			}

		}
	}

	// Called by async function on complete event
	t.singleUploadCompleted = function(req,evt){
		var id=req.opts.uploadId;
		if(evt.lengthComputable && !isSimple) progressSetPerc('PB'+uploads[id].HTMLID,100);
		totalSent+=uploads[id].file.size;
		updateUploadTotalProgress(totalSent);

		processQueue();
	}





	/**
	 * DRAG & DROP Handling
	 *
	 * Register dropbox as drag & drop receiver DIV, and process drag over, leave and drop
	 */

	// Drag out of dropbox
	t.dragleave = function(e){dropbox.e.className().remove('dropTargetOver')}

	// Drag over dropbox
	t.dragover = function(e){
		e.stopPropagation();
		e.preventDefault();
		if(!(status==3 && isSimple)) {
			dropbox.e.className().add('dropTargetOver')
			if(e.dataTransfer.dropEffect=='none') e.dataTransfer.dropEffect = 'copy'; // Chrome
		}
		return true;
	}

	// Files dropped
	t.dragdrop = function(e) {
		var f, i, files = e.dataTransfer.files, HTMLID, o;

		t.dragleave(e);
		e.stopPropagation(); e.preventDefault();

		// If a previous upload is finished, first clear the list
		if(status==4) t.cancelAll();

		if(!emptyDivHTML && !isSimple) emptyDivHTML=dropbox.innerHTML;

		// Browse dropped files
		var l="";
		for(i=0;i<files.length;i++) {
			f=files[i];
			if(f.size==0 || uploads[f.name]) continue; // If file size is null (may be dir) or upload already created, don't add
			if(uploadDDExtensionFilter && !uploadDDExtensionFilter[f.name.substr(f.name.lastIndexOf('.')+1)]) continue; // Skip unauthorized extensions
			if(uploadDDFileTypeFilter && !uploadDDFileTypeFilter[f.type.substr(0,f.type.indexOf('/'))]) continue; // Skip unauthorized extensions
			HTMLID='UL'+Math.random();
			uploads[f.name]={'HTMLID':HTMLID,'file':f,'status':0}

			if(!isSimple) dropbox.innerHTML+=uploadItem(uploads[f.name]);

			totalSize+=f.size;
			l+=', '+f.name;
			// Enable uploads start button and cancel buttons
		}

		// Simple upload: start upload now (if upload permitted and after user confirmation
		if(isSimple) {
			if(uploadAllowedDir && confirm(wl.capt.genConfirmCopy.replace(/%1/,l.substr(2)).replace(/%2/,wl.capt.basePath))) t.start(); else t.cancelAll();
		}
		else if(uploadAllowedDir){
			// (re) Generate div innerHTML
			updateUploadItemsHTML();
			// Update cancel / start buttons
			t.updateButtonsState();
			// Automatically start upload (if not already in progress)
			if(status!=3) t.start();
		}
	}

	// Update upload and cancel buttons state according to uploads list and uploads rights in current directory
	t.updateButtonsState = function(){
		if(count(uploads)){
			wl.button.enable('uploadDDCancelBt');
		}
		else {
			wl.button.disable('uploadDDCancelBt');
		}
	}

	// Safari bug Workaround: file can be dropped only once a drag has been started - deprecated: it now seems to work!
	/*
	t.safariWA = function(n){
		if(!dgi('safariWABt')) return;
		// Remove the off button if not needed anymore
		if(!n) {
			try{if(parent.safariDragWASet) $('safariWABt').e.remove()} catch(e){}
			D.body.ondragstart=function(e){return true} // Re-enable ondragstart function so safariWABt can be dragged
			return;
		}
		// Indicate to upper frame that a drag has once been done so there is no need to display the "off" button
		try{parent.safariDragWASet=1} catch(e){}
		$(n).e.remove();
	}
	*/

	// Register events
	dropbox.addEventListener("dragover", t.dragover, false);
	dropbox.addEventListener("dragleave", t.dragleave, false);
	dropbox.addEventListener("drop", t.dragdrop, false);

	// Safari bug workaround: remove "off" button if drag has already been done once for this tab.
	//t.safariWA();

	// Store item into an array so right one can be called by
	uploadDD[divId]=t;
	return t;
}
var uploadDD={};
