<?php
/**
 * Direct file link generation an management
 *
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.4
 */

/**
 * @desc Verify external authentication is not set to on, and if so, display incompatibility message
 *
 * @param $type: 'directLink', 'publishDownload', 'publishImage', 'publishVideo', 'publishAudio'
 */
function dlCheckExternalAuth($type){
	if(cfArrayItem(cfMGetVar('weezoRegInfo'),'externalAuth')){
		$title=array('icon'=>'v','title'=>' ');
		if(cfIsInApp() && $type!=='directLink') $title['title']=outBt(cfCaption('genBack'),outIcon('backRed'),"javascript:parent.slideFrame('publish','/local/uiResources/publish.php','up')",false,false,'style="position:absolute;right:0.5em"');
		outDisplayErrorPage('<div class="extendedWarning" style="width:90%">'.cfCaption('externalAuthIncompatible').'</div><br>'.outButton(cfCaption('configAccountEdit'),"javascript:wl.UICommand('openprofile')",outIcon('logoSmall')),$title);
	}
}

/**
 * @desc Return link URL
 *
 * @param string $id: item id
 * @param array $tokenItem: item data
 * @return string link
 */
function dlLinkURL($id,$tokenItem){
	static $regInfo=false;

	if($tokenItem['type']=='directLink') return dlDirectLinkURL($id,$tokenItem);

	$link='<iframe frameborder="0" scrolling="no" src="'.dlDirectLinkURL($id,$tokenItem).'"';

	switch ($tokenItem['type']){
		// Direct link / download
		case 'publishDownload':
			return $link.' style="width:220px;height:42px"></iframe>';
		// Video
		case 'publishVideo':
			return $link.' frameborder="0" scrolling="no" style="width:'.$tokenItem['w'].'px;height:'.$tokenItem['h'].'px;background:black"></iframe>';
		// Image
		case 'publishImage':
			return $link.' style="width:'.$tokenItem['w'].'px;height:'.$tokenItem['h'].'px;background:black"></iframe>';
		// Audio
		case 'publishAudio':
			return $link.' style="width:252px;height:23px;background:black"></iframe>';
	}
}

/**
 * @desc Return link URL
 *
 * @param string $id: item id
 * @param array $tokenItem: item data
 * @return string link
 */
function dlDirectLinkURL($id,$tokenItem){
	static $regInfo=false;

	if(!$regInfo) $regInfo=cfMGetVar('weezoRegInfo');
	if(!isset($regInfo['regName'])) return false;

	// Turn streaming direct link into published audio / video
	if($tokenItem['type']=='directLink' && $tokenItem['downloadFormat']=='streaming') {
		if(@$tokenItem['streaming']=='audio') $tokenItem['type']='publishAudio';
		if(@$tokenItem['streaming']=='video') $tokenItem['type']='publishVideo';
	}

	$link=DNS_SITE.'/embedded.php?userName='.$regInfo['regName'].'&type='.$tokenItem['type'].'&v='.cfGGetVar('appVersion');
	if(isset($tokenItem['w'])) $link.='&w='.$tokenItem['w'].'&h='.$tokenItem['h'];
	if(isset($tokenItem['autoplay'])) $link.='&autoplay='.(int)$tokenItem['autoplay'];
	if(isset($tokenItem['videoDuration'])) $link.='&videoDurationExt='.(int)$tokenItem['videoDuration'];
	$link.='&uri=';
	$encodedFilename=urlencode(str_replace(' ','_',basename($tokenItem['filename'])));

	switch ($tokenItem['type']){
		// Direct link / download
		case 'directLink':
		case 'publishDownload':

			// Short link format
			if(cfGGetVar('directLinkLength')>2 && $tokenItem['type']=='directLink') return cfSiteLink($regInfo['regName'],false).'/'.strtolower(substr($id,0,cfGGetVar('directLinkLength')));

			$dl=cfSiteLink($regInfo['regName']).'/downloads/dlToken'.$id.'/';
			// Torrent
			if(isset($tokenItem['torrent'])) $dl.=cfFileWithoutExtension($encodedFilename).'.torrent';
			// Zip or raw file
			else $dl.=$encodedFilename.((is_dir($encodedFilename))?'.zip':'');

			if($tokenItem['type']=='directLink') return $dl; else return $link.$dl;

		// Video
		case 'publishVideo':
			return $link.('/video/dlToken.'.$id.'/file.'.$encodedFilename);
		// Image
		case 'publishImage':
			return $link.('/img/dlToken.'.$id.'/file.'.$encodedFilename).'&fullsizeAllowed='.((@$tokenItem['fullsizeAllowed'])?1:0);
		// Audio
		case 'publishAudio':
			return $link.('/audio/dlToken.'.$id.'/file.playlist.xml');
	}
}

/**
 * @display direct link management form
 * @param $type: 'directLink' or 'publishDownload'
 */
function dlDirectLinkForm($type){
	global $tokenList, $synchronousTorrentGeneration, $createdLink;

	// Verify that external authentication is disabled into profile, else display error message
	if($type!=='directLink') dlCheckExternalAuth($type);

	require_once(INCLUDE_DIR.'explorerFunctions.php');
	$presetFile=false; // prefil file/folder input with GET-passed file/folder name (base 64 encoded)
	$synchronousTorrentGeneration=false; // Set to true to indicate a torrent is to be created, and that progres will be synchronously updated. Torrent is created just at the end of page display

	cfRCreateFakeResource();

	/**
	 * Read transfers tokens
	 */
	$tokenList=efTokensRead();

	// Read registration info
	$regInfo=cfMGetVar('weezoRegInfo');

	/**
	 * Thumbnail image
	 */
	if(isset($_GET['thumbnailSrc'])){
		header('Content-type: image/jpg');
		header('Expires: '.gmdate("D, d M Y H:i:s",time()+3600*24*100).' GMT');
		$file=base64_decode($_GET['thumbnailSrc']);
		if(efFileType($file)=='image') cfCreateResizedJPG($file,0,0,'',$_GET['w'],$_GET['h']);
		elseif(efFileType($file)=='video') efMakeVideoThumbnail($file,'',$_GET['w'],$_GET['h'],0,20);
		exit;
	}

	/**
	 * process POST
	 */
	if(cfIsAsync()) cfAsyncHeader();

	// Update global direct links download speed limit
	if(isset($_POST['directLinkDownloadSpeedLimitChange'])){
		$sl=$_POST['directLinkDownloadSpeedLimitChange'];
		if(!$sl) $sl=0;
		if(is_numeric($sl) && $sl>=0 && $sl!==cfGGetVar('directLinkDownloadSpeedLimit')) cfGUpdateVar('directLinkDownloadSpeedLimit',$sl,true);
	}

	/**
	 * Direct link creation-related actions:
	 * Values: checkFile, checkFileAndCreate, create, browseFolder, browseFile, mailLink, suppressLink, abortBittorrentCreation
	 */

	if(isset($_POST['act'])){
		$action=$_POST['act'];
		if(isset($_POST['path'])) $_POST['path']=str_replace('\\','/',$_POST['path']);

		// Check file or folder's existance, and that it matches with type's files
		if(($action=='checkFile'||$action=='checkFileAndCreate')){
			if(!isset($_POST['path'])) $_POST['path']='';
			$file=cfUTF8Decode($_POST['path'],true,false,false);

			// Check
			if(strlen($file)>3 && file_exists($file) && (
					($type=='directLink' || $type=='publishDownload') ||
					($type=='publishAudio' && is_dir($file)) ||
					($type=='publishImage' && is_file($file) && efFileType($file)=='image') ||
					($type=='publishVideo' && is_file($file) && efFileType($file)=='video'))){}
			// Check failed
			else{
				echo cfAsyncXMLJSaction('dgi("path").style.color="red"');
				echo cfAsyncXMLJSaction("if(dgi('thumbnailLoading')) dgi('thumbnailLoading').style.display='none';");
				echo cfAsyncXMLJSaction('setThumbnailSrc(false)');
				echo cfAsyncXMLJSaction('btDisable("createBt")');
				die(cfAsyncFooter());
			}
		}
		// Sync submit page for actual creation
		if($action=='checkFileAndCreate') {
			echo cfAsyncXMLJSaction('proceedCreate()');
			die(cfAsyncFooter());
		}

		// Only check file: enable ok button, transmit image/video dimensions, reset path color
		if($action=='checkFile'){
			echo cfAsyncXMLJSaction('dgi("path").style.color=""');
			switch ($type){
				case 'directLink':
					// Enable / disable streaming option depending on file type
					if(($mimeType=efFileType($file))=='audio' || $mimeType=='video')
						echo cfAsyncXMLJSaction('enableStreaming(1)');
					else
						echo cfAsyncXMLJSaction('enableStreaming(0)');

					break;
				case 'publishVideo':
					require_once(INCLUDE_DIR.'viewVideoFunctions.php');
					list($w,$h)=vvfVideoDimensions($file);
				case 'publishImage':
					if($type=='publishImage') list($w, $h, $foo, $foo2) = @getimagesize($file);
					if(!$w){$w=200;$h=150;}
					echo cfAsyncXMLJSaction('setSourceDimensions('.$w.','.$h.')');
					list($rw,$rh)=cfThumbnailDimensions($w,$h,200,150,true);
					echo cfAsyncXMLJSaction('setThumbnailSrc("'.$_SERVER['PHP_SELF'].'?thumbnailSrc='.base64_encode($file).'&w='.$rw.'&h='.$rh.'",'.$rw.','.$rh.')');
					break;
			}
			echo cfAsyncXMLJSaction('btEnable("createBt")');
			die(cfAsyncFooter());
		}

		// Browse files or folders (DEPRECATED?)
		if($action=='browseFolder' || $action=='browseFile'){
			// Browse files
			if($action=='browseFile') $result= win_browse_file(true, '', '', null, array(cfUTF8Decode(cfCaption('genFile')) => "*.*"));
			// Browse folders
			else $result= $result= win_browse_folder('c:\\', cfUTF8Decode(cfCaption('genDirectory')));
			cfAsyncHeader();
			if($result) echo cfAsyncXMLJSaction('dgi("path").value="'.cfUTF8Encode(str_replace('\\','/',$result)).'"');
			die(cfAsyncFooter());
		}
		// Send link by email
		if($action=='mailLink' && isset($_POST['tokenId']) && isset($tokenList[$_POST['tokenId']])){
			cfServerSendCommand('openMail body="'.dlDirectLinkURL($_POST['tokenId'],$tokenList[$_POST['tokenId']]).'" subject="'.basename(dlDirectLinkURL($_POST['tokenId'],$tokenList[$_POST['tokenId']])).'"');
			cfAsyncHeader();
			die(cfAsyncFooter());
		}
		// Remove existing link
		if($action=='suppressLink' && isset($_POST['tokenId']) && isset($tokenList[$_POST['tokenId']])){
			// If bittorrent transfer, remove from list
			if($transferID=@$tokenList[$_POST['tokenId']]['torrentId']) {
				require_once(INCLUDE_DIR.'transferFunctions.php');
				tRemoveTransfer($transferID);
			}
			cfAsyncHeader();
			unset($tokenList[$_POST['tokenId']]);
			efTokensWrite($tokenList);
			die(cfAsyncFooter());
		}
		// Abort bittorrent creation (done through this trick as bittorrent creation process doesn't detect connection abort)
		if($action=='abortBittorrentCreation') cfMSetVar('weezoAbortBittorrentCreation',1);



		/**************************************
		 * Create new direct link (sync)
		 **************************************/
		if($action=='create' && isset($_POST['type']) && cfRegistrationData()) {
			unset($_POST['filename']); // Security: unset filename as it can only be passed by remote sharing direct link creation
			$token=dlDirectLinkCreate($_POST['type'], $_POST);
		}



	}
	// End of $_POST processing

	if(cfIsAsync()) die(cfAsyncFooter());


	// Preset file (passed from drag & drop over UI ?)
	if (isset($_GET['presetFile']))	$presetFile=base64_decode($_GET['presetFile']);


	// File passed from Windows context menu
	if(($shellExtFiles=cfGGetVar('uiResourceShellAction')) && cfGGetVar('uiResourceAccordion')=='send' && count($shellExtFiles)==1){
		$presetFile=$shellExtFiles[0];
		cfGSetVar('uiResourceShellAction',false,true);
	}
	elseif(($shellExtFiles=cfGGetVar('uiResourceShellFiles')) && cfGGetVar('uiResourceAccordion')=='publish'){
		$presetFile=$shellExtFiles[0];
		cfGSetVar('uiResourceShellFiles',false,true);
	}


	/**
	 * Display page
	 */
	ob_start();
	cfInsertHEAD(false);
	echo cfScriptLink('winClient.js');
?>
</head>
<body title="Weezo" onload="init()">
<?php
echo cfScriptLink('wz_dragdrop.js');
echo cfScriptLink('rcScripts.js');

// Display a mask over page until it is loaded
echo outLoadMask(false); flush();
?>
<div id="mainDiv">
<script type="text/javascript">

function init(){
	if(dgi('createdLinkForm')) createdLinkFormShow();
}
function backKD(kc){
	if(kc!==27 || !dgi("backButton")) return;
	if(dgn('path') && dgn('path').value) return;
	wl.removeKeycodeListener(D)
	parent.slideFrame('publish','/local/uiResources/publish.php','up');
}
wl.setKeycodeListener(D,backKD)

// Change global direct link speed limit
function directLinkDownloadSpeedLimitChange(n){
	var v=n.value
	if(v && (isNaN(n.value) || n.value<0)){
		n.style.color='red';
		return false;
	}
	else{
		if(this.lastVal==undefined) this.lastVal=-1;
		if(this.lastVal==v) return;
		this.lastVal=n.value;
		n.style.color='';
		wl.asr.send({'directLinkDownloadSpeedLimitChange':v})
	}
}

var createdLinkFormShown=0;
function createdLinkFormShow(){
	wl.mask.show(0,1);
	var f=dgi('createdLinkForm');
	f.style.display='';
	$(f).e.center();
	wl.mask.moveAbove(f);
	dgi('linkTA').select();
	wl.setKeycodeListener(D,createdLinkFormShowKD)
	createdLinkFormShown=1;
	torrentProgressBarFade(false);
}
function createdLinkFormShowHide(){
	if(createdLinkFormShown==0) return;
<?php if($synchronousTorrentGeneration) echo 'wl.postData();';?>
	var f=dgi('createdLinkForm');
	fade(f,1,0);
	wl.mask.hide();
	createdLinkFormShown=0;
}
function createdLinkFormShowKD(kc){if(kc==13||kc==27) createdLinkFormShowHide()}
function resizeWindow(){
	return;
	var h=dgi("mainDiv").offsetHeight+1*getElementStyle(D.body,'margin-top')+1*getElementStyle(D.body,'margin-bottom')+1*getElementStyle(D.body,'padding-top')+1*getElementStyle(D.body,'padding-bottom');
	wl.UICommand('fadesize:0x'+Math.min(h+10,540))
}
var newLinkFormCollapseO;
function newLinkFormCollapse(id,s){
	winResized();
}
function tabSelected(isSelected, tabName){
	if(isSelected && tabName=='publish') D.location=wl.insertSessionIDInURL('/local/uiResources/publish.php');
}
function toggleUnlimitedLifetime(){
	if(dgi("unlimitedLifetime").checked) {dgi("lifetime").disabled=true;dgi("lifetimePeriod").disabled=true;}
	else{dgi("lifetime").disabled=false;dgi("lifetimePeriod").disabled=false;}
}
function toggleUnlimitedDownloads(){
	if(dgi("unlimitedDownloads").checked) dgi("maxDownloads").disabled=true;
	else dgi("maxDownloads").disabled=false;
}
function enableStreaming(enable){
	dgi('streamingRadio').disabled=!enable;
	dgi('streamingLabel').disabled=!enable;
	if(!enable && dgi('streamingRadio').checked) dgn('downloadFormat').checked=true;
}
function checkFileAndCreate(){
	dgi('act').value='checkFileAndCreate';
	btSetIcon('createBt','<?php echo outIcon('loading');?>');
	btDisable('createBt');
	asyncSubmitForm("createForm",false);
	if(dgi('cancelCreateBt')) {
		dgi('cancelCreateBt').style.display='';
		if(dgi('cancelBt')) dgi('cancelBt').style.display='none';
		wl.mask.moveAbove(dgi('cancelCreateBt'));
	}
}
function cancelCreate(){
	wl.asr.cancelAll();
	windowStop();
	btDisable('createBt');
	btSetIcon('createBt','<?php echo outIcon('ok16');?>');
	dgi('cancelCreateBt').style.display='none';
	if(dgi('cancelPresetBt')) dgi('cancelPresetBt').style.display='none';
	torrentProgressBarFade(false);

	if(dgi('cancelBt')) dgi('cancelBt').style.display='';
	sendData('act=abortBittorrentCreation');
	wl.mask.hide();
}
function torrentProgressBarFade(fadeIn){
	if(dgi('torrentProgressBar')) {
		if(fadeIn){
			dgi('torrentProgressBar').style.height='1px';
			dgi('torrentProgressBar').style.display='';
			fade('torrentProgressBar',0,1,30);
			wl.fadeStyle('torrentProgressBar','height',20,'exponential',10);
		}
		else{
			fade('torrentProgressBar',1,0,30);
			wl.fadeStyle('torrentProgressBar','height',0,'exponential',10);
		}
	}
}
function proceedCreate(){
	dgi('act').value='create';
	try{D.createForm.submit();} catch(e){
		try{D.location.href='http://google.fr';}
		catch(e){}

		if(0)try{wl.asr.send({'a':1},true)}
		catch(e){wl.dbgObj(e)}
		//asyncSubmitForm('createForm')
	}
}
function browseFile(){
	dgi('act').value='browseFile';
	asyncSubmitForm("createForm",false);
}
function browseFolder(){
	dgi('act').value='browseFolder';
	asyncSubmitForm("createForm",false);
}
// Link suppression animation
var suppressLinkId, suppressLinkHeight, suppressLinkI;
function suppressLink(tokenId){
	var nb=dgi('activeLinksNb').innerHTML-1;
	suppressLinkI=30;
	suppressLinkId=tokenId;
	dgi('linkFrame'+tokenId).style.overflow='hidden'
	suppressLinkHeight=dgi('linkFrame'+tokenId).offsetHeight;
	fade('linkFrame'+tokenId,1,0,suppressLinkI,'','suppressLinkShrink()');
	dgi('activeLinksNb').innerHTML=nb;
	$('activeLinksNbOuter').style.display=((nb)?'':'none')
	$('noLinkFrame').style.display=((nb)?'none':'');
	dgi('tokenId').value=tokenId;
	dgi('act').value='suppressLink';
	asyncSubmitForm("createForm",false);
	winResized();
}
function suppressLinkDone(){removeNode(dgi('linkFrame'+suppressLinkId))}
function suppressLinkShrink(){
	suppressLinkI--;
	if(suppressLinkI>5) return;
	dgi('linkFrame'+suppressLinkId).style.height=Math.max(1,Math.floor(suppressLinkHeight*suppressLinkI/15));
	if(suppressLinkI<-10) suppressLinkDone()
	else if(suppressLinkI<1) setTimeout('suppressLinkShrink()',10)
}
function mailLink(tokenId){sendData('tokenId='+tokenId+'&act=mailLink')}
function showLink(tokenId){
	if(dgi('ta'+tokenId).style.display=="inline"){
		dgi('ta'+tokenId).style.display="none";
	}
	else{
		dgi('ta'+tokenId).style.display="inline";
		dgi('ta'+tokenId).select();
	}
	resizeWindow();
}
function pathChanged(){
	if(!dgn("path").value) return;
	if(dgi('thumbnailLoading')) dgi('thumbnailLoading').style.display='';
	sendData('act=checkFile&path='+encodeURIComponent(dgn("path").value)+'&type=<?php echo $type;?>');
}
// required for rcSharedFilesBox
function eSaveBt(varName,value){
	//alert(varName+ ' '+value)
	if(varName=='sharedMode') sharedMode_changed(value)
	if(varName=='sharedMode' && value=='list') rcSharedFilesBoxUpdated(dgn('sharedItems').value)
	if(varName=='path' || value=='folder') pathChanged();
}
// Called on file list modification
function rcSharedFilesBoxUpdated(str){
	if(str) btEnable("createBt"); else btDisable("createBt")
}
// Set source image/video dimensions
var srcW,srcH;
function setSourceDimensions(w,h){
	srcW=w;srcH=h;
	if(dgi('sourceSize')) dgi('sourceSize').innerHTML=w+' x '+h+' px';
	if(dgi('sizeSel0')){
		var i=0,sw,max=-1,maxi=0;
		for(sw=160;sw<1600;sw*=2){
			if(!dgi('sizeSel'+i)) break;
			dgi('sizeVal'+i).innerHTML=sw+' x '+Math.floor(sw*h/w)+' px';
			wl.blockItem('sizeSel'+i).enable(sw<=w);
			if(sw<=w) {max=sw;maxi=i}
			i++;
		}

		if(max==-1) i=0;
		else if(w==max) i=maxi;
		else if(w>max) i=Math.min(maxi+1,i-1);
		else i--;

		dgi('sizeVal'+i).innerHTML=w+' x '+h+' px';
		wl.blockItem('sizeSel'+i).enable(1);

		for(i=0;i<10;i++){
			if(!dgi('sizeSel'+i)||!wl.blockItem('sizeSel'+i).enabled()) break;
			if(wl.blockItem('sizeSel'+i).selected()) return;
		}
		i--;
		wl.blockItem('sizeSel'+i).select(1)
		dgn('outputSize').value=dgi('sizeVal'+i).innerHTML;
	}
}
var thumbnailHeight;
// Set thumbail src
function setThumbnailSrc(s,w,h){
	if(!dgi("thumbnail")) return;
	if(!s) {
		s="<?php echo cfExtImage('foo',200,150);?>";
		srcW=w=200;
		srcH=h=150;
	}
	thumbnailHeight=h;

	with(dgi("thumbnail")){
		src=s
		style.display='';
		onload=thumbnailLoaded;
	}

	var i=0;
	while(dgi('sizeImg'+i)) {
		with(dgi('sizeImg'+i)){
			src=s;
			if(srcW>srcH) {
				style.width=i*14+20;
				style.height=Math.floor((i*14+20)*srcH/srcW);
				style.marginTop=30-Math.floor((i*14+20)*srcH/2/srcW);
			}
			else{
				style.height=i*14+20;
				style.width=Math.floor((i*14+20)*srcW/srcH);
				style.marginTop=30-Math.floor((i*14+20)/2);
			}
		}
		i++;
	}
}
function thumbnailLoaded(){
	dgi("thumbnail").style.width='';
	dgi("thumbnail").style.height=thumbnailHeight;
	if(dgi('thumbnailLoading')) dgi('thumbnailLoading').style.display='none';
}

// Set BitTorrent encoding progress
function setTorrentProgress(perc){
	progressSetPerc('syncTorrentProgress',perc)
	progressSetText('syncTorrentProgress',Math.floor(perc)+'%')
}
</script>
<?php

// Created link form
	if(isset($createdLink)){
?>
<div id="createdLinkForm" style="position:absolute;display:none">
<?php outShadowBefore('600');?>
<div class="popup" style="position:static">
<?php
	echo '<div class="popupHeader">';
	echo outImage(outIcon('ok16'),false,false,'margin-right:0.5em;');

	if($createdLink['token']['type']=='directLink' && $createdLink['link']) echo str_replace('<br>','</div><br>', cfCaption('directLinkCreated')).'<br>'.outSocialURLShare($createdLink['link'],false,cfFileWithoutExtension(basename($createdLink['token']['filename'])));
	else echo cfCaption('publishLinkCreated').'</div>'.'&nbsp;'.cfCaption('publishLinkDescription');
	echo "<br><br>\n";
	echo '<center>';
	echo '<textarea style="width:580px;height:4em" id="linkTA" onclick="this.select()" readonly="readonly">'.$createdLink['link'].'</textarea>';
	echo '<br><br>';
	if($createdLink['token']['type']!='directLink'){
		echo '</center>';
		echo '<div style="float:right">'.outSocialURLShare($createdLink['directLink'],array('clipboard'),cfFileWithoutExtension(basename($createdLink['token']['filename']))).'</div>';
		echo outButtonClipboardCopy('directLinkTA').'&nbsp;'.cfCaption('genLink').'<br>';
		echo '<textarea style="width:580px;height:4em" id="directLinkTA" onclick="this.select()" readonly="readonly">'.$createdLink['directLink'].'</textarea><center>';
		echo '<br><br>';
	}

	echo outBt(cfCaption('genOK'),outIcon('closeOK'),'javascript:createdLinkFormShowHide()');
	echo '</center>';

	echo "</div>\n";
	outShadowAfter();
	echo "</div>\n";
}
?>
<form name="createForm" id="createForm" action="<?php echo $_SERVER['PHP_SELF'];?>" method="POST">
<input type="hidden" name="type" value="<?php echo $type;?>">
<?php
	echo outDivFrame('frame1');

	/***********************************************************************************
	 * Title, description, back button
	 ***********************************************************************************/

	// Direct link / BitTorrent
	if($type=='directLink'){
		$title=cfCaption('genDirectLink');

		$description='<span style="font-size:120%">'.cfCaption('directLinkExpl').'<br>'.cfCaption('directLinkExpl2').'</span>';
		// Speed limit
		$sl=(int)cfGGetVar('directLinkDownloadSpeedLimit'); if(!$sl) $sl='';
		$rightItem='<b>'.cfCaption('directLinkDownloadSpeedLimit').'<input class="textInput" type="text" id="directLinkDownloadSpeedLimit" name="directLinkDownloadSpeedLimit" style="width:3em;margin-left:1em;margin-right:1em;text-align:right" value="'.($sl).'" onchange="directLinkDownloadSpeedLimitChange(this)" onkeyup="directLinkDownloadSpeedLimitChange(this)" >'.cfCaption('explorerKBPS').'</b>';
		$rightItem.=outButton('','javascript:var n=dgi(\'directLinkDownloadSpeedLimit\');n.value=\'\';directLinkDownloadSpeedLimitChange(n)',outIcon('infinite'),cfCaption('genUnlimited'),false,'style="margin-left:1em;margin-right:2em"');

		// Disable speed limit control for simple mode
		if(cfSimpleMode()) $rightItem='';
	}
	// Published download / image / video / playlist
	else{
		$rightItem=outBt(cfCaption('genBack'),outIcon('backRed'),"javascript:parent.slideFrame('publish','/local/uiResources/publish.php','up')",'backButton');
		$description=false;
		$title=cfCaption($type.'Label');
	}

	echo '<div style="float:right">'.$rightItem.'</div>';
	echo '<div class="uiTabTitle">'.$title.'</div>';

	echo outDivFrame('','id="scrollDiv"','height:1;overflow:auto');

	// Description
	if($description) echo '<div class="smallFont">'.$description.'</div><br>';

	// Error message
	if(isset($errorMessage)) echo '<div class="warning">'.$errorMessage.'</div>';


	/***********************************************************************************
	 * File(s) / folder selection
	 ***********************************************************************************/
	echo outDivFrame('frame2');
	echo '<div style="float:right">'.outButtonToggleCollapse2('newLink',false,true,"'newLinkFormCollapse'").'</div>';

	echo '<div class="frame2Header">'.outImageIcon('new').(($type=='directLink')?cfCaption('directLinkNew'):cfCaption('genNew'))."</div>\n";
	// Collapsable frame
	echo outCollapsableFrame('newLink',false,false,false);

	// UI command reception (from file drop or browse button)
	?>
<script type="text/javascript">
function UICommandReceive(dataName,dataValue){

	// Browse button
	if(dataName=='path') {
		dgi('path').value=dataValue.substr(2);
		pathChanged(dataValue);
	}

	// File drop
	if(dataName=='dropFileInput') {
		if(dgi('sharedModeList') && dgi('sharedModeList').style.display!="none"){
			filesAdd(dataValue);
		}
		else{
			if(dataValue.indexOf('|')!=-1) dataValue=dataValue.substr(0,dataValue.indexOf('|')); // If multiple files have been dropped, keep only the 1st one
			dgi('path').value=dataValue.substr(2);
			pathChanged(dataValue);
		}
	}
}
W.save=false; // Disable save function so filesAdd (rcScripts.js) function works properly
</script>
	<?php

	// File(s) / folder selection
	if($type=='publishAudio'){
		require_once(INCLUDE_DIR.'resourceConfigFunctions.php');
		rcSharedFilesBox('audio',false,array('noDiv'=>1,'noSubDir'=>1,'noComputerRoot'=>1,'filesOnly'=>1));
	}
	// Files selection list
	else {
		// Caption
		echo (($type=='directLink'||$type=='publishDownload')?cfCaption('genFileOrFolder'):cfCaption('genFile')).cfCaption('genSeparator');
		// <input>
		echo '<input class="textInput" type="text" style="width:30em;margin-right:1em;vertical-align" name="path" id="path" onfocus="this.style.color=\'black\'"'.(($presetFile)?' value="'.cfUTF8Encode($presetFile).'"':'').' onkeyup="pathChanged(this.value)" onblur="pathChanged(this.value)">';

		echo '<input name="dropFileInput" style="display:none">';

		// File/folder selection button
		if(!$presetFile){
			// Folder selection
			if($type=='directLink'||$type=='publishDownload') echo outBt(cfCaption('genDirectory'),outIcon('fi/folder'),"javascript:wl.UICommand('openFolder','path')");

			// Filter (audio/video/image)
			if($type=='publishImage') $fileTypeFilter='image';
			elseif($type=='publishVideo') $fileTypeFilter='video';
			else $fileTypeFilter=false;
			if($fileTypeFilter) {
				require_once(INCLUDE_DIR.'mime_type.php');
				$extFilter=array();
				foreach ($_ENV['weezoMime'] as $ext=>$mime) if(cfCmpLeft($mime,$fileTypeFilter)) $extFilterArray[$ext]=$ext;
				$extFilter=':*.'.implode(';*.',$extFilterArray);
			}
			else $extFilter='';

			echo outBt(cfCaption('genFile'),outIcon('blankFile'),"javascript:wl.UICommand('openFile".$extFilter."','path')",'fileBt');
		}
		echo '<br><br>';
	}



	/***********************************************************************************
	 * EXTRA PARAMETERS
	 ***********************************************************************************/

	// Download format (zip/bittorrent)
	if($type=='directLink' || $type=='publishDownload'){
		echo cfCaption('genFormat');
		// ZIP
		echo outImage(outIcon('fi/icoZip'),false,false,'vertical-align:middle;margin-right:0.5em;margin-left:3em');
		echo '<input type="radio" name="downloadFormat" value="zip" style="vertical-align:middle"'.((cfGGetVar('directLinkDefaultFormat')!='torrent')?' checked="checked"':'').'><span onclick="this.previousSibling.checked=true" style="cursor:pointer">ZIP</span>';

		// Bittorrent
		echo outImage(outIcon('bittorrent'),false,false,'vertical-align:middle;margin-right:0.5em;margin-left:3em');
		echo '<input type="radio" name="downloadFormat" value="torrent" style="vertical-align:middle"'.((cfGGetVar('directLinkDefaultFormat')=='torrent')?' checked="checked"':'').'><span onclick="this.previousSibling.checked=true" style="cursor:pointer">'.cfCaption('bittorrent').'</span>';

		// Streaming
		echo outImage(outIcon('play'),false,false,'vertical-align:middle;margin-right:0.5em;margin-left:3em');
		echo '<input type="radio" name="downloadFormat" value="streaming" style="vertical-align:middle" disabled="true" id="streamingRadio"><span onclick="if(!this.previousSibling.disabled) this.previousSibling.checked=true" style="cursor:pointer" disabled="true" id="streamingLabel">Streaming</span>';
		echo '<br>';
	}

	// Direct link
	if($type=='directLink'){
		// Max downloads
		echo cfCaption('directLinkLimitNb').'<input class="textInput" type="text" style="width:2em;margin-left:1em;text-align:right" value="1" name="maxDownloads" id="maxDownloads" disabled="disabled">';
		echo '<input type="checkbox" name="unlimitedDownloads" id="unlimitedDownloads" style="margin-left:2em;margin-right:1em" onChange="toggleUnlimitedLifetime()" onclick="toggleUnlimitedDownloads()" checked="checked">'.cfCaption('genUnlimited').'<br><br>';

		// Lifetime
		echo cfCaption('directLinkLifetime').'<input class="textInput" type="text" id="lifetime" name="lifetime" style="width:2em;margin-left:1em;margin-right:1em;text-align:right" value="1" disabled="disabled">';
		echo '<select class="textInput" size="1" id="lifetimePeriod" name="lifetimePeriod" disabled="disabled">';
		echo '<option selected="selected" value="days">'.cfCaption('genDays').'</option>';
		echo '<option value="hours">'.cfCaption('genHours').'</option>';
		echo '<option value="minutes">'.cfCaption('genMinutes').'</option>';
		echo '</select>';
		echo '<input type="checkbox" name="unlimitedLifetime" id="unlimitedLifetime" style="margin-left:2em;margin-right:1em" onChange="toggleUnlimitedLifetime()" onclick="toggleUnlimitedLifetime()" checked="checked">'.cfCaption('genUnlimited').'<br><br>';
	}

	// Download
	elseif ($type=='publishDownload'){
		echo '<br>';
	}

	// Video
	elseif ($type=='publishVideo'){
		// Thumbnail
		echo '<table onclick="dgi(\'fileBt\').click()" class="frame3" style="cursor	:pointer;width:210;height:160;float:left;margin-right:1em;position:relative"><tr style="vertical-align:middle;height:100%"><td style="vertical-align:middle;height:100%;text-align:center"><img id="thumbnail" style="width:200px;height:150px" src="'.cfExtImage('foo',200,250).'"><img id="thumbnailLoading" style="position:absolute;left:50%;top:50%;margin-left:-8px;margin-top:-8px;display:none" src="'.outIcon('loadingBlack').'" alt=""></td></tr></table>';

		// Size selection
		echo cfCaption('genSize').'<br>';
		for($i=0;$i<4;$i++){
			$c='<img id="sizeImg'.$i.'" style="position:relative;width:'.(($i)*12+20).';height:'.(($i)*9+15).';margin-top:'.floor(25-(($i)*9+15)/2).';background:black;border:1px solid #AAA" src="/ext/image.php">';
			$c.='<div style="position:absolute;bottom:5;width:100;text-align:center;left:0" id="sizeVal'.$i.'">---</div>';
			echo outBlockItem('sizeSel',$c,'id="sizeSel'.$i.'"','width:100;height:90;text-align:center;padding:5',array('onclick'=>"dgn('outputSize').value=dgi('sizeVal".$i."').innerHTML"),'disabled');
		}
		echo '<input name="outputSize" style="display:none">';

		// Flash output video quality
		echo '<br><br>'.cfCaption('genQuality')."<br>\n";
		?>
<select name="flashOutputQuality" class="textInput" size="1">
<option selected="selected" value="original"><?php echo cfCaption('videoCfgSource');?></option>
<option value="high"><?php echo cfCaption('genHigh');?></option>
<option value="medium"><?php echo cfCaption('genMedium');?></option>
<option value="low"><?php echo cfCaption('genLow');?></option>
</select>
&nbsp;&nbsp;&nbsp;
<input type="checkbox" name="autoplay">&nbsp;
<span onclick="dgn('autoplay').checked=!dgn('autoplay').checked" style="cursor:pointer"><?php echo cfCaption('autoplay');?></span>
<br clear="both">
<?php
	}

	// Image
	elseif ($type=='publishImage'){
		// Thumbnail
		echo '<table onclick="dgi(\'fileBt\').click()" class="frame3" style="cursor:pointer;width:210;height:180;float:left;margin-right:1em;text-align:center"><tr style="vertical-align:middle;height:100%"><td style="vertical-align:middle;height:100%;text-align:center"><img id="thumbnail" style="width:200;height:150" src="'.cfExtImage('foo',200,250).'"></td></tr><tr><td id="sourceSize">&nbsp;</td></tr></table>';

		// Size selection
		echo cfCaption('genSize').'<br>';
		for($i=0;$i<4;$i++){
			$c='<img id="sizeImg'.$i.'" style="position:relative;width:'.(($i)*12+20).';height:'.(($i)*9+15).';margin-top:'.floor(25-(($i)*9+15)/2).';background:black;border:1px solid #AAA" src="/ext/image.php">';
			$c.='<div style="position:absolute;bottom:5;width:100;text-align:center;left:0" id="sizeVal'.$i.'">---</div>';
			echo outBlockItem('sizeSel',$c,'id="sizeSel'.$i.'"','width:100;height:90;text-align:center;padding:5',array('onclick'=>"dgn('outputSize').value=dgi('sizeVal".$i."').innerHTML"),'disabled');
		}
		echo '<input name="outputSize" style="display:none">';

		// Allow full-size image selection
		echo '<br><br><br><input type="checkbox" checked="checked" name="fullsizeAllowed">&nbsp;<span onclick="dgn(\'fullsizeAllowed\').checked=!dgn(\'fullsizeAllowed\').checked" style="cursor:pointer">'.cfCaption('explorerAllowFullSizeImage').'</span><br>';

		echo '<br clear="both">';
	}

	// Audio
	elseif ($type=='publishAudio'){
		// Flash output video quality
		echo cfCaption('genQuality').cfCaption('genSeparator');
		?>
<select name="maxOutputBitRate" class="textInput" size="1">
<option selected="selected" value="original"><?php echo cfCaption('musicCfgNoLimit');?></option>
<option value="high"><?php echo cfCaption('explorerAudio128');?></option>
<option value="medium"><?php echo cfCaption('explorerAudio64');?></option>
<option value="low"><?php echo cfCaption('explorerAudio32');?></option>
</select>
&nbsp;&nbsp;&nbsp;<input type="checkbox" name="autoplay">&nbsp;<span onclick="dgn(\'autoplay\').checked=!dgn(\'autoplay\').checked" style="cursor:pointer"><?php echo cfCaption('autoplay');?></span>
<?php
	}


	// Create / cancel buttons
	echo '<br><center>';
	echo outBt(cfCaption('adminResourceCreate'),outIcon('ok16'),'javascript:checkFileAndCreate()','createBt',false,'style="margin-right:2em"',($presetFile)?'up':'off');
	if($type!='directLink') echo outBt(cfCaption('genCancel'),outIcon('cancel'),"javascript:parent.slideFrame('publish','/local/uiResources/publish.php','up')",'cancelBt');
	if($type=='directLink' || $type=='publishDownload') echo outBt(cfCaption('genCancel'),outIcon('cancel'),"javascript:cancelCreate()",'cancelCreateBt',false,'style="display:none"');
	if($type=='directLink' && $presetFile) echo outBt(cfCaption('genCancel'),outIcon('cancel'),"javascript:wl.postData()",'cancelPresetBt');

	if($synchronousTorrentGeneration) echo '<br><div id="torrentProgressBar" style="width:80%;padding:7px;overflow:hidden;display:none">'.outProgressBar(0,'100%','0%','syncTorrentProgress').'</div>';

	echo '</center><input type="hidden" value="" name="act" id="act"/>';
	echo '<input type="hidden" value="" name="tokenId" id="tokenId"/>';
	echo '</div>'; // Collapsable frame
	echo "</div>\n"; // New link frame

	echo '<br>';


	/*
	 * Display existing links (if a file/folder has been selected by context menu, hide existing links to focus on creation form)
	 */
	if(!$presetFile){
		dlDisplayTokensLinks($type, $tokenList);
	}

	echo "</div>\n"; // scrollDiv
	echo '</div></form>';
?>
<script type="text/javascript">
function winResized(){
	alignBottom(dgi('scrollDiv'),winMe,5)
	alignBottom(dgi('noLinkFrame'),winMe,5)
}
winResized();
pathChanged();
</script>
</div>
<?php

	/***********************************************************************************
	 * Unregistered user: display warning message
	 ***********************************************************************************/
	if(!cfRegistrationData()){
?>
<div id="regsitrationWarning" style="position:absolute;width:80%;left:10%;top:45%;padding-left:10%;padding-right:10%" class="extendedWarning"><br><?php echo outImage(outIcon('alert32'),false,false,'float:left;margin-right:1em;margin-bottom:1em').cfCaption('registrationNeeded');?><br>&nbsp;</div>
<script type="text/javascript">
wl.mask.show(0.8,0,0,0,0,{'background':'#FFF'});
wl.mask.moveAbove(dgi('regsitrationWarning'));
</script>
<?php
	}


	ob_end_flush();

	/**
	 * Torrent files: genrate torrent and add it to transfers
	 */
	if($synchronousTorrentGeneration){
		wSession_write_close();
		// Insert script tag so torrent synchronous javascript state updates are executed

		// Enable cancel button
		echo "<script type=\"text/javascript\">dgi('cancelCreateBt').style.display='';btSetIcon('createBt','".outIcon('loading')."');torrentProgressBarFade(true)</script>";
		flush();
		echo '<div style="position:absolute;top:0;left:0;background:red">';

		// Create torrent
		list($token['torrentId'],$token['torrent'])=efTorrentCreate(array($token['filename']),$token['id'],array('jsSyncMonitorFunction'=>'setTorrentProgress'));

		// b64 encode torrent string to safely store it
		$token['torrent']=base64_encode($token['torrent']);

		// Save tokens
		$tokenList+=array($token['id']=>$token);
		efTokensWrite($tokenList);

		// Set link into user popup (which will be displayed onload)
		echo '<script type="text/javascript">';
		echo 'setTorrentProgress(100);dgi("linkTA").innerHTML="'.dlDirectLinkURL($token['id'],$token).'"';
		echo '</script>';
	}
	echo '</body></html>';
}

/**
 * @desc Return code for async direct link form insertion (mobile version)
 *
 * @param string $protectedFilename: protected file name
 * @return string
 */
function dlDirectLinkInsertMobileRemoteForm($protectedFilename){
	$html ='<div class="frame1Header">'.outImageIcon('share32').str_replace('...','',cfCaption('doShare')).'&nbsp;'.cfUTF8Encode(basename($protectedFilename),false,false).'</div><br>';
	$html.='<div id="directLinkInnerDiv">';
	$html.='<form id="directLinkCreateForm">';
	$html.='<input name="directLinkCreateForm" value="1" style="display:none">';
	$html.='<input name="filename" value="'.cfUTF8Encode($protectedFilename).'" style="display:none">';

	/**
	 * Link format
	 */
	$html.='<div class="frame1Header">'.cfCaption('genFormat').'</div>';
	$html.=mfListTable();

	// ZIP
	$html.=mfListItem('onclick="wl.liBlink(this);dgi(\'downloadFormatZip\').checked=1"',outImage(outIcon('fi/med/icoZip'),false,'class="nfrLi32"'),false,'ZIP',false,'<input type="radio" name="downloadFormat" id="downloadFormatZip" value="zip" style="vertical-align:middle"'.((cfGGetVar('directLinkDefaultFormat')!='torrent')?' checked="checked"':'').'>');

	// Bittorrent
	$html.=mfListItem('onclick="wl.liBlink(this);dgi(\'downloadFormatTorrent\').checked=1"',outImage(outIcon('fi/med/icoTorrent'),false,'class="nfrLi32"'),false,cfCaption('bittorrent'),false,'<input type="radio" name="downloadFormat" id="downloadFormatTorrent" value="torrent" style="vertical-align:middle"'.((cfGGetVar('directLinkDefaultFormat')=='torrent')?' checked="checked"':'').'>');

	// Streaming
	require_once(INCLUDE_DIR.'explorerFunctions.php');
	$type=efFileType($protectedFilename);
	if($type=='audio' || $type=='video'){
		$html.=mfListItem('onclick="wl.liBlink(this);dgi(\'downloadFormatStreaming\').checked=1"',outImage(outIcon('play32'),false,'class="nfrLi32"'),false,'Streaming',false,'<input type="radio" name="downloadFormat" id="downloadFormatStreaming" value="streaming" style="vertical-align:middle"'.((cfGGetVar('directLinkDefaultFormat')=='torrent')?' checked="checked"':'').'>');

		$html.=outImage(outIcon('play'),false,false,'vertical-align:middle;margin-right:0.5em;margin-left:3em');
		$html.='<input type="radio" name="downloadFormat" value="streaming" style="vertical-align:middle" id="streamingRadio"><span onclick="if(!this.previousSibling.disabled) this.previousSibling.checked=true" style="cursor:pointer" disabled="true" id="streamingLabel">Streaming</span>';
	}
	$html.='</table>';
	$html.='<br>';


	/**
	 * Link lifetime
	 */

	// Max downloads
	$html.='<div class="frame1Header">'.cfCaption('directLinkLimitNb').'</div>';
	$html.=mfListTable();
	$html.=mfListItem(false,false,false,'<input class="textInput" type="text" style="width:2em;text-align:right" value="1" name="maxDownloads" id="maxDownloads"><input class="textInput" type="checkbox" name="unlimitedDownloads" id="unlimitedDownloads" style="margin-left:2em;margin-right:1em" onChange="toggleUnlimitedLifetime()" onclick="dgi(\'maxDownloads\').disabled=this.checked">'.
	'<span class="lit">'.cfCaption('genUnlimited').'</span>');
	$html.='</table>';

	$html.='<br><div class="frame1Header">'.cfCaption('directLinkLifetime').'</div>';
	$html.=mfListTable();

	// Lifetime
	$html.=mfListItem(false,false,false,'<input type="text" id="lifetime" name="lifetime" class="textInput" style="width:2em;margin-right:1em;text-align:right" value="7">'.
			'<select class="textInput" size="1" id="lifetimePeriod" name="lifetimePeriod">'.
			'<option selected="selected" value="days">'.cfCaption('genDays').'</option>'.
			'<option value="hours">'.cfCaption('genHours').'</option>'.
			'<option value="minutes">'.cfCaption('genMinutes').'</option>'.
			'</select>'.
			'<input type="checkbox" name="unlimitedLifetime" id="unlimitedLifetime" style="margin-left:2em;margin-right:1em" onChange="dgi(\'lifetime\').disabled=dgi(\'lifetimePeriod\').disabled=this.checked" onclick="toggleUnlimitedLifetime()">'.
			'<span class="lit">'.cfCaption('genUnlimited').'</span>'
	);
	$html.='</table>';

	/**
	 * Create / Cancel controls
	 */
	$html.='<br><br><center>';
	// Validate password input button
	$html.=mfButton(str_replace('...','',cfCaption('doShare')),"javascript:void(asyncSubmitForm('directLinkCreateForm'))",false,false,false,array('extraClass'=>'wide green'));
	$html.='<br>';
	// Cancel password input button
	$html.=mfButton(str_replace('...','',cfCaption('genCancel')),"javascript:selBack()",false,false,false,array('extraClass'=>'wide red'));
	$html.='</center>';
	$html.='</form>';


	//$script1="if(!dgi('directLinkForm')) $('directLinkForm').e.create('DIV',{'class':'frame2 popup', 'style':'width:400px;position:absolute'});";
	//$script2="wl.mask.show(0,1);wl.mask.moveAbove($('directLinkForm'));$('directLinkForm').e.center();";

	//$out.=cfAsyncJSaction($script2);
	return cfAsyncXMLInnerHTMLbyId($html,'mainDiv');
}

/**
 * @desc Return code for async display of generated link into link creation dialog box (mobile version)
 *
 * @param array $token: direct link token
 * @return string
 */
function dlDirectLinkMobileShowLink($token){
	require_once(INCLUDE_DIR.'outputFunctions.php');
	$url=dlLinkURL($token['id'],$token);


	$html ='<div class="frame1Header">'.cfCaption('genLink').'</div>';
	$html.='<center><input type="text" class="textInput" readonly="readonly" value="'.$url.'" style="width:80%">';
	$html.='<br><br>';
	$html.=outSocialURLShare($url,array('mobile'=>1,'displayQRCode'=>1));
	$html.='<br><br><br><br>';
	$html.=mfButton(cfCaption('genOK'),"javascript:selBack()",outIcon('ok32')).'</center>';
	return cfAsyncXMLInnerHTMLbyId($html,'mainDiv');
}

/**
 * @desc Return code for async direct link form insertion (desktop version)
 *
 * @param string $protectedFilename: protected file name
 * @return string
 */
function dlDirectLinkInsertDesktopRemoteForm($protectedFilename){

	$html ='<div class="frame2Header">'.outImageIcon('share').str_replace('...','',cfCaption('doShare')).'&nbsp;'.cfUTF8Encode(basename($protectedFilename),false,false).'</div>';
	$html.='<div id="directLinkInnerDiv">';
	$html.='<form id="directLinkCreateForm">';
	$html.='<input name="directLinkCreateForm" value="1" style="display:none">';
	$html.='<input name="filename" value="'.cfUTF8Encode($protectedFilename).'" style="display:none">';

	/**
	 * Link format
	 */
	$html.='<br><b>'.cfCaption('genFormat').'</b>&nbsp;';
	// ZIP
	$html.=outImage(outIcon('fi/icoZip'),false,false,'vertical-align:middle;margin-right:0.5em;margin-left:3em');
	$html.='<input type="radio" name="downloadFormat" value="zip" style="vertical-align:middle"'.((cfGGetVar('directLinkDefaultFormat')!='torrent')?' checked="checked"':'').'><span onclick="this.previousSibling.checked=true" style="cursor:pointer">ZIP</span>';

	// Bittorrent
	$html.=outImage(outIcon('bittorrent'),false,false,'vertical-align:middle;margin-right:0.5em;margin-left:3em');
	$html.='<input type="radio" name="downloadFormat" value="torrent" style="vertical-align:middle"'.((cfGGetVar('directLinkDefaultFormat')=='torrent')?' checked="checked"':'').'><span onclick="this.previousSibling.checked=true" style="cursor:pointer">'.cfCaption('bittorrent').'</span>';

	// Streaming
	require_once(INCLUDE_DIR.'explorerFunctions.php');
	$type=efFileType($protectedFilename);
	if($type=='audio' || $type=='video'){
		$html.=outImage(outIcon('play'),false,false,'vertical-align:middle;margin-right:0.5em;margin-left:3em');
		$html.='<input type="radio" name="downloadFormat" value="streaming" style="vertical-align:middle" id="streamingRadio"><span onclick="if(!this.previousSibling.disabled) this.previousSibling.checked=true" style="cursor:pointer" disabled="true" id="streamingLabel">Streaming</span>';
	}
	$html.='<br><br><br>';


	/**
	 * Link lifetime
	 */

	// Max downloads
	$html.=cfCaption('directLinkLimitNb').'<input class="textInput" type="text" style="width:2em;margin-left:1em;text-align:right" value="1" name="maxDownloads" id="maxDownloads">';
	$html.='<input class="textInput" type="checkbox" name="unlimitedDownloads" id="unlimitedDownloads" style="margin-left:2em;margin-right:1em" onChange="toggleUnlimitedLifetime()" onclick="dgi(\'maxDownloads\').disabled=this.checked">'.cfCaption('genUnlimited').'<br><br>';

	// Lifetime
	$html.=cfCaption('directLinkLifetime').'<input class="textInput" type="text" id="lifetime" name="lifetime" style="width:2em;margin-left:1em;margin-right:1em;text-align:right" value="7">';
	$html.='<select class="textInput" size="1" id="lifetimePeriod" name="lifetimePeriod">';
	$html.='<option selected="selected" value="days">'.cfCaption('genDays').'</option>';
	$html.='<option value="hours">'.cfCaption('genHours').'</option>';
	$html.='<option value="minutes">'.cfCaption('genMinutes').'</option>';
	$html.='</select>';
	$html.='<input type="checkbox" name="unlimitedLifetime" id="unlimitedLifetime" style="margin-left:2em;margin-right:1em" onChange="dgi(\'lifetime\').disabled=dgi(\'lifetimePeriod\').disabled=this.checked" onclick="toggleUnlimitedLifetime()">'.cfCaption('genUnlimited').'<br>';

	/**
	 * Create / Cancel controls
	 */
	$html.='<br><br><center>';
	$html.=outBt(str_replace('...','',cfCaption('doShare')),outIcon('ok16'),"javascript:void(asyncSubmitForm('directLinkCreateForm'))",false,false,false,'up',120);
	$html.='&nbsp;&nbsp;&nbsp;';
	$html.=outBt(cfCaption('genCancel'),outIcon('cancel'),"javascript:$('directLinkForm').e.remove();wl.mask.hide()",false,false,false,'up',120);
	$html.='</center>';
	$html.='</form></div>';



	$script1="if(!dgi('directLinkForm')) $('directLinkForm').e.create('DIV',{'class':'frame2 popup', 'style':'width:400px;position:absolute'});";
	$script2="wl.mask.show(0,1);wl.mask.moveAbove($('directLinkForm'));$('directLinkForm').e.center();";

	$out=cfAsyncJSaction($script1);
	$out.=cfAsyncXMLInnerHTMLbyId($html,'directLinkForm');
	$out.=cfAsyncJSaction($script2);
	return $out;
}

/**
 * @desc Return code for async display of generated link into link creation dialog box (desktop version)
 *
 * @param array $token: direct link token
 * @return unknown
 */
function dlDirectLinkDesktopShowLink($token){
	require_once(INCLUDE_DIR.'outputFunctions.php');
	$url=dlLinkURL($token['id'],$token);

	$html='<br><br><b>'.cfCaption('genLink').'</b><br>';
	$html.='<center><input type="text" class="textInput" readonly="readonly" value="'.$url.'" style="width:80%">';
	$html.='<br><br>'.outSocialURLShare($url);
	$html.='<br><br><br><br>';
	$html.='<center>'.outBt(cfCaption('genOK'),outIcon('closeOK'),"javascript:$('directLinkForm').e.remove();wl.mask.hide()",false,false,false,'up',150).'</center>';
	return cfAsyncXMLInnerHTMLbyId($html,'directLinkInnerDiv');
}

/**
 * @desc Remote file sharing form insertion and processing (all is async)
 *
 */
function dlDirectLinkRemoteForm(){
	// Only async actions
	if(!cfIsAsync()) return;
	// Only if user may share files
	if(!cfRGetVar('shareAllowed') || !cfRGetVar('downloadAllowed')) return;
	// Only process commands related to direct link remote form
	if(!isset($_POST['directLinkCreateForm']) && !isset($_POST['directLinkShowRemoteForm'])) return;

	require_once(INCLUDE_DIR.'explorerFunctions.php');

	// Unprotect and utf8 decode filename
	$protectedFilename=cfUnprotectFilename($_POST['filename'],true);

	$_POST['filename']=cfFilenameResolve($protectedFilename,true);

	// Check access rights
	if(!$_POST['filename'] || !cfFileRights($_POST['filename']))
		$out=cfAsyncJSaction("javascript:alert('".str_replace("'",'',cfCaption('explorerErrorFileSend'))."');$('directLinkForm').e.remove();wl.mask.hide()");

	// Link creation request
	else if(isset($_POST['directLinkCreateForm'])){
		$token=dlDirectLinkCreate('directLink',$_POST);
		if(cfIsMobile())
			$out=dlDirectLinkMobileShowLink($token);
		else
			$out=dlDirectLinkDesktopShowLink($token);
	}

	// Async insert form
	else if(isset($_POST['directLinkShowRemoteForm'])){
		if(cfIsMobile())
			$out=dlDirectLinkInsertMobileRemoteForm(($protectedFilename));
		else
			$out=dlDirectLinkInsertDesktopRemoteForm(($protectedFilename));
	}

	cfAsyncHeader();
	echo $out;
	die(cfAsyncFooter());
}

/**
 * @desc Generate a link id or direct resource access token
 *
 * @param array $tokenList: list of existing direct links, used to prevent collisions
 * @param int $fullLength: length of token to be generated
 * @return string id
 */
function dlGenerateId(&$tokenList=false, $fullLength=20){
	static $idBases;

	// Build short URIs list
	if(!isset($idBases)){
		$idBases=array();

		// Get 3 1st chars of direct link tokens
		if(!$tokenList){
			require_once(INCLUDE_DIR.'explorerFunctions.php');
			if(!($tokenList=efTokensRead())) $tokenList=array();
		}
		foreach ($tokenList as $id=>$token) $idBases[]=strtolower(substr($id,0,3));

		// Get 3 1st chars of resources direct links
		foreach (WEnv::resConfigs() as $rid=>$resConfig){
			if(0) $resConfig=new WResConfig();
			$idBases[]=strtolower(substr($resConfig->getVar('directResourceAccessToken'),0,3));
		}
	}

	// Ensure no other id begins with the 3 same characters so there is no intersection between 2 short URIs
	while (1) {
		$id=strtolower(cfGenerateID(3));
		if(!in_array($id,$idBases)) break;
	}
	$idBases[]=$id;
	$id.=strtolower(cfGenerateID($fullLength-3));
	return $id;
}

/**
 * @desc Create a new direct link
 *
 * @param string $type: "directLink", "download", "publishDownload", "publishVideo", "publishImage", "publishAudio"
 * @param array $params: 'path' | ('sharedMode' & 'sharedItems'), 'lifetime', 'maxDownloads' ,'unlimitedDownloads', 'unlimitedLifetime', 'lifetimePeriod'
 */
function dlDirectLinkCreate($type,$params){
	global $tokenList, $synchronousTorrentGeneration, $createdLink;

	$token=array('type'=>$type);

	// Generate token ID
	$tokenId=dlGenerateId($tokenList);

	// Check source
	if($type=='publishAudio' && isset($params['sharedMode']) && $params['sharedMode']=='list' && strlen($params['sharedItems'])){
		$token['filename']=cfUTF8Decode($params['sharedItems'],true,false,false,16000);
	}
	elseif (isset($params['path'])){
		if(!file_exists(cfUTF8Decode($params['path'],true,false,false))) die(cfAsyncFooter());
		$token['filename']=cfUTF8Decode($params['path'],true,false,false);
	}
	// Remote sharing: file name has already been decoded and checked
	elseif (isset($params['filename'])){
		$token['filename']=$params['filename'];
	}
	else die(cfAsyncFooter());

	// Direct link
	if($type=='directLink'){
		if(!isset($params['lifetime']) || !is_numeric($params['lifetime'])) $params['lifetime']=1;
		if(!isset($params['maxDownloads']) || !is_numeric($params['maxDownloads'])) $params['maxDownloads']=1;
		//if(!isset($params['directLinkDownloadSpeedLimit']) || !is_numeric($params['directLinkDownloadSpeedLimit'])) $params['directLinkDownloadSpeedLimit']=cfGGetVar('directLinkDownloadSpeedLimit');
		if(isset($params['unlimitedDownloads'])) $token['limitNb']=0;
		else {
			$token['limitNb']=$params['maxDownloads'];
			$token['limitNbOriginal']=$params['maxDownloads'];
		}
		// lifetime
		if(isset($params['unlimitedLifetime']))$token['expiry']=0;
		else{
			if(!isset($params['lifetimePeriod'])) $params['lifetimePeriod']='days';
			if($params['lifetimePeriod']=='days') $token['expiry']=time() + $params['lifetime']*3600*24;
			elseif($params['lifetimePeriod']=='hours') $token['expiry']=time() + $params['lifetime']*3600;
			else $token['expiry']=time() + $params['lifetime']*60;
		}
		// Change max download speed if needed
		/*
		if($params['directLinkDownloadSpeedLimit']!=cfGGetVar('directLinkDownloadSpeedLimit'))
			cfGUpdateVar('directLinkDownloadSpeedLimit',$params['directLinkDownloadSpeedLimit']);
		*/
		// Download format (zip/torrent/streaming)
		$token['downloadFormat']=((isset($params['downloadFormat']) && (
			$params['downloadFormat']=='zip' ||
			$params['downloadFormat']=='torrent' ||
			$params['downloadFormat']=='streaming'))?$params['downloadFormat']:'zip');

		cfGUpdateVar('directLinkDefaultFormat',$token['downloadFormat'],false);

		// Streaming: set default size / reencoding values
		if(@$params['downloadFormat']=='streaming'){
			// Audio
			if(efFileType($token['filename'])=='audio') {
				$token['streaming']='audio';
				$token['maxOutputBitRate']='original';
			}
			// Video
			elseif(efFileType($token['filename'])=='video') {
				$token['streaming']='video';
				$token['flashOutputQuality']='original';
				// Width & height
				require_once(INCLUDE_DIR.'viewVideoFunctions.php');
				list($token['w'],$token['h'])=vvfVideoDimensions($token['filename']);
				if($token['w']==0){$token['w']=320;$token['h']=200;}
			}
			// Audio/video common
			$token['autoplay']=true;
		}
	}

	// Downloads
	elseif ($type=='publishDownload'){
		// Download format (zip/torrent)
		$token['downloadFormat']=((isset($params['downloadFormat']) && ($params['downloadFormat']=='zip'||$params['downloadFormat']=='torrent'))?$params['downloadFormat']:'zip');
	}
	// Audio files
	elseif ($type=='publishAudio'){
		if(!isset($params['maxOutputBitRate']) || array_search($params['maxOutputBitRate'],array(32,64,128))===false) $token['maxOutputBitRate']='original';
		else $token['maxOutputBitRate']=$params['maxOutputBitRate'];
		$token['autoplay']=isset($params['autoplay']) && $params['autoplay']==='on';
	}
	// Images
	elseif ($type=='publishImage'){
		// Width & height
		list($w,$h,$foo,$foo2) = @getimagesize($token['filename']);
		if($w==0){$w=320;$h=200;}
		if(isset($params['outputSize'])) $s=substr($params['outputSize'],0,strpos($params['outputSize'],' ')); else $s=$w;
		if(!is_numeric($s) || $s<=0 || $s>$w) $s=$w;
		$token['w']=$s; $token['h']=floor($s*$h/$w);
		// Full-size view allowed
		$token['fullsizeAllowed']=isset($params['fullsizeAllowed']) && $params['fullsizeAllowed']==='on';
	}
	// Videos
	elseif ($type=='publishVideo'){
		// Width & height
		require_once(INCLUDE_DIR.'viewVideoFunctions.php');
		list($w,$h)=vvfVideoDimensions($token['filename']);
		$token['videoDuration']=vvfVideoDuration($token['filename']);
		if($w==0){$w=320;$h=200;}
		if(isset($params['outputSize'])) $s=substr($params['outputSize'],0,strpos($params['outputSize'],' ')); else $s=$w;
		if(!is_numeric($s) || $s<=0 || $s>$w) $s=$w;
		$token['w']=$s; $token['h']=floor($s*$h/$w);
		// Flash reencoding quality
		$token['flashOutputQuality']=$params['flashOutputQuality'];
		$token['autoplay']=isset($params['autoplay']) && $params['autoplay']==='on';
	}

	if(!isset($token['limitNb'])) $token['limitNb']=0;

	// Torrent files: genrate torrent and add it to transfers
	if(@$token['downloadFormat']=='torrent'){
		$synchronousTorrentGeneration=true;
		// Set created link so page display info popup on load, with empty link
		// (so form won't be displayed immediately, but only once bittorrent is created)
		$createdLink=array('id'=>$tokenId,'link'=>'','directLink'=>'','token'=>$token);
	}
	// Set created link so page display info popup on load
	else {
		$createdLink=array('id'=>$tokenId,'link'=>dlLinkURL($tokenId,$token),'directLink'=>dlDirectLinkURL($tokenId,$token),'token'=>$token);
		// Save tokens
		$tokenList+=array($tokenId=>$token);
		efTokensWrite($tokenList);
	}

	return $token+array('id'=>$tokenId);
}

/**
 * @desc Display list of links from download/directLink/publishXXX tokens
 *
 * @param string $type: type of links to display:
 * 			false:all, "directLink", "download", "publishDownload", "publishVideo", "publishImage", "publishAudio"
 */
function dlDisplayTokensLinks($type='',&$tokenList=null){
	if(!isset($tokenList)) $tokenList=efTokensRead();

	// Count active links
	$nb=0; foreach ($tokenList as $value) if(!$type || $value['type']==$type) $nb++;

	// Title
	switch ($type){
		case 'directLink':
			$title=outImageIcon('directLink').cfCaption('directLinkList');
			break;
		case 'publishDownload':
			$title=outImageIcon('dl2').cfCaption('publishDownloadLabel');
			break;
		default:
			$title=cfCaption('publishExistingLinks');
	}

	//echo outDivFrame('frame2');
	echo outFrameHeaderTable('frame2Header',$title,'<span id="activeLinksNbOuter" style="display:'.(($nb)?'':'none').'">'.str_replace('%1','<span id="activeLinksNb">'.$nb.'</span>',cfCaption('publishActiveLinks')).'</span>');
?>
<div id="noLinkFrame" style="position:relative;display:<?php echo (($nb)?'none':'block'); ?>;height:50px">
	<div style="position:absolute;top:45%;width:100%;margin-top:-0.5em;text-align:center" class="watermark"><?php echo cfCaption('publishActiveLinks','0'); ?></div>
</div>
<?php
	// Display existing direct links
	if($nb>0){
		foreach ($tokenList as $key=>$value) if(!$type || $value['type']==$type){

			echo outDivFrame('frame3','id="linkFrame'.$key.'" onmouseover="$(this).e.className().add(\'over\')" onmouseout="$(this).e.className().remove(\'over\')"');
			// Link filename and suppress button
			$r='';
			switch ($value['type']){
				case 'directLink':
					$ico='directLink';
					break;
				case 'download':
					$ico='dl';
					break;
			}
			// Suppress button
			$r.='&nbsp;&nbsp;&nbsp;<span class="shownOnOver">'.outButtonSmall('','javascript:suppressLink(\''.$key.'\')',outIcon('cancel'),cfCaption('genDelete')).'</span>';

			// link type icon
			//$l=outImage(outIcon($ico),false,false,'width:16;height:16;vertical-align:middle');
			$l='';

			// file type icon & filename

			// Torrent directLink and publishDownload
			if(@$value['downloadFormat']=='torrent') $l.=outImageIcon('bittorrent','title="'.cfCaption('bittorrent').'"');

			// Torrent directLink and publishDownload
			elseif(@$value['downloadFormat']=='streaming') $l.=outImageIcon('streaming','title="Streaming"');

			// Playlist: extract some filenames and put M3U icon
			if($type=='publishAudio' && !is_dir($value['filename'])){
				$tmp='';
				foreach (explode('|',$value['filename']) as $v){
					$tmp.=', '.basename($v);
					if(strlen($tmp)>100){$tmp=cfStrTruncate($tmp,100);break;}
				}
				$l.=outImageIcon('/fi/icoM3u').cfUTF8Encode(substr($tmp,2));
			}
			// Others: display file's icon & complete filename
			else $l.=outImage(efIcon($value['filename']),false,false,'width:16;height:16;vertical-align:middle;margin-right:0.3em').cfUTF8Encode(basename($value['filename']).' ('.dirname($value['filename']).')');

			//if(isset($value['size']) && $value['size']) $l.=' ('.$value['size'].')';

			echo outFrameHeaderTable('frame3Header',$l,$r);

			// Embed code snippet
			if($type!=='directLink'){
				echo '<span style="float:left;margin-top:2;font-weight:bold;width:8em">'.cfCaption('linkCode').cfCaption('genSeparator').'</span>';
				echo '<span style="float:right;margin-top:2;margin-left:1em">'.outButtonSmall('','javascript:wl.clipboardCopy(\'ta'.$key.'\')',outIcon('paste'),cfCaption('genCopy')).'</span>';
				echo '<input id="ta'.$key.'" class="textinput" style="width:100%" onclick="this.select();return true" value="'.str_replace('"','&quot;',cfUTF8Encode(dlLinkURL($key,$value))).'" readonly="readonly">';
				echo '<br clear="all">';
			}

			// LINK
			echo '<span style="float:left;margin-top:2;font-weight:bold;width:8em;position:relative">'.cfCaption('genLink').outImageIcon('share',false,'position:absolute;right:0px').'</span>';

			// Social sharing
			echo '<span style="float:right;margin-top:2;margin-left:1em">'.outSocialURLShare(dlDirectLinkURL($key,$value),false,cfFileWithoutExtension(basename(@$value['filename']))).'</span>';
			echo '<input id="tb'.$key.'" class="textinput" style="width:100%" onclick="this.select();return true" value="'.str_replace('"','&quot;',cfUTF8Encode(dlDirectLinkURL($key,$value))).'" readonly="readonly">';

			// Link lifetime
			if($value['type']=='directLink'){
				echo cfCaption('directLinkLifetime').cfCaption('genSeparator');
				if($value['expiry']==0) echo cfCaption('genUnlimited'); else echo date(cfCaption('_FILE_DATE_FORMAT'),$value['expiry']);
			}

			$l='';
			// Max downloads number
			if($value['type']=='directLink'){
				$l.=cfCaption('directLinkLimitNb').cfCaption('genSeparator');
				if($value['limitNb']<=0) $l.=cfCaption('genUnlimited'); else $l.=cfUTF8Encode($value['limitNbOriginal']);
			}
			// Videos
			elseif($value['type']=='publishVideo'){
				$l.=cfCaption('genSize').cfCaption('genSeparator').$value['w'].'x'.$value['h'].' - '.cfCaption('genQuality').cfCaption('genSeparator');
				$tmp=array("original"=>cfCaption('videoCfgSource'),"high"=>cfCaption('genHigh'),"medium"=>cfCaption('genMedium'),"low"=>cfCaption('genLow'));
				$l.=$tmp[$value['flashOutputQuality']];
			}
			// Images
			elseif($value['type']=='publishImage'){
				$l.=cfCaption('genSize').cfCaption('genSeparator').$value['w'].'x'.$value['h'];
			}
			// Audio
			elseif($value['type']=='publishAudio'){
				$l.='';
			}


			// Completed downloads
			$r=cfCaption('directLinkCompletedDl').cfCaption('genSeparator');
			if($value['limitNb']<=0) $nb=(0-$value['limitNb']); else $nb=$value['limitNbOriginal']-$value['limitNb'];
			if($nb==0) $r.=$nb; else $r='<b>'.$r.$nb.'</b>';
			if($l) echo outFrameFooterTable('',$l,$r);
			else echo '<br>'.$r;

			echo "</div>\n";
		}
	}
	else echo '<br>';
	//echo '</div>';
	echo cfDragRegisterItems();
}

/**
 * @desc Check if a passed token is a short link
 *
 * @param string $shortLink
 * @param boolean $justReturnResult: if true, return true/false, else process short link request (proceed to /ext/download.php or /login.php)
 * @return void or array(id, type, passwordHas, resourceId)
 */
function dlResolveShortLink($shortLink, $justReturnResult){
	require_once(INCLUDE_DIR.'explorerFunctions.php');
	$shortLink=strtolower($shortLink);

	// 1st search in direct link tokens
	foreach (efTokensRead() as $id=>$token) if(cfCmpLeft(strtolower($id),$shortLink)){
		$_GET=array(
			'shortLink'=>true,
			'dlToken'=>$id,
			'resId'=>'',
			'file'=>basename($token['filename'])
		);

		if($justReturnResult) return array('id'=>$id,'type'=>$token['type'],'passwordHas'=>0,'filename'=>$token['filename']);

		// And route to download.php
		require(cfAppDocRoot().'/ext/download.php');
		exit;
	}

	// then search into resource direct access links
	foreach (WEnv::resConfigs() as $rid=>$resConfig){
		$id=$resConfig->getVar('directResourceAccessToken');
		if(cfCmpLeft(strtolower($id),$shortLink)){
			// "Rebuild HTTP" request
			$_SERVER['REQUEST_URI']='/index.php?directResourceAccessToken='.$id;
			$_SERVER['PHP_SELF']='/index.php';
			$_GET=array('directResourceAccessToken'=>$id);

			if($justReturnResult) return array('id'=>$id,'type'=>'resource','passwordHas'=>$resConfig->getVar('directResourceAccessPasswordHas'),'resourceId'=>$rid);

			// And route to index.php
			require(cfAppDocRoot().'/index.php');
			exit;
		}
	}

	return false;
}
?>