<?php
/**
 * Update an HTML page on a personal website throuht FTP
 * Asynchronously called by cfLogEvent not to block calling script if this script takes time to respond
 * Basic presentation:
 * - Retreive HTML template located into theme directory.
 *  (This template is copied from /www/local/logOutput if not existing, thus allowing user to modify template located into theme dir)
 * - retreive previous events (such as server state or last connections / resources views) from /data/lastDisplayedFTPEvents.txt
 * 	(this file is a serialized array :
 * 	'serverState'=>array(timestamp,logstring)
 *  'events'>array(timestamp=>logstring, timestamp=>eventType/logstring, timestamp=>logstring, ...)
 *
 * - rebuild page mixing new event and previous ones
 * - send to FTP server
 *
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 2.0.0
 */

ignore_user_abort(true);

if(!($logString=base64_decode(@$_GET['b64LogString']))) die();
$extraData=base64_decode(@$_GET['b64ExtraData']); // userId/resourceId

// Get Growl configuration
$growlConfig=@unserialize(cfStoredPasswordDecrypt(cfGGetVar('growlEventConfig')));


/**
 * Server state
 */
if($_GET['eventType']==EVENT_SERVERSTATE){
	$regInfo=cfRegistrationData();
	$title=cfCaption('apacheStartedWeb',false,false,false,true);
	$logString='';
	if(isset($regInfo['regName'])) $logString.=cfSiteLink($regInfo['regName']);
	if(@$growlConfig['displayIP']) $logString.="\n".cfExternalHostName();
	require_once(INCLUDE_DIR.'outputFunctions.php');
	$icon=cfAppDocRoot().outIcon('serverOnline');
}

/**
 * New connection
 */
if($_GET['eventType']==EVENT_CONNECTION){
	$title=cfCaption('msgNewConnection');
	$userId=$extraData;
	// Retreive user Icon
	if($icon=cfArrayItem(cfMGetVar('weezoUsers'),$userId,'icon')) $icon=cfAppDocRoot().'/gfx/icons/'.$icon;
}

/**
 * Resource access
 */
if($_GET['eventType']==EVENT_RESOURCEACCESS){
	list($userId,$resourceId)=explode('/',$extraData);
	// Set user name as title
	$title=cfArrayItem(cfMGetVar('weezoUsers'),$userId,'name');
	// Get resource icon (32x32)
	if($icon=cfArrayItem(cfMGetVar('weezoResData'.$resourceId),'definition','resourceIcon')) $icon=cfAppDocRoot().$icon;
}

require_once(INCLUDE_DIR.'Net/Growl.php');
define('GROWL_NOTIFY_STATUS', 'GROWL_NOTIFY_STATUS');

// define a PHP application that sends notifications to Growl

$appName = APPLICATION_NAME;
$notifications = array(GROWL_NOTIFY_STATUS => array('display' => 'Status'));
$password = @$growlConfig['password'];
//cfDbg($title."\n".$logString."\n".@$icon);
$options  = array('host' => '127.0.0.1', 'protocol' => 'tcp', 'port' => Net_Growl::GNTP_PORT, 'timeout' => 7, 'AppIcon' => cfAppDocRoot().'/gfx/appIcon.png');

try {
	// Register application
    $growl = Net_Growl::singleton($appName, $notifications, $password, $options);
    $growl->register();

    // Send message
    $name        = GROWL_NOTIFY_STATUS;
    $options     = array('sticky' => false);
    if(@$icon) $options['icon']=$icon;
    $title=cfUTF8Decode($title,true,false,false);
    $logString=cfUTF8Decode($logString,true,false,false);

    $growl->notify($name, $title, $logString, $options);
}
catch (Net_Growl_Exception $e) {}
