<?php
/**
 * General options configuration
 *
 * this script is included by index.php and shouldn't be directly called
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2006 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

define('LOGFILE_DISPLAYED_SIZE',4096); // Max size of displayed application log
define('LOGFILE_UPDATE_INTERVAL',2000); // update interval (ms) for application log file

require_once(INCLUDE_DIR.'resourceConfigFunctions.php');

$displayFinished=false; // Set this value to true to indicate not to display HTML page and JS Stuff (used when dumping a log to output)
$logFile=cfAppLogDir().'/'.MAIN_LOG_FILE; // Application log file


/**
 * @desc Read tail of log file and parse lines
 *
 * @return array of utf8 encoded lines
 */
function getRawLines(){
	global $logFile;

	// Read log
	if($fp=fopen($logFile,'r')){
		$start=max(0,filesize($logFile)-120*cfGGetVar('applicationLogMaxLines'));
		fseek($fp,$start); // Read only end of file
		$lines=fread($fp,filesize($logFile)-$start);
		fclose($fp);
		$lines=explode("\n",cfUTF8Encode($lines));
	}
	else $lines=array();
	return $lines;
}



/**
 * @desc Format log line
 */
function formatedLine($l,&$time){
	static $logLevel,$sep,$morning=0,$icoLocal,$icoLan,$icoWeb,$icoApp,$icoErr;

	// Init function
	if(!$morning) {
		$morning=mktime(0,0,0,date("m"),date("d"),date("Y"));
		$logLevel=cfGGetVar('applicationLogDetail');
		$sep=cfCaption('genSeparator');
		$icoLocal=outImageIcon('localhost','','width:12px;height:12px'); // prod
		$icoLan=outImageIcon('LAN2','','width:12px;height:12px');
		$icoWeb=outImageIcon('web','','width:12px;height:12px');
		$icoApp='<img src="'.outIconApp(16).'" style="width:12px;height:12px;vertical-align:middle;margin-right:0.5em">';
		$icoErr=outImageIcon('warning','','width:12px;height:12px');
	}
	if(!($l=str_replace("\r","",$l))) return;

	// Parse list
	@list($ts,$src,$level,$ip,$str)=explode('/',$l,5);
	if(!$ip) return '<div class="dbg">'.$l.'/</div>';

	$time=$ts;
	$str=str_replace('<','&lt;',$str);

	// Only display if level >= logLevel
	if($logLevel=='dbg' || ($logLevel=='det'&& $level=='det') || $level=='inf'){

		// Icon
		if($src=='app') $ico=$icoApp;
		elseif($ip=='127.0.0.1') $ico=$icoLocal;
		elseif(cfIsOnLAN($ip)) $ico=$icoLan;
		else $ico=$icoWeb;

		// Return formated line
		return '<div class="'.$level.'">'.$ico.(($level=='err')?$icoErr:'').cfFixedDate(cfCaption((($ts>$morning)?'_HMS_DATE_FORMAT':'_FULL_DATE_FORMAT')),$ts).$sep.$str.'</div>';
	}
	return '';
}

/**
 * @desc Return HTML code for users table and disconnect button
 * 		Save list of sessions to RVar to accelerate need for async update computations
 *
 * @return string
 */
function connectedUsersTable(){
	$nb=0;$connectedUsers='';
	$sessions=glob(cfAppDataDir().'/sessiondata/sess_*');

	// Save list of sessions
	cfRSetVar('lastSessionUpdate',WEnv::usersLogged()->lastUpdate);

	// Browse sessions
	foreach (WEnv::usersLogged()->getUsers(true) as $sessId=>$userLogged){
		if(0) $userLogged=new WUserLogged(); // Zend IDE WA
		$browserIcon='/gfx/browsers/'.$userLogged->browserName.'.png';
		if(!file_exists(cfAppDocRoot().$browserIcon)) $browserIcon='/gfx/browsers/Default.png';


		// Disconnect button
		if($sessId==wSession_id()) {
			$connectedUsers.=outButtonDisabled(cfCaption('genLogout'),false,outIcon('logout'),false,false,'style="margin-right:1em;"');
		}
		else {
			$connectedUsers.=outButton(cfCaption('genLogout'),'javascript:disconnect(\''.$sessId.'\')',outIcon('logout'),false,false,'style="margin-right:1em;"');
		}

		// Ban button
		if($sessId!==wSession_id() && $userLogged->ip!=='127.0.0.1') {
			$connectedUsers.=outButton(cfCaption('genBan'),'javascript:ban(\''.$sessId.'\')',outIcon('forbidden'),false,false,'style="margin-right:1em;"');
		}
		else{
			$connectedUsers.=outButtonDisabled(cfCaption('genBan'),false,outIcon('forbidden'),false,false,'style="margin-right:1em;"');
		}

		// Browser icon
		$connectedUsers.=outImage($browserIcon,false,false,'vertical-align:middle;margin-right:0.5em');

		// Name & IP
		$connectedUsers.='<span style="margin:1em"><b>'.cfUTF8Encode($userLogged->groupName.(($userLogged->pseudo)?' - '.$userLogged->pseudo:'')).'</b> ('.cfUTF8Encode($userLogged->ip).')</span>';
		// Connection date
		$connectedUsers.='<span>&nbsp;'.cfCaption('activityConnectedSince').'&nbsp;'.cfUTF8Encode(@date(cfCaption('_FULL_DATE_FORMAT'),$userLogged->connectionTime)).'</span><br/>';
		$nb++;

	}
	return outFrameHeaderTable('frame2Header',cfCaption('adminConnectedUsers'),cfCaption('mainNumberConnectedUsers',$nb)).$connectedUsers;
}




/*
 ***************************************************************************************************************************
 * Asynchronous requests
 ***************************************************************************************************************************
 */
if(cfIsAsync()){
	cfAsyncHeader();

	// Change log level
	if(isset($_POST['applicationLogDetail'])){
		$logLevel=$_POST['applicationLogDetail'];
		if($logLevel=='inf' || $logLevel=='det' || $logLevel=='dbg') WEnv::config()->updateVar('applicationLogDetail',$logLevel);
		$_POST['appLogMTime']=0;
		$_POST['appLogUpdate']=true;
		echo cfAsyncJSaction('appLogClear()');
	}
	// Update application log and connected users list
	if(isset($_POST['appLogUpdate'])){
		// If any modification in session files, refresh users display
		if(cfRGetVar('lastSessionUpdate')<WEnv::usersLogged()->lastUpdate) echo cfAsyncXMLInnerHTMLbyId(connectedUsersTable(),'usersDiv');

		// Update log
		if(($filemtime=filemtime($logFile))>$_POST['appLogMTime']){

			// Read log
			$lines=getRawLines();
			$nb=count($lines);

			$logTime=$_POST['appLogMTime'];
			$out='';
			$skipped=0;
			for ($i=0;$i<$skipped+cfGGetVar('applicationLogMaxLines') && $nb-$i-2-$skipped>=0;$i++) {
				$line=formatedLine($lines[$nb-$i-2-$skipped],$time);
				if($line){
					if($time>=$_POST['appLogMTime']){
						$logTime=$time;
						$out.=$line;
					}
					else $skipped++;
				}
			}
			unset($lines);
			cfRSetVar('prevLogTime',$logTime);

			// Send updated log part to output
			if($out) {
				echo cfAsyncXMLInnerHTMLbyId($out,'logApplicationTmpDiv');
				echo cfAsyncJSaction('appLogUpdated("'.$logTime.'",'.$filemtime.')');
				die(cfAsyncFooter());
			}
		}
	}
	echo cfAsyncJSaction('appLogUpdated()');
	die(cfAsyncFooter());
}


/*
 * Verify access rights
*/
if($_SERVER['REMOTE_ADDR']!='127.0.0.1'){
	if(cfUGetVar('administrator')!=true || !cfGGetVar('remoteAdministrationAuthorized')) outDisplayErrorPage(cfcaption('errorRemoteAdministrationNotAuthorized'));
}

?>
<script type="text/javascript">function reloadPage(){D.location="<?php echo $_SERVER['PHP_SELF'];?>";}</script>
<?php

/*
 ***************************************************************************************************************************
 * Process POST commands
 ***************************************************************************************************************************
 */
if(isset($_POST['command'])){
	$command=cfUTF8Decode($_POST['command']);
	$completeFilename=false;
	switch ($command){
		// Ban user (then disconnect)
		case 'ban':
			$file=cfAppDataDir().'/sessionData/'.cfUTF8Decode($_POST['argument']);
			if(file_exists($file)) {
				$sess=cfUnserializeSession(file_get_contents($file));
				cfIPBan($sess['accountIP']);
			}
		// Disconnect user
		case 'disconnect':
			WEnv::usersLogged()->disconnectUser(cfUTF8Decode($_POST['argument']));
		break;
		// Restart server
		case 'restartServer':
		echo outDivFrame('frame1').'<div class="frame1Header">'.cfCaption('adminGeneral').'</div><br><br><center>'.cfCaption('apacheRestarting').'</center><br><br></div>';
		echo '<script type="text/javascript">window.setTimeout("reloadPage()",12000);</script>';
		cfServerSendCommand('restartServer');
		$displayFinished=true;
		break;
		// Stop server
		case 'stopServer':
		echo outDivFrame('frame1').'<div class="frame1Header">'.cfCaption('adminGeneral').'</div><br><br><center>'.cfCaption('apacheStopping').'</center><br><br></div><br>';
		cfServerSendCommand('stopServer');
		$displayFinished=true;
		break;
		// Reboot computer
		case 'restartComputer':
		echo outDivFrame('frame1').'<div class="frame1Header">'.cfCaption('adminGeneral').'</div><br><br><center>'.cfCaption('computerRestarting').'</center><br><br></div><br>';
		echo '<script type="text/javascript">window.setTimeout("reloadPage()",60000);</script>';
		cfServerSendCommand('restartComputer');
		$displayFinished=true;
		break;
		// Stop computer
		case 'stopComputer':
		echo outDivFrame('frame1').'<div class="frame1Header">'.cfCaption('adminGeneral').'</div><br><br><center>'.cfCaption('computerStopping').'</center><br><br></div><br>';
		cfServerSendCommand('stopComputer');
		$displayFinished=true;
		break;
		// Reboot Weezo
		case 'restartWeezo':
		echo outDivFrame('frame1').'<div class="frame1Header">'.cfCaption('adminGeneral').'</div><br><br><center>'.cfCaption('genRestarting').'</center><br><br></div>';
		cfServerSendCmd('restartWeezo'); // Ask Weezo to reboot itself through weezoReboot (works fine if Weezo is OK)
		sleep(5);
		cfExecFork(cfAppBinDir().'/weezoReboot.exe','SW_HIDE','restart'); // Ask weezoReboot to relaunch Weezo (if Weezo didn't responded)
		$displayFinished=true;
		break;
		// Site appearance modification
		case 'changeSite':
		cfGSetVar('pageTitle',cfStrTruncate(cfUTF8Decode($_POST['argument']),255));
		cfServerSendCommand('changeParameter name="pageTitle" value="'.cfGGetVar('pageTitle').'"');
		cfGSetVar('theme',basename(cfStrTruncate(cfUTF8Decode($_POST['argument2']),64)));
		cfServerSendCommand('changeParameter name="theme" value="'.cfGGetVar('theme').'"');
		echo '<script type="text/javascript">parent.parent.location.href="/index.php"</script>';
		$displayFinished=true;
		break;
		// Apache log
		case 'logApacheAccess':
		$completeFilename=cfAppLogDir().'/'.HTTPD_ACCESS_LOG_FILE;
		// Apache error log
		case 'logApacheError':
		if(!$completeFilename) $completeFilename=cfAppLogDir().'/'.HTTPD_ERRORS_LOG_FILE;
		// Application log
		case 'logApplication':
		if(!$completeFilename) $completeFilename=cfAppLogDir().'/'.MAIN_LOG_FILE;
		if(file_exists($completeFilename) && ($handle = fopen ($completeFilename, "r"))){
			echo outDivFrame('frame1');
			echo outFrameHeaderTable('frame1Header',outImage(outIcon('fi/icoText'),false,false,'margin-right:0.5em').cfCaption(($command=='logApacheAccess')?'logApache':$command),outButton(cfCaption('genBack'),'?panel=general',outIcon('back')));
			$bufferSize=1024;
			echo '<pre>';
			while (!feof($handle)) echo(cfUTF8Encode((fread($handle, $bufferSize)))); flush();
			fclose($handle);
			echo '</pre>';
			echo "</div>\n";
			$displayFinished=true;
		}
		break;
		case 'toggleShowProcesses':
			cfRSetVar('showProcesses',!cfRGetVar('showProcesses'));
			if(!cfRGetVar('advancedAdministration')) cfRSetVar('showProcesses',false);
		break;
		// Kill a process
		case 'killProcess':
		if(is_numeric($_POST['argument'])){
			foreach (cfProcList() as $proc){
				if($proc['pid']==$_POST['argument'] && mayKill($proc['exe'])) cfTerminateProcess($_POST['argument']);
			}
		}
		break;
	}
}


if(!$displayFinished){
	// Loading mask
	echo outLoadMask(false);
	flush();
/*
 ***************************************************************************************************************************
 * Javascript functions
 ***************************************************************************************************************************
 */
?>
<script type="text/javascript">
function restartServer(){dgi('command').value="restartServer";D.commandForm.submit();}
function stopServer(){if(confirm('<?php echo cfCaptionJS('adminConfirmServerShutdown');?>')) {dgi('command').value="stopServer";D.commandForm.submit();}}
function restartComputer(){if(confirm('<?php echo cfCaptionJS('adminConfirmComputerShutdown');?>')) {dgi('command').value="restartComputer";D.commandForm.submit()}}
function restartWeezo(){dgi('command').value="restartWeezo";D.commandForm.submit();}
function stopComputer(){if(confirm('<?php echo cfCaptionJS('adminConfirmComputerShutdown');?>')) {dgi('command').value="stopComputer";D.commandForm.submit();}}
function disconnect(sess_id){
	dgi('command').value="disconnect";
	dgi('argument').value=sess_id;
	D.commandForm.submit();
}
function ban(sess_id){
	dgi('command').value="ban";
	dgi('argument').value=sess_id;
	D.commandForm.submit();
}
function changeSite(){
	dgi('command').value="changeSite";
	dgi('argument').value=dgn('pageTitle').value;
	dgi('argument2').value=dgn('theme').value;
	D.commandForm.submit();
}
function logApacheAccess(){dgi('command').value="logApacheAccess";D.commandForm.submit();}
function logApacheError(){dgi('command').value="logApacheError";D.commandForm.submit();}
function logApplication(){dgi('command').value="logApplication";D.commandForm.submit();}
function kp(pid){dgi('command').value="killProcess";dgi('argument').value=pid;D.commandForm.submit();}
function toggleShowProcesses(){dgi('command').value="toggleShowProcesses";D.commandForm.submit();}
</script>
<form name="commandForm" style="display:none" method="POST" enctype="multipart/form-data" action="<?php echo $_SERVER['PHP_SELF'];?>">
	<input type="hidden" name="command" id="command">
	<input type="hidden" name="argument" id="argument">
	<input type="hidden" name="argument2" id="argument2">
	<input type="hidden" name="resId" value="<?php echo $_SESSION['activeResourceId']; ?>">
</form>
<?php

if(cfIsMobile()) return ;


/*
 ***************************************************************************************************************************
 * Display general tab
 ***************************************************************************************************************************
 */
echo outDivFrame('frame1');
echo '<div class="frame1Header">'.outThemeImage(outicon('config'),false,false,'margin-right:1em').cfCaption('adminGeneral').'</div>';

// Logs
echo outDivFrame('frame2');

$select='<select size="1" onchange="wl.asr.send(\'applicationLogDetail=\'+this.value)">';
$select.='<option value="inf" name="applicationLogDetailInf" '.((cfGGetVar('applicationLogDetail')=='inf')?'selected="selected"':'').'>'.cfCaption('activityLogDetailInf').'</option>';
$select.='<option value="det" name="applicationLogDetailDet" '.((cfGGetVar('applicationLogDetail')=='det')?'selected="selected"':'').'>'.cfCaption('activityLogDetailDet').'</option>';
if(cfGGetVar('UIDebug')) $select.='<option value="dbg" name="applicationLogDetailDet" '.((cfGGetVar('applicationLogDetail')=='dbg')?'selected="selected"':'').'>'.cfCaption('activityLogDetailDbg').'</option>';
$select.='</select>';
echo outFrameHeaderTable('frame2Header',cfCaption('activityLogs'),$select);
echo outImage('Log.png',false,false,'margin-right:2em;float:left');
echo outButton(cfCaption('logApache'),'javascript:logApacheAccess()',outIcon('log2'),false,false,'style="margin-right:2em"',true);
echo outButton(cfCaption('logApacheError'),'javascript:logApacheError()',outIcon('log2'),false,false,'style="margin-right:2em"',true);

/**
 * Application real-time updated log
 */
?>
<div id="logApplicationTmpDiv" style="display:none"></div>
<div class="frame3" style="overflow:auto;height:10em" id="logApplicationDiv">
<?php

// Read log
$lines=getRawLines();
$nb=count($lines);
$logTime=0;

$skipped=0;
for ($i=0;$i<$skipped+cfGGetVar('applicationLogMaxLines') && $nb-$i-2-$skipped>=0;$i++) {
	$line=formatedLine($lines[$nb-$i-$skipped-2],$time);
	if($line) echo $line; else $skipped++;
	if($time>$logTime) $logTime=$time;
}

unset($lines);
cfRSetVar('prevLogTime',$logTime);

?>
<script type="text/javascript">
var appLogTimePos="<?php echo $logTime;?>",appLogMTime="<?php echo @filemtime($logFile);?>";
function appLogUpdate(){
	wl.asr.send({'appLogUpdate':1,'appLogTimePos':appLogTimePos,'appLogMTime':appLogMTime})
}
function appLogClear(){(dgi('logApplicationDiv').innerHTML='')}
function appLogUpdated(aLTP,aLMT){
	var d=$('logApplicationTmpDiv').innerHTML;
	if(d){
		$('logApplicationTmpDiv').innerHTML='';
		dgi('logApplicationDiv').innerHTML=d+dgi('logApplicationDiv').innerHTML;
		appLogTimePos=aLTP;
		appLogMTime=aLMT;
	}
	setTimeout(appLogUpdate,<?php echo LOGFILE_UPDATE_INTERVAL;?>);
}
setTimeout(appLogUpdate,<?php echo LOGFILE_UPDATE_INTERVAL;?>);
</script>
</div>
<?php
echo "</div>\n";

/**
 * Connected users
 */
echo outDivFrame('frame2','id="usersDiv"');
echo connectedUsersTable();
echo "</div>\n";

/**
 * Bittorrent seeder
 */
echo outDivFrame('frame2');
echo '<div class="frame2Header">'.cfCaption('bittorrent').'</div>';
if(cfGGetVar('bittorrentEnabled')) {
	echo outImage(outIcon('isOn'),false,false,'margin-right:1em;vertical-align:middle');
	echo cfCaption('bittorrent').cfCaption('genSeparator').cfCaption('genActivated').'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
	// Ping seeder
	if($sock=@fsockopen('127.0.0.1',cfGGetVar('bittorrentPort'),$errno,$errstr,3)){
		fclose($sock);
		echo outImage(outIcon('isOn'),false,false,'margin-right:1em;vertical-align:middle');
		echo 'Seeder'.cfCaption('genSeparator').cfCaption('genActivated').'<br/>';
	}
	else {
		echo outImage(outIcon('isOff'),false,false,'margin-right:1em;vertical-align:middle');
		echo 'Seeder'.cfCaption('genSeparator').cfCaption('genActivatedFalse').'<br/>';
	}

}
else {
	echo outImage(outIcon('isOff'),false,false,'margin-right:1em;vertical-align:middle');
	echo cfCaption('bittorrent').cfCaption('genSeparator').cfCaption('genActivatedFalse').'&nbsp;&nbsp;&nbsp;&nbsp;';
	echo outImage(outIcon('isOff'),false,false,'margin-right:1em;vertical-align:middle');
	echo 'Seeder'.cfCaption('genSeparator').cfCaption('genActivatedFalse').'<br>';
}
echo "</div>\n";

if(cfRGetVar('advancedAdministration')){
	echo outTableTransparent('frame1','position:relative').'<tr><td>';
// Server Shutdown
	echo outDivFrame('frame2',false,'vertical-align:top');
	echo '<div class="frame2Header">'.cfCaption('mainServer').'</div>';
	echo outTableTransparent('frame2');
	echo '<tr style="text-align:center"><td width="50%" valign="top">';
	echo outImage('serverRestart.png',false,false,'margin-bottom:0.5em').'<br>'.outButton(cfCaption('adminRestart'),'javascript:restartServer()',outIcon('change'));
	echo '</td><td style="text-align:center;width:50%">';
	echo outImage('serverShutdown.png',false,false,'margin-bottom:0.5em').'<br>'.outButton(cfCaption('menuServerOff'),'javascript:stopServer()',outIcon('off'));
	echo '</td></tr></table>';
	echo "</div>\n";

	echo '</td><td>&nbsp;</td><td>';
// Computer turn off / restart
	echo outDivFrame('frame2');
	if(!cfRGetVar('advancedAdministration')) echo '<div class="frame2Header">'.cfCaption('adminComputer').'</div>';
	else{
		echo outFrameHeaderTable('frame2Header',cfCaption('adminComputer'),outButton(cfCaption('adminProcesses'),'javascript:toggleShowProcesses()',((cfRGetVar('showProcesses'))?outIcon('minus'):outIcon('plus'))));

	}
	echo outTableTransparent('frame2');
	echo '<tr style="text-align:center"><td width="50%" valign="top">';
	echo outImage('computerRestart.png',false,false,'margin-bottom:0.5em').'<br>'.outButton(cfCaption('adminRestart'),'javascript:restartComputer()',outIcon('change'));
	echo '</td><td style="text-align:center;width:50%">';
	echo outImage('computerShutdown.png',false,false,'margin-bottom:0.5em').'<br>'.outButton(cfCaption('adminComputerShutdown'),'javascript:stopComputer()',outIcon('off'));
	echo '</td></tr></table>';
	echo "</div>";

	echo '</td><td>&nbsp;</td><td>';
// Weezo reboot
	echo outDivFrame('frame2');
	echo '<div class="frame2Header">Weezo</div>';
	echo outTableTransparent('frame2');
	echo '<tr style="text-align:center"><td valign="top">';
	echo outImage('weezoRestart.png',false,false,'margin-bottom:0.5em').'<br>'.outButton(cfCaption('adminRestart'),'javascript:restartWeezo()',outIcon('change'));
	echo '</td></tr></table>';
	echo "</div>";

	echo '</td></tr></table>';
	}

}
/**
 * @desc Return false if user may not kill process
 *
 * @param string $processName
 * @return boolean
 */
function mayKill($processName,$completeFilename=false){
	$forbidden=array('weezohttpd','weezo','explorer','system','smss','winlogon','services','interrupts','dpcs','ccapp','[system process]');

	if(!$completeFilename){$completeFilename=$processName;$processName=basename($processName);}
	$processName=cfFileWithoutExtension(strtolower($processName));
	$completeFilename=strtolower($completeFilename);
	if(strpos($completeFilename,'antivi')!==false || strpos($completeFilename,'firewa')!==false) return false;
	if(in_array($processName,$forbidden)) return false;
	return true;
}


// Close session (there seems to be problems with process functions, so if there is any failure, user won't get stuck)
wSession_write_close();


/**
 * Process management
 */
if(cfRGetVar('advancedAdministration') && cfRGetVar('showProcesses')){
	echo '<div class="frame2">';
	echo '<div class="frame2Header">'.(cfCaption('adminProcesses'))."</div>\n";

	// Get CPU usage
	$t=microtime(true);
	foreach (@win32_ps_list_procs() as $proc){
		$cpu=@win32_ps_stat_proc($proc['pid']);
		$p[$proc['pid']]=0*$cpu['tms']['kernel']+$cpu['tms']['user'];
	}

	usleep(500000);
	$t=microtime(true)-$t;
	foreach (cfProcList() as $proc){
		// Get stats about process (CPU & memory)
		if(isset($proc['stat'])) $cpu=$proc['stat']; else $cpu=@win32_ps_stat_proc($proc['pid']);
		if(isset($p[$proc['pid']])) $c=floor(5000*(0*$cpu['tms']['kernel']+$cpu['tms']['user']-$p[$proc['pid']])/$t)/100; else $c=0;
		$process[$proc['pid']]=array(
			'pid'=>$proc['pid'],
			'shortName'=>strtolower(basename($proc['exe'])),
			'name'=>(basename(cfFileWithoutExtension($proc['exe']))),
			'exe'=>$proc['exe'],
			'offset'=>0,
			'ws'=>isset($proc['mem']['working_set_size'])?$proc['mem']['working_set_size']:0,
			'cpu'=>$c
			);
	}


	// Get Parent/child relations
	foreach(cfProcParentPID() as $child=>$tmp){
		list($parent,$exeName)=explode(':',$tmp);
		$rel[$child]=$parent;
		if(!isset($process[$child])) $process[$child]=array(
			'pid'=>$child,
			'shortName'=>strtolower(basename($exeName)),
			'name'=>$exeName,
			'exe'=>$exeName,
			'offset'=>0,
			'ws'=>0,
			'cpu'=>0
			);
		$process[$child]['parent']=$parent;
	}

	ksort($process);
	//cfVarDump($process);

	// Get 1st level processes
	$p2=$process;
	$process=array();
	foreach ($p2 as $pid=>$proc){
		if($pid==0 || !isset($p2[$rel[$pid]])){
			$process[$pid]=$proc+array('offset'=>0);
			unset($rel[$pid]);
		}
	}

	while (count($rel)) {
		foreach ($p2 as $pid=>$proc){
			if(isset($rel[$pid])){
				$parent=$rel[$pid];
				if(isset($process[$parent])){
					$tmp=array();
					foreach ($process as $key=>$value){
						$tmp[$key]=$value;
						if($key==$parent) {
							$tmp[$pid]=$proc;
							$tmp[$pid]['offset']=$process[$parent]['offset']+1;
						}
					}
					$process=$tmp;
					unset($tmp);
					unset($rel[$pid]);
				}
			}
		}
	}

	/**
	 * Display processes
	 */
	echo outTableTransparent('frame2','text-align:left');
	echo '<tr style="font-weight:bold"><td style="text-align:right">PID &nbsp;</td><td style="text-align:left">'.cfCaption('genName').'</td>'.((cfBGetVar('screenSize')=='monitor')?'<td>EXE</td>':'').'</td><td style="text-align:right;padding-right:2em">'.cfCaption('adminProcessesMemory').'</td><td>CPU</td><td> </td></tr>';
	$i=0;
	require(INCLUDE_DIR.'explorerFunctions.php');
	foreach ($process as $proc){
		// Get process' icon src
		if(file_exists($proc['exe'])) $iconSrc=cfExtImage($proc['exe'],16,16,true,0,array('icon'=>1));
		else $iconSrc=outIcon('fi/icoExe');

		echo '<tr id="pid'.$proc['pid'].'" class="eTC '.(($i%2)?'odd':'even').'" eH=1>';
		echo '<td class="eTCLeft">'.$proc['pid'].' &nbsp;</td>';
		echo '<td style="font-weight:bold;text-align:left;padding-left:'.(16*$proc['offset']).'px">';
		echo '<img alt="" src="'.$iconSrc.'" style="vertical-align:middle;margin-right:0.5em">'.cfUTF8Encode($proc['name']).'</td>';
		if(cfBGetVar('screenSize')=='monitor') echo '<td style="overflow:hidden;text-overflow:ellipsis">'.cfUTF8Encode($proc['exe']).'</td>';

		$killBt='';
		if(cfRGetVar('remoteDesktopAuthorized') && mayKill($proc['name'],$proc['exe']))
			$killBt=outButton(cfCaption('genClose'),'javascript:kp(\''.$proc['pid'].'\')',outIcon('stop'));
		echo '<td style="text-align:right;padding-right:2em">'.(($proc['ws'])?efFileSizeFormat($proc['ws']):'-').'</td>';
		echo '<td id="cpu'.$i.'">'.$proc['cpu'].'</td>';
		echo '<td class="eTCRight">'.$killBt."</td></tr>\n\n";
		$i++;
	}
	echo "</table></div>\n";
	echo '<div id="PBT" style="display:none">'.outProgressBar(0.99,'80px','CAPT').'</div>';
?>
<script type="text/javascript">
pbt=dgi('PBT').innerHTML;
var cpus=D.getElementsByTagName('td');
for(var i=0;i<cpus.length;i++) {
	cpu=cpus[i];
	if(cpu.id && cpu.id.substr(0,3)=='cpu')	cpu.innerHTML=pbt.replace(/99/,cpu.innerHTML).replace(/CAPT/,cpu.innerHTML+'%');
	if(cpu.id) n=1;
}
</script>
</div>
<?php
}

//Appearance
echo outDivFrame('frame2');
echo '<div class="frame2Header">'.cfCaption('genSite').'</div>';
echo outImage('Colors.png',false,false,'margin-right:2em;float:left');
echo cfCaption('configWebsiteTitle').'<input class="textInput" name="pageTitle" style="margin-left:1em;margin-right:1em;" type="text" size="20" value="'.cfGGetVar('pageTitle').'">';
echo rcThemeSelectionControl(cfGGetVar('theme'),array('noWindowSizeControl'=>1));
echo outButton(cfCaption('genOK'),'javascript:changeSite()',outIcon('ok16')).'<br clear="all">';
echo "</div>\n";
?>
