<?php
/**
 * WEBCAM
 *
 * Display a page with webcam image
 * Page refreshes itself to update image
 * webcam image is obtained through cfServerSendCommand function
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

require_once(INCLUDE_DIR.'outputFunctions.php');
require_once(INCLUDE_DIR.'viewVideoFunctions.php');
require_once(INCLUDE_DIR.'webcamFunctions.php');

$_ENV['webcams']=webcamGetList();

/**
 * Display an error page when no Webcam is found. This page will perdiodically check for webcam
 *
 */
function resWebcamErrorPage(){
	if(cfIsAsync()){
		cfAsyncHeader();
		cfAsyncFooter();
	}
	if(cfIsMobile()){
		require_once(INCLUDE_DIR.'mobileFunctions.php');
		mfInsertHead(array('topControlsLabel'=>cfRGetVar('name')));
?>
<div style="position:absolute;top:40%;width:100%;text-align:center" class="warning"><?php echo outImage(outIcon('alert32'),false,false,'margin-right:1em;vertical-align:middle').cfCaption('webcamError');?></div>
<?php
		exit;
	}
	outDisplayErrorPage(cfCaption('webcamError'),false,false);
?>
<script type="text/javascript">
wl.setInterval(function(){wl.asr.send({'webcamListCheck':1})},5100)
</script>
<?php
	exit;
}

/**
 * @desc Return true if a (allowed) webcam is detected
 * 		 If selected camera is not available and another one is available, automatically select id
 *
 * @return boolean
 */
function resWebcamFound(){

	if(!count($_ENV['webcams'])) return false;
	if(isset($_ENV['webcams']['video'][cfRGetVar('videoSource')])) return true;
	if(cfRGetVar('sourceLocked')) return false;

	// Selected webcam is not found but user may change source: select 1st found webcam
	foreach ($_ENV['webcams']['video'] as $k=>$v) {
		cfRGetVar('videoSource',$k);

		// Async refresh: reload page
		if(cfIsAsync() && cfRGetVar('resourceInitialized')){
			cfAsyncHeader();
			echo cfAsyncJSaction('wl.goURL()');
			die(cfAsyncFooter());
		}
		break;
	}
	return true;
}

/**
 * @desc Process POST command
 *
 */
function resWebcamProcessPOST(){

	if(cfIsAsync()) cfAsyncHeader();

	// Toggle between still images and video mode
	if(@$_POST['data4']=='setVideoPlayer' && !cfRGetVar('renderingLocked')){
		cfRSetVar('stillImages',(@$_POST['data5']=='stillImages'));
		if(cfIsAsync()) {
			echo cfAsyncJSaction('wl.goURL()');
			cfAsyncFooter();
		}
	}

	// Video mode: force POST data to enter videoFunctions
	if(!cfRGetVar('stillImages') && !@($_POST['data3'])) $_POST['data3']='view';




	// Change still images refresh rate
	if(@$_POST['data3']=='setRefreshRate' && !cfRGetVar('renderingLocked')){
		if(is_numeric(@$_POST['data4']) && $_POST['data4']>=2 && $_POST['data4']<1001) cfRSetVar('refreshRate',$_POST['data4']);
	}
	// Change ouput resolution
	if(@$_POST['data3']=='changeWidth' && !cfRGetVar('renderingLocked')){
		if(is_numeric(@$_POST['data4']) && $_POST['data4']>=160 && $_POST['data4']<1920) cfRSetVar('flashOutputSize',$_POST['data4']);
	}
	// Change video source
	if(@$_POST['data3']=='setVideoSource' && !cfRGetVar('sourceLocked')){
		$list=webcamGetList();
		if(isset($list['video'][@$_POST['data4']])) cfRSetVar('videoSource',$_POST['data4']);
		$_POST['data3']='view';
	}
	// Change audio source
	if(@$_POST['data3']=='setAudioSource' && !cfRGetVar('sourceLocked')){
		$list=webcamGetList();
		if(isset($list['audio'][@$_POST['data4']])) cfRSetVar('audioSource',$_POST['data4']);
		$_POST['data3']='view';
	}



	/**
	 * Video mode: set minimal vars and call video functions
	 */
	if(@$_POST['data3']=='view' && !cfRGetVar('stillImages')){

		// Set displayed video filename
		cfRSetVar('vvfForcedVideoName',@$_ENV['webcams']['video'][cfRGetVar('videoSource')]['label']);

		// force channel external id into comform, to prevent url to be set in
		cfRSetVar('vvfForcedVideoExternalId','webcam:'.cfRGetVar('videoSource').':'.cfRGetVar('audioSource'));

		// Set singleVideo value, that will be used by /ext/video.php script
		cfRSetVar('singleVideo','webcam:'.cfRGetVar('videoSource').':'.cfRGetVar('audioSource'));


		// Call video functions to display player and controls
		if(cfIsMobile()){
			require_once(INCLUDE_DIR.'mobileFunctions.php');
			require_once(INCLUDE_DIR.'mobileViewFunctions.php');

			VVOptions::setMobileTopControlsExtra(
				mfButton(false,"javascript:fAS('','view','setVideoPlayer','stillImages');",outIcon('webcamImages'))
			);


			mvfViewFileVideo('webcam:'.cfRGetVar('videoSource').':'.cfRGetVar('audioSource'),@$_POST['data4'],@$_POST['data5'],array('externalFilename'=>'webcam:'.cfRGetVar('videoSource').':'.cfRGetVar('audioSource'),'streaming'=>1));
		}
		else{

			VVOptions::setDesktopTopControlsExtra(resWebcamSourceDropDownMenus(true));

			vfvViewVideo('webcam:'.cfRGetVar('videoSource').':'.cfRGetVar('audioSource'),@$_POST['data4'],@$_POST['data5'],array('externalFilename'=>'webcam:'.cfRGetVar('videoSource').':'.cfRGetVar('audioSource'),'streaming'=>1));
		}
		exit;
	}


	/**
	 * Mobile still images view
	 */

	// Retreive available space (mobiles)
	if(isset($_POST['viewportWidth']) && is_numeric($_POST['viewportWidth']) && isset($_POST['viewportHeight']) && is_numeric($_POST['viewportHeight'])){
		cfRSetVar('userScreenWidth',$_POST['viewportWidth']);
		cfRSetVar('userScreenHeight',$_POST['viewportHeight']);


		list($w,$h)=webcamGetResolution(cfRGetVar('videoSource'));
		list($w,$h)=cfThumbnailDimensions($w,$h,$_POST['viewportWidth'],$_POST['viewportHeight']);

		cfRSetVar('flashOutputSize',$w);

		echo cfAsyncXMLJSaction('setImageSize('.$w.','.$h.');');

		echo cfAsyncXMLJSaction('setImage("index.php?getImage&rnd='.rand().'")');
	}
	if(cfIsAsync()) cfAsyncFooter();
}

/**
 * Refresh rate and resolution selection menus
 *
 * @param int $w
 * @param int $h
 * @return string
 */
function resWebcamDropDownMenus($w,$h){
	if(cfRGetVar('renderingLocked')) return;
	$out='';

	$frequencies=array(1,2,5,10,30,60,300,600,1800,3600);

	$items=array();
	$res=array();
	if($w) $r=$h/$w; else $r=0.75;

	/**
	 * Refresh rate
	 */
	for($i=0;$i<count($frequencies);$i++) $items[]=array(
		'text'=>$frequencies[$i].' '.cfCaption('genSeconds'),
		'action'=>'changeRefreshRate('.$frequencies[$i].')',
		'default'=>(cfRGetVar('refreshRate')==$frequencies[$i])
	);

	$out.='&nbsp;&nbsp;&nbsp;&nbsp;'.outDropDownMenu(cfCaption('genRefreshRate'),$items).'&nbsp;&nbsp;&nbsp;';

	/**
	 * Output resolution
	 */
	switch (cfRGetVar('flashOutputSizeMax')){
		case 'original':
			//echo '<option value="original" selected>'.cfCaption('videoCfgSource').(($sourceWidth && $videoHeight)?' ('.$sourceWidth.'x'.$videoHeight.')':'').'</option>';
		case 1280:
			if($w>=1280) $res[1280]=array('text'=>'1280x'.floor(1280*$r),'action'=>'changeWidth(1280)');
		case 900:
			if($w>=900) $res[900]=array('text'=>'900x'.floor(900*$r),'action'=>'changeWidth(900)');
		case 640:
			if($w>=640) $res[640]=array('text'=>'640x'.floor(640*$r),'action'=>'changeWidth(640)');
		case 400:
			if($w>=400) $res[400]=array('text'=>'400x'.floor(400*$r),'action'=>'changeWidth(400)');
		case 320:
			if($w>=320) $res[320]=array('text'=>'320x'.floor(320*$r),'action'=>'changeWidth(320)');
		default:
			if($w>=160) $res[160]=array('text'=>'160x'.floor(160*$r),'action'=>'changeWidth(160)');
	}
	// Set currently selected resolution
	if(isset($res[cfRGetVar('flashOutputSize')])) $res[cfRGetVar('flashOutputSize')]['default']=true;
	else if(isset($res[$w]) && cfRGetVar('flashOutputSize')=='original') $res[$w]['default']=true;

	$out.=outDropDownMenu(cfCaption('genSize'),$res);
	return $out;
}

/**
 * Audio/Video source selection menus
 *
 * @param bool $forVideoPage
 * @return string
 */
function resWebcamSourceDropDownMenus($forVideoPage=false){
	$out='';
	if(!cfRGetVar('sourceLocked')) {
		$list=webcamGetList();

		/**
		 * Video source
		 */
		if(count($list['video'])>1){
			$items=array();
			foreach ($list['video'] as $id=>$video) {
				$items[]=array(
						'text'=>$video['label'],
						'action'=>'changeVideoSource(\''.$id.'\')',
						'default'=>(cfRGetVar('videoSource')==$id)
				);
			}
			$out.=outDropDownMenu(outImageIcon('webcam').cfCaption('webcamSource'),$items);
		}

		/**
		 * Audio source
		 */
		if(count($list['audio'])>1){
			$items=array();
			foreach ($list['audio'] as $id=>$audio) {
				$items[]=array(
						'text'=>$audio['label'],
						'action'=>'changeAudioSource(\''.$id.'\')',
						'default'=>(cfRGetVar('audioSource')==$id)
				);
			}
			$out.='&nbsp;&nbsp;&nbsp;&nbsp;'.outDropDownMenu(outImageIcon('webcamMicro').cfCaption('webcamSource'),$items);
		}

		if($forVideoPage) $out.='&nbsp;&nbsp;&nbsp;&nbsp;';
	}
	if($out){
		$out.='<script type="text/javascript">';
		$out.="function changeVideoSource(id){fAS('','setVideoSource',id,0,'sync');}";
		$out.="function changeAudioSource(id){fAS('','setAudioSource',id,0,'sync');}";
		$out.='</script>';
	}
	return $out;
}

/**
 * @desc initialize resource
 *
 */
function resWebcamInit(){

	VVOptions::setNoRatio();
	VVOptions::setForbiddenPlayers(array('flashF4v', 'htlm5Ogv', 'qtReenc'));
	VVOptions::setWindowAutoAdapt();
	VVOptions::useDropDownMenuForPlayerSelection(true);

	// iOS devices: cannot play video so force to still images
	if(cfBGetVar('OS')=='iOS'){
		cfRSetVar('renderingLocked',true);
		cfRSetVar('stillImages',true);
	}

	if(!cfRGetVar('renderingLocked'))
		VVOptions::setExtraPlayers(array(array('stillImages','webcamImages16',cfCaption('genImages'))));

	if(cfRGetVar('resourceInitialized')) return ;

	cfRInitVar('flashOutputSizeMax','original');

	cfRSetVar('reencodedVideo',true);
	cfRSetVar('allowOriginalVideo',false);
	cfRSetVar('flvAllowed',true);
	cfRSetVar('webmAllowed',true);
	cfRSetVar('ogvAllowed',true);
	cfRSetVar('flashStream',true);
	cfRInitVar('flashOutputSize',min(640,cfRGetVar('flashOutputSizeMax')));

	cfRSetVar('videoBufferTime',0);

	cfRSetVar('extAccessFunction','require_once(INCLUDE_DIR."webcamFunctions.php");return webcamCheckAccessRights($completeFilename);');

	cfRSetVar('resourceInitialized',1);
}

/**
 * Generate and send a webcam image
 *
 */
function resWebcamGetImage(){
	$lastUpdate=cfRGetVar('lastUpdateTime');
	$refreshRate=1000*cfRGetVar('refreshRate');
	while(microtime(true)-$lastUpdate<$refreshRate) usleep(10000);
	webcamScreenshot(cfRGetVar('videoSource'),cfRGetVar('flashOutputSize'));
}

resWebcamInit();

if(!resWebcamFound()) resWebcamErrorPage();

// Generate still image if requested
if(isset($_GET['getImage'])) resWebcamGetImage();

/*
 ***************************************************************************************************************************
 * Process POST commands
 ***************************************************************************************************************************
 */

resWebcamProcessPOST();


/*
 ***************************************************************************************************************************
 * Mobiles
 ***************************************************************************************************************************
 */
if(cfIsMobile()) {require(dirname($_SERVER['SCRIPT_FILENAME']).'/mobileIndex.php');exit;}


/*
 ***************************************************************************************************************************
 * Insert HEAD, JAVASCRIPT functions and form
 ***************************************************************************************************************************
 */
cfInsertHEAD(false);

list($w,$h)=webcamGetResolution(cfRGetVar('videoSource'));
list($width,$height)=cfThumbnailDimensions($w,$h,cfRGetVar('flashOutputSize'));

// Reset window size so next async update reset image size & position
cfRSetVar('windowSizeSet',0);
?>
<script type="text/javascript">
var refreshRate=<?php echo (cfRGetVar('refreshRate'));?>,
resName="<?php echo addslashes(cfUTF8Encode(cfRGetVar('name')));?>",
width="<?php echo $width;?>",
height="<?php echo $height;?>",
mWidth="<?php echo $w;?>",
mHeight="<?php echo $h;?>",
lastUpdate=0,
imgIndex=0,
updating=0;

// Swith to movie mode
function setMovie(){
	fAS('','view','setVideoPlayer','video');
}
// Change still images refresh rate
function changeRefreshRate(s){
	fAS('','setRefreshRate',s,0,'async');
	refreshRate=s;
}

// Change output width
function changeWidth(nw){
	width=nw;
	//for(i=0;i<1;i++) {$('webcamImg'+i).style.width=width+'px';$('webcamImg'+i).style.height=height+'px'}
	fAS('','changeWidth',nw,0,'async');
}

// called on image load
function webcamImageLoaded(index){
	if(index!==undefined){
		$('webcamImg'+index).style.visibility="hidden";
		$('webcamImg'+index).style.zIndex=1;
		$('webcamImg'+(1-index)).style.zIndex=0;
		$('webcamImg'+index).e.fadeIn(300);
		$('webcamImg'+index).style.height='auto';
		index=1-index;
	}
	else index=0;
	imgIndex=index;
	updating=0;
}

function winResized(){
	alignBottom($('topDiv'),winMe);
}
function init(){
	webcamImageLoaded();
	winResized();
}
function updateMonitor(){
	if(wl.chrono.microtime() - lastUpdate > refreshRate*1000 && !updating){
		$('webcamImg'+imgIndex).src="/res/<?php echo cfRGetVar('type').'/'.cfRGetVar('subType').'/index.php?getImage=';?>"+Math.random();
		lastUpdate = wl.chrono.microtime();
		updating=1;
	}
}
function updateWindowSize(){
	winMe.resizeInnerTo()
}
var mi=setInterval(updateMonitor,50);
</script>
</head>
<?php
/**
 * BODY
 */
?>
<body onload="init()" style="overflow:hidden">
<?php outInsertStandardComForm(); ?>
<div id="topDiv2" style="height:100%">
<div style="position:absolute;top:0.3em;left:0.3em;z-index:2">
<?php
	echo resWebcamDropDownMenus($w,$h);
	echo '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
	echo resWebcamSourceDropDownMenus();
?>
</div>
<div style="position:absolute;top:0.3em;right:0.3em;z-index:2">
<?php echo outBt(false,outIcon('webcamVideo16'),'javascript:setMovie()'); ?>
</div>
<img style="position:absolute;top:2em;left:5%;width:90%;height:90%;z-index:0;background:black;display:block" id="webcamImg0" onload="webcamImageLoaded(0)">
<img style="position:absolute;top:2em;left:5%;width:90%;height:90%;z-index:1;background:black;display:block" id="webcamImg1" onload="webcamImageLoaded(1)">
</body></html>